//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/bit
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// file		bit.h
//-----------------------------------------------------------------------------
#ifndef __BIT_H__
#define __BIT_H__


//-----------------------------------------------------------------------------
// macro	bit_count_128
//
// purpose	Count the number of bits that are on in a 128 bit integer.
//
// arguments	(uint128_t) unsigned integer with bits to count
//
// returns	(int) <  0 : error
//		(int) >= 0 : number of bits that are on
//-----------------------------------------------------------------------------
#define bit_count_128_mask_1	UINT128_C( 0x55555555555555555555555555555555 )
#define bit_count_128_mask_2	UINT128_C( 0x33333333333333333333333333333333 )
#define bit_count_128_mask_3	UINT128_C( 0x07070707070707070707070707070707 )
#define bit_count_128_mask_4	UINT128_C( 0x000f000f000f000f000f000f000f000f )
#define bit_count_128_mask_5	UINT128_C( 0x0000001f0000001f0000001f0000001f )
#define bit_count_128_mask_6	UINT128_C( 0x000000000000003f000000000000003f )
#define bit_count_128_mask_7	( (int) 0x007f )
#define bit_count_128( b ) ({							\
	uint128_t d = ( b );							\
	d = ( d >>  1 & bit_count_128_mask_1 ) + ( d & bit_count_128_mask_1 );	\
	d = ( d >>  2 & bit_count_128_mask_2 ) + ( d & bit_count_128_mask_2 );	\
	d = ( d >>  4 & bit_count_128_mask_3 ) + ( d & bit_count_128_mask_3 );	\
	d = ( d >>  8 & bit_count_128_mask_4 ) + ( d & bit_count_128_mask_4 );	\
	d = ( d >> 16 & bit_count_128_mask_5 ) + ( d & bit_count_128_mask_5 );	\
	d = ( d >> 32 & bit_count_128_mask_6 ) + ( d & bit_count_128_mask_6 );	\
	( (int) ( d >> 64 ) & bit_count_128_mask_7 ) + ( (int) d & bit_count_128_mask_7 ); })

//-----------------------------------------------------------------------------
// macro	bit_count_16
//
// purpose	Count the number of bits that are on in a 16 bit integer.
//
// arguments	(uint16_t) unsigned integer with bits to count
//
// returns	(int) <  0 : error
//		(int) >= 0 : number of bits that are on
//-----------------------------------------------------------------------------
#define bit_count_16_mask_1	UINT16_C( 0x5555 )
#define bit_count_16_mask_2	UINT16_C( 0x3333 )
#define bit_count_16_mask_3	UINT16_C( 0x0707 )
#define bit_count_16_mask_4	( (int) 0x000f )
#define bit_count_16( b ) ({							\
	uint16_t d = ( b );							\
	d = ( d >>  1 & bit_count_16_mask_1 ) + ( d & bit_count_16_mask_1 );	\
	d = ( d >>  2 & bit_count_16_mask_2 ) + ( d & bit_count_16_mask_2 );	\
	d = ( d >>  4 & bit_count_16_mask_3 ) + ( d & bit_count_16_mask_3 );	\
	( (int) ( d >> 8 ) & bit_count_16_mask_4 ) + ( (int) d & bit_count_16_mask_4 ); })

//-----------------------------------------------------------------------------
// macro	bit_count_32
//
// purpose	Count the number of bits that are on in a 32 bit integer.
//
// arguments	(uint32_t) unsigned integer with bits to count
//
// returns	(int) <  0 : error
//		(int) >= 0 : number of bits that are on
//-----------------------------------------------------------------------------
#define bit_count_32_mask_1	UINT32_C( 0x55555555 )
#define bit_count_32_mask_2	UINT32_C( 0x33333333 )
#define bit_count_32_mask_3	UINT32_C( 0x07070707 )
#define bit_count_32_mask_4	UINT32_C( 0x000f000f )
#define bit_count_32_mask_5	( (int) 0x001f )
#define bit_count_32( b ) ({							\
	uint32_t d = ( b );							\
	d = ( d >>  1 & bit_count_32_mask_1 ) + ( d & bit_count_32_mask_1 );	\
	d = ( d >>  2 & bit_count_32_mask_2 ) + ( d & bit_count_32_mask_2 );	\
	d = ( d >>  4 & bit_count_32_mask_3 ) + ( d & bit_count_32_mask_3 );	\
	d = ( d >>  8 & bit_count_32_mask_4 ) + ( d & bit_count_32_mask_4 );	\
	( (int) ( d >> 16 ) & bit_count_32_mask_5 ) + ( (int) d & bit_count_32_mask_5 ); })

//-----------------------------------------------------------------------------
// macro	bit_count_64
//
// purpose	Count the number of bits that are on in a 64 bit integer.
//
// arguments	(uint64_t) unsigned integer with bits to count
//
// returns	(int) <  0 : error
//		(int) >= 0 : number of bits that are on
//-----------------------------------------------------------------------------
#define bit_count_64_mask_1	UINT64_C( 0x5555555555555555 )
#define bit_count_64_mask_2	UINT64_C( 0x3333333333333333 )
#define bit_count_64_mask_3	UINT64_C( 0x0707070707070707 )
#define bit_count_64_mask_4	UINT64_C( 0x000f000f000f000f )
#define bit_count_64_mask_5	UINT64_C( 0x0000001f0000001f )
#define bit_count_64_mask_6	( (int) 0x003f )
#define bit_count_64( b ) ({							\
	uint64_t d = ( b );							\
	d = ( d >>  1 & bit_count_64_mask_1 ) + ( d & bit_count_64_mask_1 );	\
	d = ( d >>  2 & bit_count_64_mask_2 ) + ( d & bit_count_64_mask_2 );	\
	d = ( d >>  4 & bit_count_64_mask_3 ) + ( d & bit_count_64_mask_3 );	\
	d = ( d >>  8 & bit_count_64_mask_4 ) + ( d & bit_count_64_mask_4 );	\
	d = ( d >> 16 & bit_count_64_mask_5 ) + ( d & bit_count_64_mask_5 );	\
	( (int) ( d >> 32 ) & bit_count_64_mask_6 ) + ( (int) d & bit_count_64_mask_6 ); })

//-----------------------------------------------------------------------------
// macro	bit_count_ui
//
// purpose	Count the number of bits that are set to 1 in an unsigned
//		int
//
// arguments	(unsigned int) unsigned integer with bits to count
//
// returns	(int) <  0 : error
//		(int) >= 0 : number of bits set to 1
//-----------------------------------------------------------------------------
#if UINT_MAX > 65535U
#if UINT_MAX > 4294967295U
#if UINT_MAX > 18446744073709551615U
#define bit_count_ui(b) bit_count_128((b))
#else
#define bit_count_ui(b) bit_count_64((b))
#endif
#else
#define bit_count_ui(b) bit_count_32((b))
#endif
#else
#define bit_count_ui(b) bit_count_16((b))
#endif

//-----------------------------------------------------------------------------
// macro	bit_count_ul
//
// purpose	Count the number of bits that are set to 1 in an unsigned
//		long
//
// arguments	(unsigned long) unsigned integer with bits to count
//
// returns	(int) <  0 : error
//		(int) >= 0 : number of bits set to 1
//-----------------------------------------------------------------------------
#if ULONG_MAX > 4294967295U
#if ULONG_MAX > 18446744073709551615U
#define bit_count_ul(b) bit_count_128((b))
#else
#define bit_count_ul(b) bit_count_64((b))
#endif
#else
#define bit_count_ul(b) bit_count_32((b))
#endif

//-----------------------------------------------------------------------------
// macro	bit_count_ull
//
// purpose	Count the number of bits that are set to 1 in an unsigned
//		long long
//
// arguments	(unsigned long long) unsigned integer with bits to count
//
// returns	(int) <  0 : error
//		(int) >= 0 : number of bits set to 1
//-----------------------------------------------------------------------------
#ifdef ULONGLONG_MAX
#if ULONGLONG_MAX > 18446744073709551615U
#define bit_count_ull(b) bit_count_128((b))
#else
#define bit_count_ull(b) bit_count_64((b))
#endif
#else
#define bit_count_ull(b) (0)
#endif

//-----------------------------------------------------------------------------
// function	bit_reverse_128
//
// purpose	Reverse all the bits in a 128 bit integer.
//
// arguments	(uint128_t) unsigned integer with bits to reverse
//
// returns	(uint128_t) unsigned integer with bits reversed
//-----------------------------------------------------------------------------
#define bit_reverse_128_mask_1	UINT128_C( 0x55555555555555555555555555555555 )
#define bit_reverse_128_mask_2	UINT128_C( 0x33333333333333333333333333333333 )
#define bit_reverse_128_mask_3	UINT128_C( 0x0f0f0f0f0f0f0f0f0f0f0f0f0f0f0f0f )
#define bit_reverse_128_mask_4	UINT128_C( 0x00ff00ff00ff00ff00ff00ff00ff00ff )
#define bit_reverse_128_mask_5	UINT128_C( 0x0000ffff0000ffff0000ffff0000ffff )
#define bit_reverse_128_mask_6	UINT128_C( 0x00000000ffffffff00000000ffffffff )
#define bit_reverse_128_mask_7	UINT128_C( 0x0000000000000000ffffffffffffffff )
#define bit_reverse_128( b ) ({									\
	uint128_t d = ( b );									\
	d = ( d >>  1 & bit_reverse_128_mask_1 ) | ( ( d & bit_reverse_128_mask_1 ) <<  1 );	\
	d = ( d >>  2 & bit_reverse_128_mask_2 ) | ( ( d & bit_reverse_128_mask_2 ) <<  2 );	\
	d = ( d >>  4 & bit_reverse_128_mask_3 ) | ( ( d & bit_reverse_128_mask_3 ) <<  4 );	\
	d = ( d >>  8 & bit_reverse_128_mask_4 ) | ( ( d & bit_reverse_128_mask_4 ) <<  8 );	\
	d = ( d >> 16 & bit_reverse_128_mask_5 ) | ( ( d & bit_reverse_128_mask_5 ) << 16 );	\
	d = ( d >> 32 & bit_reverse_128_mask_6 ) | ( ( d & bit_reverse_128_mask_6 ) << 32 );	\
	    ( d >> 64 & bit_reverse_128_mask_7 ) | ( ( d & bit_reverse_128_mask_7 ) << 64 ); })

//-----------------------------------------------------------------------------
// function	bit_reverse_16
//
// purpose	Reverse all the bits in a 16 bit integer.
//
// arguments	(uint16_t) unsigned integer with bits to reverse
//
// returns	(uint16_t) unsigned integer with bits reversed
//-----------------------------------------------------------------------------
#define bit_reverse_16_mask_1	UINT16_C( 0x5555 )
#define bit_reverse_16_mask_2	UINT16_C( 0x3333 )
#define bit_reverse_16_mask_3	UINT16_C( 0x0f0f )
#define bit_reverse_16_mask_4	UINT16_C( 0x00ff )
#define bit_reverse_16( b ) ({									\
	uint16_t d = ( b );									\
	d = ( d >>  1 & bit_reverse_16_mask_1 ) | ( ( d & bit_reverse_16_mask_1 ) <<  1 );	\
	d = ( d >>  2 & bit_reverse_16_mask_2 ) | ( ( d & bit_reverse_16_mask_2 ) <<  2 );	\
	d = ( d >>  4 & bit_reverse_16_mask_3 ) | ( ( d & bit_reverse_16_mask_3 ) <<  4 );	\
	    ( d >>  8 & bit_reverse_16_mask_4 ) | ( ( d & bit_reverse_16_mask_4 ) <<  8 ); })

//-----------------------------------------------------------------------------
// function	bit_reverse_32
//
// purpose	Reverse all the bits in a 32 bit integer.
//
// arguments	(uint32_t) unsigned integer with bits to reverse
//
// returns	(uint32_t) unsigned integer with bits reversed
//-----------------------------------------------------------------------------
#define bit_reverse_32_mask_1	UINT32_C( 0x55555555 )
#define bit_reverse_32_mask_2	UINT32_C( 0x33333333 )
#define bit_reverse_32_mask_3	UINT32_C( 0x0f0f0f0f )
#define bit_reverse_32_mask_4	UINT32_C( 0x00ff00ff )
#define bit_reverse_32_mask_5	UINT32_C( 0x0000ffff )
#define bit_reverse_32( b ) ({									\
	uint32_t d = ( b );									\
	d = ( d >>  1 & bit_reverse_32_mask_1 ) | ( ( d & bit_reverse_32_mask_1 ) <<  1 );	\
	d = ( d >>  2 & bit_reverse_32_mask_2 ) | ( ( d & bit_reverse_32_mask_2 ) <<  2 );	\
	d = ( d >>  4 & bit_reverse_32_mask_3 ) | ( ( d & bit_reverse_32_mask_3 ) <<  4 );	\
	d = ( d >>  8 & bit_reverse_32_mask_4 ) | ( ( d & bit_reverse_32_mask_4 ) <<  8 );	\
	    ( d >> 16 & bit_reverse_32_mask_5 ) | ( ( d & bit_reverse_32_mask_5 ) << 16 ); })

//-----------------------------------------------------------------------------
// function	bit_reverse_64
//
// purpose	Reverse all the bits in a 64 bit integer.
//
// arguments	(uint64_t) unsigned integer with bits to reverse
//
// returns	(uint64_t) unsigned integer with bits reversed
//-----------------------------------------------------------------------------
#define bit_reverse_64_mask_1	UINT64_C( 0x5555555555555555 )
#define bit_reverse_64_mask_2	UINT64_C( 0x3333333333333333 )
#define bit_reverse_64_mask_3	UINT64_C( 0x0f0f0f0f0f0f0f0f )
#define bit_reverse_64_mask_4	UINT64_C( 0x00ff00ff00ff00ff )
#define bit_reverse_64_mask_5	UINT64_C( 0x0000ffff0000ffff )
#define bit_reverse_64_mask_6	UINT64_C( 0x00000000ffffffff )
#define bit_reverse_64( b ) ({									\
	uint64_t d = ( b );									\
	d = ( d >>  1 & bit_reverse_64_mask_1 ) | ( ( d & bit_reverse_64_mask_1 ) <<  1 );	\
	d = ( d >>  2 & bit_reverse_64_mask_2 ) | ( ( d & bit_reverse_64_mask_2 ) <<  2 );	\
	d = ( d >>  4 & bit_reverse_64_mask_3 ) | ( ( d & bit_reverse_64_mask_3 ) <<  4 );	\
	d = ( d >>  8 & bit_reverse_64_mask_4 ) | ( ( d & bit_reverse_64_mask_4 ) <<  8 );	\
	d = ( d >> 16 & bit_reverse_64_mask_5 ) | ( ( d & bit_reverse_64_mask_5 ) << 16 );	\
	    ( d >> 32 & bit_reverse_64_mask_6 ) | ( ( d & bit_reverse_64_mask_6 ) << 32 ); })

//-----------------------------------------------------------------------------
// macro	bit_reverse_ui
//
// purpose	Reverse all the bits in an unsigned int.
//
// arguments	(unsigned int) unsigned integer with bits to reverse
//
// returns	(unsigned int) unsigned integer with bits reversed
//-----------------------------------------------------------------------------
#if UINT_MAX > 65535U
#if UINT_MAX > 4294967295U
#if UINT_MAX > 18446744073709551615U
#define bit_reverse_ui(b) bit_reverse_128((b))
#else
#define bit_reverse_ui(b) bit_reverse_64((b))
#endif
#else
#define bit_reverse_ui(b) bit_reverse_32((b))
#endif
#else
#define bit_reverse_ui(b) bit_reverse_16((b))
#endif

//-----------------------------------------------------------------------------
// macro	bit_reverse_ul
//
// purpose	Reverse all the bits in an unsigned long.
//
// arguments	(unsigned long) unsigned integer with bits to reverse
//
// returns	(unsigned long) unsigned integer with bits reversed
//-----------------------------------------------------------------------------
#if ULONG_MAX > 4294967295U
#if ULONG_MAX > 18446744073709551615U
#define bit_reverse_ul(b) bit_reverse_128((b))
#else
#define bit_reverse_ul(b) bit_reverse_64((b))
#endif
#else
#define bit_reverse_ul(b) bit_reverse_32((b))
#endif

//-----------------------------------------------------------------------------
// macro	bit_reverse_ull
//
// purpose	Reverse all the bits in an unsigned long long.
//
// arguments	(unsigned long long) unsigned integer with bits to reverse
//
// returns	(unsigned long long) unsigned integer with bits reversed
//-----------------------------------------------------------------------------
#ifdef ULONGLONG_MAX
#if ULONGLONG_MAX > 18446744073709551615U
#define bit_reverse_ull(b) bit_reverse_128((b))
#else
#define bit_reverse_ull(b) bit_reverse_64((b))
#endif
#else
#define bit_reverse_ull(b) (0)
#endif

//-----------------------------------------------------------------------------
// macro	low_bits_on
//
// purpose	Yield a constant expression that can be evaluated at compile
//		time to the value of a specified type with the specified number
//		of bits being 1, when the given value is a constant.
//
// arguments	1 (type) type to yield
//		2 (int) number of bits to be 1
//
// yields	(type) integer with specified number of bits being 1
//-----------------------------------------------------------------------------
#define low_bits_on(t,n) ( (t) (						\
	(n) <= ((t)0)								\
	? ((t)0)								\
	: ( ( ((t)1) << (n) ) | ( ( ((t)1) << (n) ) - ((t)1) ) )		\
) )

//-----------------------------------------------------------------------------
// macro	low_bits_on_u128
//
// purpose	Yield a constant expression that can be evaluated at compile
//		time to the uint128_t value with the specified number of bits
//		being 1, when the given value is a constant.
//
// arguments	1 (int) number of bits to be on
//
// yields	(uint128_t) value with specified number of bits on
//-----------------------------------------------------------------------------
#define low_bits_on_u128(n) ( (uint128_t) (						\
	(n) <= UINT128_C(0)								\
	? UINT128_C(0)									\
	: ( ( UINT128_C(1) << (n) ) | ( ( UINT128_C(1) << (n) ) - UINT128_C(1) ) )	\
) )

//-----------------------------------------------------------------------------
// macro	low_bits_on_u32
//
// purpose	Yield a constant expression that can be evaluated at compile
//		time to the uint32_t value with the specified number of bits
//		being 1, when the given value is a constant.
//
// arguments	1 (int) number of bits to be on
//
// yields	(uint32_t) value with specified number of bits on
//-----------------------------------------------------------------------------
#define low_bits_on_u32(n) ( (uint32_t) (					\
	(n) <= UINT32_C(0)							\
	? UINT32_C(0)								\
	: ( ( UINT32_C(1) << (n) ) | ( ( UINT32_C(1) << (n) ) - UINT32_C(1) ) )	\
) )

//-----------------------------------------------------------------------------
// macro	low_bits_on_u64
//
// purpose	Yield a constant expression that can be evaluated at compile
//		time to the uint64_t value with the specified number of bits
//		being 1, when the given value is a constant.
//
// arguments	1 (int) number of bits to be on
//
// yields	(uint64_t) value with specified number of bits on
//-----------------------------------------------------------------------------
#define low_bits_on_u64(n) ( (uint64_t) (					\
	(n) <= UINT64_C(0)							\
	? UINT64_C(0)								\
	: ( ( UINT64_C(1) << (n) ) | ( ( UINT64_C(1) << (n) ) - UINT64_C(1) ) )	\
) )

//-----------------------------------------------------------------------------
// macro	low_bits_on_ui
//
// purpose	Yield a constant expression that can be evaluated at compile
//		time to the unsigned int value with the specified number of
//		bits being 1, when the given value is a constant.
//
// arguments	1 (int) number of bits to be on
//
// yields	(unsigned int) value with specified number of bits on
//-----------------------------------------------------------------------------
#define low_bits_on_ui(n) ( (unsigned int) (					\
	(n) <= 0U								\
	? 0U									\
	: ( ( 1U << (n) ) | ( ( 1U << (n) ) - 1U ) )				\
) )

//-----------------------------------------------------------------------------
// macro	low_bits_on_ul
//
// purpose	Yield a constant expression that can be evaluated at compile
//		time to the unsigned long value with the specified number of
//		bits being 1, when the given value is a constant.
//
// arguments	1 (int) number of bits to be on
//
// yields	(unsigned long) value with specified number of bits on
//-----------------------------------------------------------------------------
#define low_bits_on_ul(n) ( (unsigned long) (					\
	(n) <= 0UL								\
	? 0UL									\
	: ( ( 1UL << (n) ) | ( ( 1UL << (n) ) - 1UL ) )				\
)

//-----------------------------------------------------------------------------
// macro	low_bits_on_ull
//
// purpose	Yield a constant expression that can be evaluated at compile
//		time to the unsigned long long value with the specified number
//		of bits being 1, when the given value is a constant.
//
// arguments	1 (int) number of bits to be on
//
// yields	(unsigned long long) value with specified number of bits on
//-----------------------------------------------------------------------------
#define low_bits_on_ull(n) ( (unsigned long long) (				\
	(n) <= 0ULL								\
	? 0ULL									\
	: ( ( 1ULL << (n) ) | ( ( 1ULL << (n) ) - 1ULL ) )			\
) )

#endif /* __BIT_H__ */
