//-----------------------------------------------------------------------------
// Copyright  2004 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/byte
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// file		byte.h
//-----------------------------------------------------------------------------
#ifndef __BYTE_H__
#define __BYTE_H__

#include <stdint.h>

//-----------------------------------------------------------------------------
// Define structures that unions a character array over various common types.
// Architectural byte order issues will apply here.
//-----------------------------------------------------------------------------
struct bytes_short {
    union {
	signed short		svalue				;
	unsigned short		uvalue				;
	unsigned char		bytes	[sizeof (short)]	;
    };
};
typedef struct bytes_short bytes_short;

struct bytes_int16 {
    union {
	int16_t			svalue				;
	uint16_t		uvalue				;
	unsigned char		bytes	[sizeof (int16_t)]	;
    };
};
typedef struct bytes_int bytes_int16;

struct bytes_int {
    union {
	signed int		svalue				;
	unsigned int		uvalue				;
	unsigned char		bytes	[sizeof (int)]		;
    };
};
typedef struct bytes_int bytes_int;

struct bytes_int32 {
    union {
	int32_t			svalue				;
	uint32_t		uvalue				;
	unsigned char		bytes	[sizeof (int32_t)]	;
    };
};
typedef struct bytes_int bytes_int32;

struct bytes_long {
    union {
	signed long		svalue				;
	unsigned long		uvalue				;
	unsigned char		bytes	[sizeof (long)]		;
    };
};
typedef struct bytes_long bytes_long;

struct bytes_int64 {
    union {
	int64_t			svalue				;
	uint64_t		uvalue				;
	unsigned char		bytes	[sizeof (int64_t)]	;
    };
};
typedef struct bytes_int bytes_int64;

struct bytes_long_long {
    union {
	signed long long	svalue				;
	unsigned long long	uvalue				;
	unsigned char		bytes	[sizeof (long long)]	;
    };
};
typedef struct bytes_long_long bytes_long_long;

struct bytes_float {
    union {
	float			value				;
	unsigned char		bytes	[sizeof (float)]	;
    };
};
typedef struct bytes_float bytes_float;

struct bytes_double {
    union {
	double			value				;
	unsigned char		bytes	[sizeof (double)]	;
    };
};
typedef struct bytes_double bytes_double;

struct bytes_long_double {
    union {
	long double		value				;
	unsigned char		bytes	[sizeof (long double)]	;
    };
};
typedef struct bytes_long_double bytes_long_double;

struct bytes_pointer {
    union {
	void *			value				;
	intptr_t		svalue				;
	uintptr_t		uvalue				;
	unsigned char		bytes	[sizeof (void *)]	;
    };
};
typedef struct bytes_pointer bytes_pointer;

//-----------------------------------------------------------------------------
// macro	int_to_2_bytes_be
//
// purpose	Store a given integer value into 2 bytes in big endian order.
//
// arguments	1 (integer) value to store
//		2 (unsigned char *) pointer to bytes to store into
//
// yields	(void)
//-----------------------------------------------------------------------------
#define int_to_2_bytes_be(n,p) ({						\
	int16_t libh__n;							\
	unsigned char *libh__p;							\
	libh__n = (n);								\
	libh__p = (p) + 1;							\
	* libh__p = libh__n & 0x00ff;						\
	libh__n >>= 8;								\
	* -- libh__p = libh__n & 0x00ff;					\
})

//-----------------------------------------------------------------------------
// macro	int_to_2_bytes_le
//
// purpose	Store a given integer value into 2 bytes in little endian order.
//
// arguments	1 (integer) value to store
//		2 (unsigned char *) pointer to bytes to store into
//
// yields	(void)
//-----------------------------------------------------------------------------
#define int_to_2_bytes_le(n,p) ({						\
	int16_t libh__n;							\
	unsigned char *libh__p;							\
	libh__n = (n);								\
	libh__p = (p);								\
	* libh__p ++ = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* libh__p = libh__n & 0x00ff;						\
})

//-----------------------------------------------------------------------------
// macro	int_to_4_bytes_be
//
// purpose	Store a given integer value into 4 bytes in big endian order.
//
// arguments	1 (integer) value to store
//		2 (unsigned char *) pointer to bytes to store into
//
// yields	(void)
//-----------------------------------------------------------------------------
#define int_to_4_bytes_be(n,p) ({						\
	int32_t libh__n;							\
	unsigned char *libh__p;							\
	libh__n = (n);								\
	libh__p = (p) + 3;							\
	* libh__p = libh__n & 0x00ff;						\
	libh__n >>= 8;								\
	* -- libh__p = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* -- libh__p = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* -- libh__p = libh__n & 0x00ff;					\
})

//-----------------------------------------------------------------------------
// macro	int_to_4_bytes_le
//
// purpose	Store a given integer value into 4 bytes in little endian order.
//
// arguments	1 (integer) value to store
//		2 (unsigned char *) pointer to bytes to store into
//
// yields	(void)
//-----------------------------------------------------------------------------
#define int_to_4_bytes_le(n,p) ({						\
	int32_t libh__n;							\
	unsigned char *libh__p;							\
	libh__n = (n);								\
	libh__p = (p);								\
	* libh__p ++ = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* libh__p ++ = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* libh__p ++ = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* libh__p = libh__n & 0x00ff;						\
})

//-----------------------------------------------------------------------------
// macro	int_to_8_bytes_be
//
// purpose	Store a given integer value into 8 bytes in big endian order.
//
// arguments	1 (integer) value to store
//		2 (unsigned char *) pointer to bytes to store into
//
// yields	(void)
//-----------------------------------------------------------------------------
#define int_to_8_bytes_be(n,p) ({						\
	int64_t libh__n;							\
	unsigned char *libh__p;							\
	libh__n = (n);								\
	libh__p = (p) + 7;							\
	* libh__p = libh__n & 0x00ff;						\
	libh__n >>= 8;								\
	* -- libh__p = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* -- libh__p = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* -- libh__p = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* -- libh__p = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* -- libh__p = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* -- libh__p = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* -- libh__p = libh__n & 0x00ff;					\
})

//-----------------------------------------------------------------------------
// macro	int_to_8_bytes_le
//
// purpose	Store a given integer value into 8 bytes in little endian order.
//
// arguments	1 (integer) value to store
//		2 (unsigned char *) pointer to bytes to store into
//
// yields	(void)
//-----------------------------------------------------------------------------
#define int_to_8_bytes_le(n,p) ({						\
	int64_t libh__n;							\
	unsigned char *libh__p;							\
	libh__n = (n);								\
	libh__p = (p);								\
	* libh__p ++ = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* libh__p ++ = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* libh__p ++ = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* libh__p ++ = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* libh__p ++ = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* libh__p ++ = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* libh__p ++ = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* libh__p = libh__n & 0x00ff;						\
})

//-----------------------------------------------------------------------------
// macro	int_to_bytes_be
//
// purpose	Store a given integer value into a specified number of bytes
//		in big endian order.
//
// arguments	1 (integer) value to store
//		2 (int) number of bytes to store into
//		3 (unsigned char *) pointer to bytes to store into
//
// yields	(void)
//-----------------------------------------------------------------------------
#define int_to_bytes_be(i,n,p) ({						\
	__type_of__((i)) libh__i;						\
	unsigned char *libh__p;							\
	unsigned char *libh__q;							\
	libh__i = (i);								\
	libh__p = (p);								\
	libh__q = libh__p + (n);						\
	while ( libh__q > libh__p ) {						\
		* -- libh__q = libh__i & 0x00ff;				\
		libh__i >>= 8;							\
	}									\
})

//-----------------------------------------------------------------------------
// macro	int_to_bytes_le
//
// purpose	Store a given integer value into a specified number of bytes
//		in little endian order.
//
// arguments	1 (integer) value to store
//		2 (int) number of bytes to store into
//		3 (unsigned char *) pointer to bytes to store into
//
// yields	(void)
//-----------------------------------------------------------------------------
#define int_to_bytes_le(i,n,p) ({						\
	__type_of__((i)) libh__i;						\
	unsigned char *libh__p;							\
	unsigned char *libh__q;							\
	libh__i = (i);								\
	libh__p = (p);								\
	libh__q = libh__p + (n);						\
	while ( libh__p < libh__q ) {						\
		* libh__p ++ = libh__i & 0x00ff;				\
		libh__i >>= 8;							\
	}									\
})

//-----------------------------------------------------------------------------
// macro	uint_to_2_bytes_be
//
// purpose	Store a given integer value into 2 bytes in big endian order.
//
// arguments	1 (integer) value to store
//		2 (unsigned char *) pointer to bytes to store into
//
// yields	(void)
//-----------------------------------------------------------------------------
#define uint_to_2_bytes_be(n,p) ({						\
	uint16_t libh__n;							\
	unsigned char *libh__p;							\
	libh__n = (n);								\
	libh__p = (p) + 1;							\
	* libh__p = libh__n & 0x00ff;						\
	libh__n >>= 8;								\
	* -- libh__p = libh__n & 0x00ff;					\
})

//-----------------------------------------------------------------------------
// macro	uint_to_2_bytes_le
//
// purpose	Store a given integer value into 2 bytes in little endian order.
//
// arguments	1 (integer) value to store
//		2 (unsigned char *) pointer to bytes to store into
//
// yields	(void)
//-----------------------------------------------------------------------------
#define uint_to_2_bytes_le(n,p) ({						\
	uint16_t libh__n;							\
	unsigned char *libh__p;							\
	libh__n = (n);								\
	libh__p = (p);								\
	* libh__p ++ = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* libh__p = libh__n & 0x00ff;						\
})

//-----------------------------------------------------------------------------
// macro	uint_to_4_bytes_be
//
// purpose	Store a given integer value into 4 bytes in big endian order.
//
// arguments	1 (integer) value to store
//		2 (unsigned char *) pointer to bytes to store into
//
// yields	(void)
//-----------------------------------------------------------------------------
#define uint_to_4_bytes_be(n,p) ({						\
	uint32_t libh__n;							\
	unsigned char *libh__p;							\
	libh__n = (n);								\
	libh__p = (p) + 3;							\
	* libh__p = libh__n & 0x00ff;						\
	libh__n >>= 8;								\
	* -- libh__p = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* -- libh__p = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* -- libh__p = libh__n & 0x00ff;					\
})

//-----------------------------------------------------------------------------
// macro	uint_to_4_bytes_le
//
// purpose	Store a given integer value into 4 bytes in little endian order.
//
// arguments	1 (integer) value to store
//		2 (unsigned char *) pointer to bytes to store into
//
// yields	(void)
//-----------------------------------------------------------------------------
#define uint_to_4_bytes_le(n,p) ({						\
	uint32_t libh__n;							\
	unsigned char *libh__p;							\
	libh__n = (n);								\
	libh__p = (p);								\
	* libh__p ++ = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* libh__p ++ = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* libh__p ++ = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* libh__p = libh__n & 0x00ff;						\
})

//-----------------------------------------------------------------------------
// macro	uint_to_8_bytes_be
//
// purpose	Store a given integer value into 8 bytes in big endian order.
//
// arguments	1 (integer) value to store
//		2 (unsigned char *) pointer to bytes to store into
//
// yields	(void)
//-----------------------------------------------------------------------------
#define uint_to_8_bytes_be(n,p) ({						\
	uint64_t libh__n;							\
	unsigned char *libh__p;							\
	libh__n = (n);								\
	libh__p = (p) + 7;							\
	* libh__p = libh__n & 0x00ff;						\
	libh__n >>= 8;								\
	* -- libh__p = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* -- libh__p = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* -- libh__p = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* -- libh__p = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* -- libh__p = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* -- libh__p = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* -- libh__p = libh__n & 0x00ff;					\
})

//-----------------------------------------------------------------------------
// macro	uint_to_8_bytes_le
//
// purpose	Store a given integer value into 8 bytes in little endian order.
//
// arguments	1 (integer) value to store
//		2 (unsigned char *) pointer to bytes to store into
//
// yields	(void)
//-----------------------------------------------------------------------------
#define uint_to_8_bytes_le(n,p) ({						\
	uint64_t libh__n;							\
	unsigned char *libh__p;							\
	libh__n = (n);								\
	libh__p = (p);								\
	* libh__p ++ = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* libh__p ++ = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* libh__p ++ = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* libh__p ++ = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* libh__p ++ = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* libh__p ++ = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* libh__p ++ = libh__n & 0x00ff;					\
	libh__n >>= 8;								\
	* libh__p = libh__n & 0x00ff;						\
})

//-----------------------------------------------------------------------------
// macro	uint_to_bytes_be
//
// purpose	Store a given integer value into a specified number of bytes
//		in big endian order.
//
// arguments	1 (integer) value to store
//		2 (int) number of bytes to store into
//		3 (unsigned char *) pointer to bytes to store into
//
// yields	(void)
//-----------------------------------------------------------------------------
#define uint_to_bytes_be(i,n,p) ({						\
	__type_of__((i)) libh__i;						\
	unsigned char *libh__p;							\
	unsigned char *libh__q;							\
	libh__i = (i);								\
	libh__p = (p);								\
	libh__q = libh__p + (n);						\
	while ( libh__q > libh__p ) {						\
		* -- libh__q = libh__i & 0x00ff;				\
		libh__i >>= 8;							\
	}									\
})

//-----------------------------------------------------------------------------
// macro	uint_to_bytes_le
//
// purpose	Store a given integer value into a specified number of bytes
//		in little endian order.
//
// arguments	1 (integer) value to store
//		2 (int) number of bytes to store into
//		3 (unsigned char *) pointer to bytes to store into
//
// yields	(void)
//-----------------------------------------------------------------------------
#define uint_to_bytes_le(i,n,p) ({						\
	__type_of__((i)) libh__i;						\
	unsigned char *libh__p;							\
	unsigned char *libh__q;							\
	libh__i = (i);								\
	libh__p = (p);								\
	libh__q = libh__p + (n);						\
	while ( libh__p < libh__q ) {						\
		* libh__p ++ = libh__i & 0x00ff;				\
		libh__i >>= 8;							\
	}									\
})

#endif /* __BYTE_H__ */
