//-----------------------------------------------------------------------------
// Copyright  2004 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/div
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// file		libh/div.h
//-----------------------------------------------------------------------------
#ifndef __DIV_H__
#define __DIV_H__

//-----------------------------------------------------------------------------
// macro	divq
//
// purpose	Calculate the quotient and remainder (e.g. x/y and x%y) at the
//		same time via the div() standard function, but instead of
//		returning a div_t structure, return the quotient and store the
//		remainder in the location specified in the 3rd argument.
//
// arguments	1 (int) divident
//		2 (int) divisor
//		3 (integer *) where to store remainder
//
// returns	(int) quotient
//-----------------------------------------------------------------------------
#define divq(x,y,r) ({								\
	div_t libh__dt;								\
	__typeof__( (r) ) libh__r;						\
	libh__dt = div( (x), (y) );						\
	libh__r = (r);								\
	if ( libh__r ) * libh__r = libh__dt.rem;				\
	libh__dt.quot;								\
})

//-----------------------------------------------------------------------------
// macro	divqr
//
// purpose	Calculate the quotient and remainder (e.g. x/y and x%y) at the
//		same time via the div() standard function, but instead of
//		returning a div_t structure, store the quotient in the location
//		specified in the 3rd argument and store the remainder in the
//		location specified in the 4th argument.
//
// arguments	1 (int) divident
//		2 (int) divisor
//		3 (integer *) where to store quotient
//		4 (integer *) where to store remainder
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define divqr(x,y,q,r) ({							\
	div_t libh__dt;								\
	__typeof__( (q) ) libh__q;						\
	__typeof__( (r) ) libh__r;						\
	libh__dt = div( (x), (y) );						\
	libh__q = (q);								\
	if ( libh__q ) * libh__q = libh__dt.quot;				\
	libh__r = (r);								\
	if ( libh__r ) * libh__r = libh__dt.rem;				\
	0;									\
})

//-----------------------------------------------------------------------------
// macro	divr
//
// purpose	Calculate the quotient and remainder (e.g. x/y and x%y) at the
//		same time via the div() standard function, but instead of
//		returning a div_t structure, return the remainder and store the
//		quotient in the location specified in the 3rd argument.
//
// arguments	1 (int) divident
//		2 (int) divisor
//		3 (integer *) where to store quotient
//
// returns	(int) remainder
//-----------------------------------------------------------------------------
#define divr(x,y,q) ({								\
	div_t libh__dt;								\
	__typeof__( (q) ) libh__q;						\
	libh__dt = div( (x), (y) );						\
	libh__q = (q);								\
	if ( libh__q ) * libh__q = libh__dt.quot;				\
	libh__dt.rem;								\
})

//-----------------------------------------------------------------------------
// macro	ldivq
//
// purpose	Calculate the quotient and remainder (e.g. x/y and x%y) at the
//		same time via the ldiv() standard function, but instead of
//		returning a ldiv_t structure, return the quotient and store the
//		remainder in the location specified in the 3rd argument.
//
// arguments	1 (long) divident
//		2 (long) divisor
//		3 (integer *) where to store remainder
//
// returns	(long) quotient
//-----------------------------------------------------------------------------
#define ldivq(x,y,r) ({								\
	ldiv_t libh__dt;							\
	__typeof__( (r) ) libh__r;						\
	libh__dt = ldiv( (x), (y) );						\
	libh__r = (r);								\
	if ( libh__r ) * libh__r = libh__dt.rem;				\
	libh__dt.quot;								\
})

//-----------------------------------------------------------------------------
// macro	ldivqr
//
// purpose	Calculate the quotient and remainder (e.g. x/y and x%y) at the
//		same time via the ldiv() standard function, but instead of
//		returning a ldiv_t structure, store the quotient in the location
//		specified in the 3rd argument and store the remainder in the
//		location specified in the 4th argument.
//
// arguments	1 (long) divident
//		2 (long) divisor
//		3 (integer *) where to store quotient
//		4 (integer *) where to store remainder
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define ldivqr(x,y,q,r) ({							\
	ldiv_t libh__dt;							\
	__typeof__( (q) ) libh__q;						\
	__typeof__( (r) ) libh__r;						\
	libh__dt = ldiv( (x), (y) );						\
	libh__q = (q);								\
	if ( libh__q ) * libh__q = libh__dt.quot;				\
	libh__r = (r);								\
	if ( libh__r ) * libh__r = libh__dt.rem;				\
	0;									\
})

//-----------------------------------------------------------------------------
// macro	ldivr
//
// purpose	Calculate the quotient and remainder (e.g. x/y and x%y) at the
//		same time via the ldiv() standard function, but instead of
//		returning a ldiv_t structure, return the remainder and store the
//		quotient in the location specified in the 3rd argument.
//
// arguments	1 (long) divident
//		2 (long) divisor
//		3 (integer *) where to store quotient
//
// returns	(long) remainder
//-----------------------------------------------------------------------------
#define ldivr(x,y,q) ({								\
	ldiv_t libh__dt;							\
	__typeof__( (q) ) libh__q;						\
	libh__dt = ldiv( (x), (y) );						\
	libh__q = (q);								\
	if ( libh__q ) * libh__q = libh__dt.quot;				\
	libh__dt.rem;								\
})

//-----------------------------------------------------------------------------
// macro	lldivq
//
// purpose	Calculate the quotient and remainder (e.g. x/y and x%y) at the
//		same time via the lldiv() standard function, but instead of
//		returning a lldiv_t structure, return the quotient and store the
//		remainder in the location specified in the 3rd argument.
//
// arguments	1 (long long) divident
//		2 (long long) divisor
//		3 (integer *) where to store remainder
//
// returns	(long long) quotient
//-----------------------------------------------------------------------------
#define lldivq(x,y,r) ({							\
	lldiv_t libh__dt;							\
	__typeof__( (r) ) libh__r;						\
	libh__dt = lldiv( (x), (y) );						\
	libh__r = (r);								\
	if ( libh__r ) * libh__r = libh__dt.rem;				\
	libh__dt.quot;								\
})

//-----------------------------------------------------------------------------
// macro	lldivqr
//
// purpose	Calculate the quotient and remainder (e.g. x/y and x%y) at the
//		same time via the lldiv() standard function, but instead of
//		returning a lldiv_t structure, store the quotient in the location
//		specified in the 3rd argument and store the remainder in the
//		location specified in the 4th argument.
//
// arguments	1 (long long) divident
//		2 (long long) divisor
//		3 (integer *) where to store quotient
//		4 (integer *) where to store remainder
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define lldivqr(x,y,q,r) ({							\
	lldiv_t libh__dt;							\
	__typeof__( (q) ) libh__q;						\
	__typeof__( (r) ) libh__r;						\
	libh__dt = lldiv( (x), (y) );						\
	libh__q = (q);								\
	if ( libh__q ) * libh__q = libh__dt.quot;				\
	libh__r = (r);								\
	if ( libh__r ) * libh__r = libh__dt.rem;				\
	0;									\
})

//-----------------------------------------------------------------------------
// macro	lldivr
//
// purpose	Calculate the quotient and remainder (e.g. x/y and x%y) at the
//		same time via the lldiv() standard function, but instead of
//		returning a lldiv_t structure, return the remainder and store the
//		quotient in the location specified in the 3rd argument.
//
// arguments	1 (long long) divident
//		2 (long long) divisor
//		3 (integer *) where to store quotient
//
// returns	(long long) remainder
//-----------------------------------------------------------------------------
#define lldivr(x,y,q) ({							\
	lldiv_t libh__dt;							\
	__typeof__( (q) ) libh__q;						\
	libh__dt = lldiv( (x), (y) );						\
	libh__q = (q);								\
	if ( libh__q ) * libh__q = libh__dt.quot;				\
	libh__dt.rem;								\
})

//-----------------------------------------------------------------------------
// macro	mod_ni
//
// purpose	Calculate the integer modulus result of division truncated
//		towards negative infinity.  This operation would be used for
//		situations involving repeating periods that are not inverted
//		in the negative dividend range, such as calculations for date
//		and time purposes with arbitrary epochs.
//
// note		Standard C 1999 defines the / and % operators to calculate the
//		integer quotient result of division truncated towards zero.
//
// example	for the following values divided by 4:
//
// dividend	-9 -8 -7 -6 -5 -4 -3 -2 -1  0  1  2  3  4  5  6  7  8  9
//
//		the following is the result of division towards zero
// quotient	-2 -2 -1 -1 -1 -1  0  0  0  0  0  0  0  1  1  1  1  2  2
// remainder	-1  0 -3 -2 -1  0 -3 -2 -1  0  1  2  3  0  1  2  3  0  1
//
//		the following is the result of division towards -infinity
// quotient	-3 -2 -2 -2 -2 -1 -1 -1 -1  0  0  0  0  1  1  1  1  2  2
// modulus	 3  0  1  2  3  0  1  2  3  0  1  2  3  0  1  2  3  0  1
//
// arguments	1 (integer) dividend
//		2 (integer) divisor
//
// returns	(integer) modulus
//
// note		This macro assumes compliance with C99 / and % operators.
//		Implementations of C that are not in compliance with C99 in
//		this regard may produce incorrect results.
//
// note		Do not use this macro with non-integer argument types.
//		Results are undefined for anything but integers.
//-----------------------------------------------------------------------------
#define mod_ni(x,y) ({								\
	__typeof__( (x) ) libh__x;						\
	__typeof__( (y) ) libh__y;						\
	libh__x = (x) % ( libh__y = (y) );					\
	libh__x < 0								\
		? libh__x + ( libh__y < 0 ? - libh__y : libh__y )		\
		: libh__x;							\
})
//-----------------------------------------------------------------------------
// macro	quo_ni
//
// purpose	Calculate the integer quotient result of division truncated
//		towards negative infinity.  This operation would be used for
//		situations involving repeating periods that are not inverted
//		in the negative dividend range, such as calculations for date
//		and time purposes with arbitrary epochs.
//
// note		Standard C 1999 defines the / and % operators to calculate the
//		integer quotient result of division truncated towards zero.
//
// example	for the following values divided by 4:
//
// dividend	-9 -8 -7 -6 -5 -4 -3 -2 -1  0  1  2  3  4  5  6  7  8  9
//
//		the following is the result of division towards zero
// quotient	-2 -2 -1 -1 -1 -1  0  0  0  0  0  0  0  1  1  1  1  2  2
// remainder	-1  0 -3 -2 -1  0 -3 -2 -1  0  1  2  3  0  1  2  3  0  1
//
//		the following is the result of division towards -infinity
// quotient	-3 -2 -2 -2 -2 -1 -1 -1 -1  0  0  0  0  1  1  1  1  2  2
// modulus	 3  0  1  2  3  0  1  2  3  0  1  2  3  0  1  2  3  0  1
//
// arguments	1 (integer) dividend
//		2 (integer) divisor
//
// returns	(integer) quotient
//
// note		This macro assumes compliance with C99 / and % operators.
//		Implementations of C that are not in compliance with C99 in
//		this regard may produce incorrect results.
//
// note		Do not use this macro with non-integer argument types.
//		Results are undefined for anything but integers.
//-----------------------------------------------------------------------------
#define quo_ni(x,y) ({								\
	__typeof__( (x) ) libh__x;						\
	__typeof__( (y) ) libh__y;						\
	libh__x = (x);								\
	libh__y = (y);								\
	libh__x >= 0								\
		? libh__x / libh__y						\
		: ( libh__x + 1 ) / libh__y + ( libh__y >= 0 ? - 1 : 1 );	\
})

#endif /* __DIV_H__ */
