//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/option
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// file		libh/option.h
//-----------------------------------------------------------------------------
#ifndef __OPTION_H__
#define __OPTION_H__
//-----------------------------------------------------------------------------
// function	option_flag
//
// purpose	Check if an option matches, and if so return true, else false.
//
// arguments	1 (int *) pointer to number of options
//		2 (char * * *) pointer to argv for options
//		3..N (const char *) pointers to possible options
//
// returns	(int) 0 : option did not match
//		(int) 1 : option did match
//-----------------------------------------------------------------------------
int
option_flag (
    int *		arg_argc_ptr
    ,
    char * * *		arg_argv_ptr
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	option_string
//
// purpose	Check if an option matches, and if so, store a pointer to
//		the string value for that option.
//
// arguments	1 (int *) pointer to number of options      (as in: & argc)
//		2 (char * * *) pointer to argv for options  (as in: & argv)
//		3 (char * *) pointer where to store result pointer
//		4..N (const char *) pointers to possible options
//
// returns	(int) 0 : option did not match, pointers unchanged
//		(int) 1 : option did match, result pointer stored, and if
//			a 2nd arguments is used, argc and argv are updated
//-----------------------------------------------------------------------------
int
option_string (
    int *		arg_argc_ptr
    ,
    char * * *		arg_argv_ptr
    ,
    char * *		arg_result_ptr
    ,
    ...
    )
;

#endif /* __OPTION_H__ */
