//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/trace
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// header	trace_off.h
//
// purpose	Define symbols to disable compilation of trace actions.
//		If TRACE_ON is defined, trace will be enabled.
//
// note		The name trace is declared as a function and may also be
//		declared as a macro at the same time.
//-----------------------------------------------------------------------------

#include <stdio.h>

#ifndef TRACE_DECLARED
#define TRACE_DECLARED
int (trace)(int,const char *,...);
int (trace_set_file)(FILE *);
int (trace_set_level)(int);
#endif

#ifndef TRACE_OFF

#define trace_signed(x,v)		(trace)((x),#v " = %ld",(signed long)(v))
#define trace_unsigned(x,v)		(trace)((x),#v " = %lu",(unsigned long)(v))
#define trace_pointer(x,v)		(trace)((x),#v " = %p",(void*)(v))
#define trace_string(x,v)		(trace)((x),#v " = \"%s\"",(char*)(v))
#define trace_cetal(x,v)		(trace)((x),#v " = 0x%08lx",(unsigned long)(v))

#else

#define trace(...)

#define trace_set_file(x)
#define trace_set_level(x)

#define trace_signed(x,v)
#define trace_unsigned(x,v)
#define trace_pointer(x,v)
#define trace_string(x,v)
#define trace_cetal(x,v)

#endif
