//-----------------------------------------------------------------------------
// Copyright  2005 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/string
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// file		libh/string.h
//
// purpose      Define resources used by the string library.
//-----------------------------------------------------------------------------
#ifndef __STRING_H__
#define __STRING_H__
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>

//-----------------------------------------------------------------------------
// struct	split_s
//
// purpose	Keep the current state of a string splitting sequence.
//-----------------------------------------------------------------------------
struct split_s {
    int			flags					;
    int ( *		call	)( struct split_s * )		;
    char *		data_ptr				;
    char *		data_end				;
    char *		part_ptr				;
    char *		part_end				;
    char *		next_ptr				;
    union {
	int		    ch					;
	int ( *		    ch_call 	)( int )		;
	ssize_t ( *	    call 	)( char *, size_t )	;
	struct {
	    char *		ptr				;
	    char *		end				;
	}		    data				;
    }			sep					;
};

#define SPLIT_OBJ_ALLOCATED	0x0001
#define SPLIT_STR_ALLOCATED	0x0002

#define SPLIT_SEP_REFERENCE	0x0004
#define SPLIT_SEP_ALLOCATED	0x0008
#define SPLIT_SEP_REF_ALLOC	((SPLIT_SEP_REFERENCE)|(SPLIT_SEP_ALLOCATED))

#define SPLIT_SEP_INVERT	0x0100
#define SPLIT_SEP_NOCASE	0x0200
#define SPLIT_SEP_TRIMSPACES	0x0400

typedef struct split_s		split_s			;
typedef struct split_s		split_t		[1]	;
typedef struct split_s *	split_p			;

//-----------------------------------------------------------------------------
// macro	STR_EQ_NOCASE
// purpose	Specify the option flag to disregard case for alphabetics.
//
// macro	STR_EQ_SPACE
// purpose	Specify the option flag to treat whitespace clusters as one.
//
// macro	STR_EQ_NUMERIC
// purpose	Specify the option flag to handle digits like numeric
//-----------------------------------------------------------------------------
#define STR_EQ_NOCASE	1
#define STR_EQ_SPACE	2
#define STR_EQ_NUMERIC	4

//-----------------------------------------------------------------------------
// function alias macros
//-----------------------------------------------------------------------------
#define str_to_i	strtol
#define str_to_si	strtol
#define str_to_ui	strtoul
#define str_to_l	strtol
#define str_to_sl	strtol
#define str_to_ul	strtoul

#define str_to_d	strtod

//-----------------------------------------------------------------------------
// In glibc (from 2.1 to 2.3.1) strtold() is majorly broken so leave the
// internal version of str_to_ld in use.  Enable this define only when
// a version of glibc with a working strtold() is used.
//-----------------------------------------------------------------------------
// #define str_to_ld	strtold

//-----------------------------------------------------------------------------
// Define these according to the environment.
//-----------------------------------------------------------------------------
#if defined(HAVE_C99) && HAVE_C99==1

#define str_to_ll	strtoll
#define str_to_ull	strtoull

#else

#define str_to_ll	strtoq
#define str_to_ull	strtouq

#endif

//-----------------------------------------------------------------------------
// struct	strpe_s
// typedef	strpe_t, strpe_p
//
// purpose	Define a struct containing a string pointer and end pointer.
//-----------------------------------------------------------------------------
struct strpe_s {
    char *	ptr				;
    char *	end				;
};
typedef struct strpe_s		strpe_t		;
typedef struct strpe_s *	strpe_p		;

//-----------------------------------------------------------------------------
// struct       strpea_s
// typedef      strpea_t, strpea_p
//
// purpose      Define a value containing a string pointer, end pointer, and
//		allocation end pointer.
//-----------------------------------------------------------------------------
struct strpea_s {
    char *	ptr				;
    char *	end				;
    char *	aend				;
};
typedef struct strpea_s		strpea_t	;
typedef struct strpea_s *	strpea_p	;

//-----------------------------------------------------------------------------
// struct	strpl_s
// typedef	strpl_t, strpl_p
//
// purpose	Define a struct containing a string pointer and length.
//-----------------------------------------------------------------------------
struct strpl_s {
    char *	ptr				;
    size_t	len				;
};
typedef struct strpl_s		strpl_t		;
typedef struct strpl_s *	strpl_p		;

//-----------------------------------------------------------------------------
// struct       strpla_s
// typedef      strpla_t, strpla_p
//
// purpose      Define a value containing a string pointer, length, and
//		allocation length.
//-----------------------------------------------------------------------------
struct strpla_s {
    char *	ptr				;
    size_t	len				;
    size_t	alen				;
};
typedef struct strpla_s		strpla_t	;
typedef struct strpla_s *	strpla_p	;


//-----------------------------------------------------------------------------
// macro	split_diagnose
//
// purpose	Print diagnostic info about a split state struct.
//
// note		This is mostly for debugging these split macros/functions
//		but it may be handy for programmers to detect other problems.
//
// arguments	1 (split_p) struct to initialize
//		2 (FILE *) open file to print to
//
// returns	(void)
//-----------------------------------------------------------------------------
#define split_diagnose(o,f) ({								\
	split_p		libh__spx;							\
	FILE *		libh__file;							\
	char *		libh__ptr;							\
	libh__spx = (o);								\
	libh__file = (f);								\
	fprintf( libh__file, "%8p: (struct split_s)\n", libh__spx );			\
	fprintf( libh__file, "->flags         = %08x\n", libh__spx->flags );		\
	fprintf( libh__file, "->call          = %08p\n", libh__spx->call );		\
	fprintf( libh__file, "->data_ptr      = %08p (\"", libh__spx->data_ptr );	\
	libh__ptr = libh__spx->data_ptr;						\
	while ( libh__ptr < libh__spx->data_end )					\
		fputc( * libh__ptr ++, libh__file );					\
	fputs( "\")\n", libh__file );							\
	fprintf( libh__file, "->data_end      = %08p\n", libh__spx->data_end );		\
	fprintf( libh__file, "->part_ptr      = %08p (\"", libh__spx->part_ptr );	\
	libh__ptr = libh__spx->part_ptr;						\
	while ( libh__ptr < libh__spx->data_end )					\
		fputc( * libh__ptr ++, libh__file );					\
	fputs( "\")\n", libh__file );							\
	fprintf( libh__file, "->part_end      = %08p\n", libh__spx->part_end );		\
	fprintf( libh__file, "->next_ptr      = %08p\n", libh__spx->next_ptr );		\
	fprintf( libh__file, "->sep.data.ch   = %02x\n", libh__spx->sep.data.ch );	\
	fprintf( libh__file, "->sep.data.ptr  = %08p\n", libh__spx->sep.data.ptr );	\
	fprintf( libh__file, "->sep.data.end  = %08p\n", libh__spx->sep.data.end );	\
})

//-----------------------------------------------------------------------------
// macro	split_init
//
// purpose	Initialize a string split state struct.
//
// warning	This function should only be called for objects not yet
//		initialized, otherwise memory allocation leakage may occur.
//
// arguments	1 (split_p) struct to initialize
//
// returns	(void)
//-----------------------------------------------------------------------------
#define split_init(o) ({							\
	split_p		libh__sp;						\
	libh__sp = (o);								\
	libh__sp->flags = 0;							\
	libh__sp->call = NULL;							\
	libh__sp->data_ptr = NULL;						\
	libh__sp->data_end = NULL;						\
	libh__sp->part_ptr = NULL;						\
	libh__sp->part_end = NULL;						\
	libh__sp->next_ptr = NULL;						\
	libh__sp->sep.data.ptr = NULL;						\
	libh__sp->sep.data.end = NULL;						\
})

//-----------------------------------------------------------------------------
// macro	split_new
//
// purpose	Create a new string split state object in allocated memory.
//		It is initialized so that when split_end() is called, it will
//		be deallocated.
//
// arguments	-none-
//
// returns	(split_p) pointer to string split state object
//-----------------------------------------------------------------------------
#define split_new() ({								\
	split_p		libh__sp;						\
	libh__sp = malloc( sizeof (split_t) );					\
	if ( libh__sp ) {							\
		libh__sp->flags = SPLIT_OBJ_ALLOCATED;				\
		libh__sp->call = NULL;						\
		libh__sp->data_ptr = NULL;					\
		libh__sp->data_end = NULL;					\
		libh__sp->part_ptr = NULL;					\
		libh__sp->part_end = NULL;					\
		libh__sp->next_ptr = NULL;					\
		libh__sp->sep.data.ptr = NULL;					\
		libh__sp->sep.data.end = NULL;					\
	}									\
	libh__sp;								\
})

//-----------------------------------------------------------------------------
// macro	split_new_alloca
//
// purpose	Create a new string split state object in memory allocated in
//		the current automatic memory frame that goes away upon return
//		from the function doing split_new_alloca().  All caveats about
//		using the alloca() function apply here.
//
// warning	Do not use split_new_alloca() unless you understand all the
//		implications of using alloca().
//
// arguments	-none-
//
// returns	(split_p) pointer to string split state object
//-----------------------------------------------------------------------------
#define split_new_alloca() ({							\
	split_p		libh__sp;						\
	libh__sp = alloca( sizeof (split_t) );					\
	if ( libh__sp ) {							\
		libh__sp->flags = 0;						\
		libh__sp->call = NULL;						\
		libh__sp->data_ptr = NULL;					\
		libh__sp->data_end = NULL;					\
		libh__sp->part_ptr = NULL;					\
		libh__sp->part_end = NULL;					\
		libh__sp->next_ptr = NULL;					\
		libh__sp->sep.data.ptr = NULL;					\
		libh__sp->sep.data.end = NULL;					\
	}									\
	libh__sp;								\
})

//-----------------------------------------------------------------------------
// macro	split_end
//
// purpose	End an existing split state object, releasing memory if
//		previously allocated.
//
// arguments	1 (split_p) split object to destroy
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_end(o) ({								\
	split_p		libh__sp;						\
	libh__sp = (o);								\
	if ( libh__sp->flags & SPLIT_STR_ALLOCATED ) {				\
		free( libh__sp->data_ptr );					\
	}									\
	if ( libh__sp->flags & SPLIT_OBJ_ALLOCATED ) {				\
		free( libh__sp );						\
	}									\
})

//-----------------------------------------------------------------------------
// macro	split_data_str
//
// purpose	Specify a string to be split.  Any previously specified string
//		is replaced, and the split state is reset.
//
// arguments	1 (split_p) split object to bind string to
//		2 (char *) string to bind to split object
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_data_str(o,s) ({							\
	split_p		libh__sp;						\
	libh__sp = (o);								\
	libh__sp->part_ptr = NULL;						\
	libh__sp->part_end = NULL;						\
	if ( libh__sp->flags & SPLIT_STR_ALLOCATED ) {				\
		free( libh__sp->data_ptr );					\
		libh__sp->flags &= ~ ( SPLIT_STR_ALLOCATED );			\
	}									\
	libh__sp->data_ptr = (char *)(s);					\
	libh__sp->next_ptr = libh__sp->data_ptr;				\
	libh__sp->data_end = libh__sp->data_ptr + strlen( libh__sp->data_ptr );	\
})

//-----------------------------------------------------------------------------
// macro	split_data_strpl
//
// purpose	Specify a string to be split, with length, as one argument.
//		Any previously specified string is replaced, and the split
//		state is reset.
//
// arguments	1 (split_p) split object to bind string to
//		2 (strpl_t) string to bind to split object
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_data_strpl(o,s) ({						\
	split_p		libh__sp;						\
	strpl_t		libh__st;						\
	libh__sp = (o);								\
	libh__sp->part_ptr = NULL;						\
	libh__sp->part_end = NULL;						\
	if ( libh__sp->flags & SPLIT_STR_ALLOCATED ) {				\
		free( libh__sp->data_ptr );					\
		libh__sp->flags &= ~ ( SPLIT_STR_ALLOCATED );			\
	}									\
	libh__st = (s);								\
	libh__sp->data_ptr = strpl_ptr(libh__st);				\
	libh__sp->next_ptr = libh__sp->data_ptr;				\
	libh__sp->data_end = libh__sp->data_ptr + strpl_len(libh__st);		\
})

//-----------------------------------------------------------------------------
// macro	split_data_str_pl
//
// purpose	Specify a string to be split, with length, as separate
//		arguments.  Any previously specified string is replaced, and
//		the split state is reset.
//
// arguments	1 (split_p) split object to bind string to
//		2 (char *) string to bind to split object
//		3 (size_t) length of string to bind to split object
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_data_str_pl(o,s,l) ({						\
	split_p		libh__sp;						\
	libh__sp = (o);								\
	libh__sp->part_ptr = NULL;						\
	libh__sp->part_end = NULL;						\
	if ( libh__sp->flags & SPLIT_STR_ALLOCATED ) {				\
		free( libh__sp->data_ptr );					\
		libh__sp->flags &= ~ ( SPLIT_STR_ALLOCATED );			\
	}									\
	libh__sp->data_ptr = (char *)(s);					\
	libh__sp->next_ptr = libh__sp->data_ptr;				\
	libh__sp->data_end = libh__sp->data_ptr + (l);				\
})

//-----------------------------------------------------------------------------
// macro	split_data_strpe
//
// purpose	Specify a string to be split, with end pointer, as one
//		argument.  Any previously specified string is replaced, and
//		the split state is reset.
//
// arguments	1 (split_p) split object to bind string to
//		2 (strpe_t) string to bind to split object
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_data_strpe(o,s) ({						\
	split_p		libh__sp;						\
	strpe_t		libh__st;						\
	libh__sp = (o);								\
	libh__sp->part_ptr = NULL;						\
	libh__sp->part_end = NULL;						\
	if ( libh__sp->flags & SPLIT_STR_ALLOCATED ) {				\
		free( libh__sp->data_ptr );					\
		libh__sp->flags &= ~ ( SPLIT_STR_ALLOCATED );			\
	}									\
	libh__st = (s);								\
	libh__sp->data_ptr = strpe_ptr(libh__st);				\
	libh__sp->next_ptr = libh__sp->data_ptr;				\
	libh__sp->data_end = strpe_end(libh__st);				\
})

//-----------------------------------------------------------------------------
// macro	split_data_str_pe
//
// purpose	Specify a string to be split, with end pointer, as separate
//		arguments.  Any previously specified string is replaced, and
//		the split state is reset.
//
// arguments	1 (split_p) split object to bind string to
//		2 (char *) string to bind to split object
//		3 (char *) end pointer to string
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_data_str_pe(o,s,e) ({						\
	split_p		libh__sp;						\
	libh__sp = (o);								\
	libh__sp->part_ptr = NULL;						\
	libh__sp->part_end = NULL;						\
	if ( libh__sp->flags & SPLIT_STR_ALLOCATED ) {				\
		free( libh__sp->data_ptr );					\
		libh__sp->flags &= ~ ( SPLIT_STR_ALLOCATED );			\
	}									\
	libh__sp->data_ptr = (char *)(s);					\
	libh__sp->next_ptr = libh__sp->data_ptr;				\
	libh__sp->data_end = (char *)(e);					\
})

//-----------------------------------------------------------------------------
// macro	split_data_dup
//
// purpose	Make an internal duplicate of the currently specified string
//		to be split, so it is no longer referenced from its original
//		memory location.  The split state is reset to the beginning.
//
// note		This string will be deallocated when a new string is specified
//		or when this split object is ended.
//
// warning	Any previously obtained split part pointers will be invalid.
//
// arguments	1 (split_p) split object to duplicate string in
//
// yields	(int)  0 : OK, now duplicated
//		(int)  1 : OK, already duplicated
//		(int) -1 : ERROR, malloc failed
//-----------------------------------------------------------------------------
#define split_data_dup(o) ({							\
	split_p		libh__sp;						\
	char *		libh__str;						\
	ptrdiff_t	libh__diff;						\
	size_t		libh__len;						\
	libh__sp = (o);								\
	( libh__sp->flags & SPLIT_STR_ALLOCATED )				\
	? 1									\
	: ({	libh__len = libh__sp->data_end - libh__sp->data_ptr;		\
		libh__str = malloc( libh__len + 1 );				\
	})									\
	? ({	memcpy( libh__str, libh__sp->data_ptr, libh__len );		\
		libh__str[libh__len] = 0;					\
		libh__sp->flags |= SPLIT_STR_ALLOCATED;				\
		libh__sp->next_ptr = libh__str;					\
		libh__sp->data_ptr = libh__str;					\
		libh__sp->data_end = libh__str + libh__len;			\
		libh__sp->part_ptr = NULL;					\
		libh__sp->part_end = NULL;					\
		0;								\
	})									\
	: -1;									\
})

//-----------------------------------------------------------------------------
// macro	split_data_dup_alloca
//
// purpose	Make an internal duplicate of the currently specified string
//		to be split, so it is no longer referenced from its original
//		memory location, using automatic frame memory as returned by
//		the alloca() function.  The split state is reset to the
//		beginning.
//
// note		This string will be deallocated when the function that does
//		split_data_dup_alloca() returns.
//
// warning	Do not use split_data_dup_alloca() unless you understand
//		all the implications of using alloca().
//
// warning	Any previously obtained split part pointers will be invalid.
//
// arguments	1 (split_p) split object to duplicate string in
//
// yields	(int)  0 : OK, now duplicated
//		(int)  1 : OK, already duplicated
//		(int) -1 : ERROR, alloca failed
//-----------------------------------------------------------------------------
#define split_data_dup_alloca(o) ({						\
	split_p		libh__sp;						\
	char *		libh__str;						\
	ptrdiff_t	libh__diff;						\
	size_t		libh__len;						\
	libh__sp = (o);								\
	( libh__sp->flags & SPLIT_STR_ALLOCATED )				\
	? 1									\
	: ({	libh__len = libh__sp->data_end - libh__sp->data_ptr;		\
		libh__str = alloca( libh__len + 1 );				\
	})									\
	? ({	memcpy( libh__str, libh__sp->data_ptr, libh__len );		\
		libh__str[libh__len] = 0;					\
		libh__sp->next_ptr = libh__str;					\
		libh__sp->data_ptr = libh__str;					\
		libh__sp->data_end = libh__str + libh__len;			\
		libh__sp->part_ptr = NULL;					\
		libh__sp->part_end = NULL;					\
		0;								\
	})									\
	: -1;									\
})

//-----------------------------------------------------------------------------
// macro	split_reset
//
// purpose	Reset a split object to start back at the beginning.
//
// arguments	1 (split_p) split object to reset
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_reset(o) ({							\
	split_p		libh__sp;						\
	libh__sp = (o);								\
	libh__sp->next_ptr = libh__sp->data_ptr;				\
	libh__sp->part_ptr = NULL;						\
	libh__sp->part_end = NULL;						\
})

//-----------------------------------------------------------------------------
// macro	split_sep_sptab
//
// purpose	Specify splitting on a group of one or more spaces and tabs.
//
// arguments	1 (split_p) split object to bind separator to
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_sep_sptab(o) ({							\
	split_p		libh__sp;						\
	libh__sp = (o);								\
	if ( libh__sp->flags & SPLIT_SEP_ALLOCATED ) {				\
		free( libh__sp->sep.data.ptr );					\
		libh__sp->flags &= ~ ( SPLIT_SEP_REF_ALLOC );			\
	}									\
	libh__sp->call = split_sep_find_sptab;					\
	libh__sp->next_ptr = libh__sp->data_ptr;				\
	libh__sp->part_ptr = NULL;						\
	libh__sp->part_end = NULL;						\
})

//-----------------------------------------------------------------------------
// macro	split_sep_space
//
// purpose	Specify splitting on a group of one or more whitespace
//		characters.
//
// arguments	1 (split_p) split object to bind separator to
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_sep_space(o) ({							\
	split_p		libh__sp;						\
	libh__sp = (o);								\
	if ( libh__sp->flags & SPLIT_SEP_ALLOCATED ) {				\
		free( libh__sp->sep.data.ptr );					\
		libh__sp->flags &= ~ ( SPLIT_SEP_REF_ALLOC );			\
	}									\
	libh__sp->call = split_sep_find_space;					\
	libh__sp->next_ptr = libh__sp->data_ptr;				\
	libh__sp->part_ptr = NULL;						\
	libh__sp->part_end = NULL;						\
})

//-----------------------------------------------------------------------------
// macro	split_sep_spctl
//
// purpose	Specify splitting on a group of one or more whitespace
//		characters and control characters.
//
// arguments	1 (split_p) split object to bind separator to
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_sep_spctl(o) ({							\
	split_p		libh__sp;						\
	libh__sp = (o);								\
	if ( libh__sp->flags & SPLIT_SEP_ALLOCATED ) {				\
		free( libh__sp->sep.data.ptr );					\
		libh__sp->flags &= ~ ( SPLIT_SEP_REF_ALLOC );			\
	}									\
	libh__sp->call = split_sep_find_spctl;					\
	libh__sp->next_ptr = libh__sp->data_ptr;				\
	libh__sp->part_ptr = NULL;						\
	libh__sp->part_end = NULL;						\
})

//-----------------------------------------------------------------------------
// macro	split_sep_punct
//
// purpose	Specify splitting on any punctuation character.
//
// arguments	1 (split_p) split object to bind separator to
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_sep_punct(o) ({							\
	split_p		libh__sp;						\
	libh__sp = (o);								\
	if ( libh__sp->flags & SPLIT_SEP_ALLOCATED ) {				\
		free( libh__sp->sep.data.ptr );					\
		libh__sp->flags &= ~ ( SPLIT_SEP_REF_ALLOC );			\
	}									\
	libh__sp->call = split_sep_find_punct;					\
	libh__sp->next_ptr = libh__sp->data_ptr;				\
	libh__sp->part_ptr = NULL;						\
	libh__sp->part_end = NULL;						\
})

//-----------------------------------------------------------------------------
// macro	split_sep_crlf
//
// purpose	Specify splitting on a CRLF, or lone CR, or lone LF.
//
// arguments	1 (split_p) split object to bind separator to
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_sep_crlf(o) ({							\
	split_p		libh__sp;						\
	libh__sp = (o);								\
	if ( libh__sp->flags & SPLIT_SEP_ALLOCATED ) {				\
		free( libh__sp->sep.data.ptr );					\
		libh__sp->flags &= ~ ( SPLIT_SEP_REF_ALLOC );			\
	}									\
	libh__sp->call = split_sep_find_crlf;					\
	libh__sp->next_ptr = libh__sp->data_ptr;				\
	libh__sp->part_ptr = NULL;						\
	libh__sp->part_end = NULL;						\
})

//-----------------------------------------------------------------------------
// macro	split_sep_line
//
// purpose	Specify splitting on a CR(any newline), lone CR, or lone
//		(any newline).
//
// arguments	1 (split_p) split object to bind separator to
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_sep_line(o) ({							\
	split_p		libh__sp;						\
	libh__sp = (o);								\
	if ( libh__sp->flags & SPLIT_SEP_ALLOCATED ) {				\
		free( libh__sp->sep.data.ptr );					\
		libh__sp->flags &= ~ ( SPLIT_SEP_REF_ALLOC );			\
	}									\
	libh__sp->call = split_sep_find_line;					\
	libh__sp->next_ptr = libh__sp->data_ptr;				\
	libh__sp->part_ptr = NULL;						\
	libh__sp->part_end = NULL;						\
})

//-----------------------------------------------------------------------------
// macro	split_sep_char
//
// purpose	Specify splitting on a specified single character.
//
// arguments	1 (split_p) split object to bind separator to
//		2 (int) separator character
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_sep_char(o,c) ({							\
	split_p		libh__sp;						\
	libh__sp = (o);								\
	if ( libh__sp->flags & SPLIT_SEP_ALLOCATED ) {				\
		free( libh__sp->sep.data.ptr );					\
		libh__sp->flags &= ~ ( SPLIT_SEP_REF_ALLOC );			\
	}									\
	libh__sp->call = split_sep_find_char;					\
	libh__sp->sep.data.ch = (c);						\
	libh__sp->next_ptr = libh__sp->data_ptr;				\
	libh__sp->part_ptr = NULL;						\
	libh__sp->part_end = NULL;						\
})

//-----------------------------------------------------------------------------
// macro	split_sep_ch_call
//
// purpose	Specify splitting on a single character tested true by a
//		specified callback function.
//
// arguments	1 (split_p) split object to bind separator to
//		2 (int (*)(int)) function to test character for separator
//
// yields	(void)
//
// note		This macro can use the isxxxxx() functions in the Standard
//		C library as callbacks; specify them without arguments and
//		enclosed in parenthesis to avoid expansion from macro
//		implementations.  For example:
//
// example	split_sep_ch_call( this_obj, (islower) )
//-----------------------------------------------------------------------------
#define split_sep_ch_call(o,f) ({						\
	split_p		libh__sp;						\
	libh__sp = (o);								\
	if ( libh__sp->flags & SPLIT_SEP_ALLOCATED ) {				\
		free( libh__sp->sep.data.ptr );					\
		libh__sp->flags &= ~ ( SPLIT_SEP_REF_ALLOC );			\
	}									\
	libh__sp->call = split_sep_find_ch_call;				\
	libh__sp->sep.data.ch_call = (f);					\
	libh__sp->next_ptr = libh__sp->data_ptr;				\
	libh__sp->part_ptr = NULL;						\
	libh__sp->part_end = NULL;						\
})

//-----------------------------------------------------------------------------
// macro	split_sep_call
//
// purpose	Specify splitting on whatever separator a callback function
//		decides is a separator at a given point in the data string.
//
// arguments	1 (split_p) split object to bind separator to
//		2 (size_t (*)(char *,size_t)) function to test for separator
//
// yields	(void)
//
// note		The callback function is called with a pointer into the data
//		string and the length of the remainder of the data string.  It
//		is expected to return either -1 to indicate a separator is not
//		present at that point, or the length of the separator if there
//		is one at that point.
//-----------------------------------------------------------------------------
#define split_sep_call(o,f) ({							\
	split_p		libh__sp;						\
	libh__sp = (o);								\
	if ( libh__sp->flags & SPLIT_SEP_ALLOCATED ) {				\
		free( libh__sp->sep.data.ptr );					\
		libh__sp->flags &= ~ ( SPLIT_SEP_REF_ALLOC );			\
	}									\
	libh__sp->call = split_sep_find_call;					\
	libh__sp->sep.data.call = (f);						\
	libh__sp->next_ptr = libh__sp->data_ptr;				\
	libh__sp->part_ptr = NULL;						\
	libh__sp->part_end = NULL;						\
})

//-----------------------------------------------------------------------------
// macro	split_sep_clist_str
//
// purpose	Specify splitting on any character in a list given as a
//		zero terminated string.  The zero character cannot be a
//		separator character in this case.
//
// arguments	1 (split_p) split object to bind separator to
//		2 (char *) pointer to character list string.
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_sep_clist_str(o,l) ({						\
	split_p		libh__sp;						\
	libh__sp = (o);								\
	if ( libh__sp->flags & SPLIT_SEP_ALLOCATED ) {				\
		free( libh__sp->sep.data.ptr );					\
		libh__sp->flags &= ~ ( SPLIT_SEP_ALLOCATED );			\
	}									\
	libh__sp->flags |= SPLIT_SEP_REFERENCE;					\
	libh__sp->sep.data.ptr = (char *)(l);					\
	libh__sp->sep.data.end = libh__sp->sep.data.ptr + strlen( libh__sp->sep.data.ptr );	\
	libh__sp->call = split_sep_find_clist;					\
	libh__sp->next_ptr = libh__sp->data_ptr;				\
	libh__sp->part_ptr = NULL;						\
	libh__sp->part_end = NULL;						\
})

//-----------------------------------------------------------------------------
// macro	split_sep_ctable
//
// purpose	Specify splitting on any 8-bit character in a table.  Such a
//		table is an array indexed by a value from 0 to 255 yielding a
//		zero value for an index equal to the code of a character that
//		is not a separator, or a non-zero value for an index equal to
//		the code of a character that that is a separator.
//
// arguments	1 (split_p) split object to bind separator to
//		2 (char *) pointer to character indexed table
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_sep_ctable(o,t) ({						\
	split_p		libh__sp;						\
	libh__sp = (o);								\
	if ( libh__sp->flags & SPLIT_SEP_ALLOCATED ) {				\
		free( libh__sp->sep.data.ptr );					\
		libh__sp->flags &= ~ ( SPLIT_SEP_ALLOCATED );			\
	}									\
	libh__sp->flags |= SPLIT_SEP_REFERENCE;					\
	libh__sp->sep.data.ptr = (char *)(t);					\
	libh__sp->sep.data.end = libh__sp->sep.data.ptr + ( (UCHAR_MAX) + 1 );	\
	libh__sp->call = split_sep_find_ctable;					\
	libh__sp->next_ptr = libh__sp->data_ptr;				\
	libh__sp->part_ptr = NULL;						\
	libh__sp->part_end = NULL;						\
})

//-----------------------------------------------------------------------------
// macro	split_sep_str
//
// purpose	Specify splitting on specified string given as a pointer to a
//		zero terminated string.  The zero character cannot be part of
//		the separator string.
//
// arguments	1 (split_p) split object to bind separator to
//		2 (char *) separator string
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_sep_str(o,s) ({							\
	split_p		libh__sp;						\
	libh__sp = (o);								\
	if ( libh__sp->flags & SPLIT_SEP_ALLOCATED ) {				\
		free( libh__sp->sep.data.ptr );					\
		libh__sp->flags &= ~ ( SPLIT_SEP_ALLOCATED );			\
	}									\
	libh__sp->flags |= SPLIT_SEP_REFERENCE;					\
	libh__sp->sep.data.ptr = (char *)(s);					\
	libh__sp->sep.data.end = libh__sp->sep.data.ptr + strlen( libh__sp->sep.data.ptr );	\
	libh__sp->call = split_sep_find_string;					\
	libh__sp->next_ptr = libh__sp->data_ptr;				\
	libh__sp->part_ptr = NULL;						\
	libh__sp->part_end = NULL;						\
})

//-----------------------------------------------------------------------------
// macro	split_sep_strpl
//
// purpose	Specify splitting on specified string given as a pointer and
//		length in one argument.
//
// arguments	1 (split_p) split object to bind separator to
//		2 (strpl_t) separator string pointer/length
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_sep_strpl(o,s) ({							\
	split_p		libh__sp;						\
	libh__sp = (o);								\
	if ( libh__sp->flags & SPLIT_SEP_ALLOCATED ) {				\
		free( libh__sp->sep.data.ptr );					\
		libh__sp->flags &= ~ ( SPLIT_SEP_ALLOCATED );			\
	}									\
	{									\
		strpl_t	libh__st;						\
		libh__st = (s);							\
		libh__sp->sep.data.ptr = strpl_ptr(libh__st);			\
		libh__sp->sep.data.end = strpl_end(libh__st);			\
	}									\
	libh__sp->flags |= SPLIT_SEP_REFERENCE;					\
	libh__sp->call = split_sep_find_string;					\
	libh__sp->next_ptr = libh__sp->data_ptr;				\
	libh__sp->part_ptr = NULL;						\
	libh__sp->part_end = NULL;						\
})

//-----------------------------------------------------------------------------
// macro	split_sep_str_pl
//
// purpose	Specify splitting on specified string given as a pointer and
//		length in two separate arguments.
//
// arguments	1 (split_p) split object to bind separator to
//		2 (char *) separator string pointer
//		3 (size_t) separator string length
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_sep_str_pl(o,s,l) ({						\
	split_p		libh__sp;						\
	libh__sp = (o);								\
	if ( libh__sp->flags & SPLIT_SEP_ALLOCATED ) {				\
		free( libh__sp->sep.data.ptr );					\
		libh__sp->flags &= ~ ( SPLIT_SEP_ALLOCATED );			\
	}									\
	libh__sp->flags |= SPLIT_SEP_REFERENCE;					\
	libh__sp->sep.data.ptr = (char *)(s);					\
	libh__sp->sep.data.end = libh__sp->sep.data.ptr + (l);			\
	libh__sp->call = split_sep_find_string;					\
	libh__sp->next_ptr = libh__sp->data_ptr;				\
	libh__sp->part_ptr = NULL;						\
	libh__sp->part_end = NULL;						\
})

//-----------------------------------------------------------------------------
// macro	split_sep_strpe
//
// purpose	Specify splitting on specified string given as a pointer and
//		end pointer in one argument.
//
// arguments	1 (split_p) split object to bind separator to
//		2 (strpe_t) separator string pointer/end
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_sep_strpe(o,s) ({							\
	split_p		libh__sp;						\
	libh__sp = (o);								\
	if ( libh__sp->flags & SPLIT_SEP_ALLOCATED ) {				\
		free( libh__sp->sep.data.ptr );					\
		libh__sp->flags &= ~ ( SPLIT_SEP_ALLOCATED );			\
	}									\
	{									\
		strpe_t libh__st;						\
		libh__st = (s);							\
		libh__sp->sep.data.ptr = strpe_ptr(libh__st);			\
		libh__sp->sep.data.end = strpe_end(libh__st);			\
	}									\
	libh__sp->flags |= SPLIT_SEP_REFERENCE;					\
	libh__sp->call = split_sep_find_string;					\
	libh__sp->next_ptr = libh__sp->data_ptr;				\
	libh__sp->part_ptr = NULL;						\
	libh__sp->part_end = NULL;						\
})

//-----------------------------------------------------------------------------
// macro	split_sep_str_pe
//
// purpose	Specify splitting on specified string given as a pointer and
//		end pointer in two separate arguments.
//
// arguments	1 (split_p) split object to bind separator to
//		2 (char *) separator string pointer
//		3 (char *) separator string end pointer
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_sep_str_pe(o,s,e) ({						\
	split_p		libh__sp;						\
	libh__sp = (o);								\
	if ( libh__sp->flags & SPLIT_SEP_ALLOCATED ) {				\
		free( libh__sp->sep.data.ptr );					\
		libh__sp->flags &= ~ ( SPLIT_SEP_ALLOCATED );			\
	}									\
	libh__sp->flags |= SPLIT_SEP_REFERENCE;					\
	libh__sp->sep.data.ptr = (char *)(s);					\
	libh__sp->sep.data.end = (char *)(e);					\
	libh__sp->call = split_sep_find_string;					\
	libh__sp->next_ptr = libh__sp->data_ptr;				\
	libh__sp->part_ptr = NULL;						\
	libh__sp->part_end = NULL;						\
})

//-----------------------------------------------------------------------------
// macro	split_sep_trim_spaces
//
// purpose	Specify that split parts will have spaces trimmed.
//
// arguments	1 (split_p) split object to modify
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_sep_trim_spaces(o) ({						\
	(o)->sep_flags |= SPLIT_SEP_TRIMSPACES;					\
})

//-----------------------------------------------------------------------------
// macro	split_sep_no_trim_spaces
//
// purpose	Specify that split parts will not have spaces trimmed.
//
// arguments	1 (split_p) split object to modify
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_sep_no_trim_spaces(o) ({						\
	(o)->sep_flags &= ~ ( SPLIT_SEP_TRIMSPACES );				\
})

//-----------------------------------------------------------------------------
// macro	split_sep_invert
//
// purpose	Specify that separator character detection in a list or table
//		is to be logically inverted.
//
// note		If a list or table is not currently specified as the separator,
//		then this setting will have no effect on string splitting.
//		However, the setting will be stored and retained so that it
//		will be in effect should a list or table be specified as the
//		separator at a later time.  This allows the calling program
//		to call split_sep_invert() first then set the list or table
//		later, possibly multiple times.
//
// arguments	1 (split_p) split object to modify
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_sep_invert(o) ({							\
	(o)->sep_flags |= SPLIT_SEP_INVERT;					\
})

//-----------------------------------------------------------------------------
// macro	split_sep_no_invert
//
// purpose	Specify that separator character detection in a list or table
//		is not to be logically inverted.
//
// note		If a list or table is not currently specified as the separator,
//		then this setting will have no effect on string splitting.
//		However, the setting will be stored and retained so that it
//		will be in effect should a list or table be specified as the
//		separator at a later time.  This allows the calling program
//		to call split_sep_invert() first then set the list or table
//		later, possibly multiple times.
//
// arguments	1 (split_p) split object to modify
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_sep_no_invert(o) ({						\
	(o)->sep_flags &= ~ ( SPLIT_SEP_INVERT );				\
})

//-----------------------------------------------------------------------------
// macro	split_sep_no_case
//
// purpose	Specify that separator comparison will not be case sensitive.
//
// arguments	1 (split_p) split object to modify
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_sep_no_case(o) ({							\
	(o)->sep_flags |= SPLIT_SEP_NOCASE;					\
})

//-----------------------------------------------------------------------------
// macro	split_sep_case
//
// purpose	Specify that separator comparison will be case sensitive.
//
// arguments	1 (split_p) split object to modify
//
// yields	(void)
//-----------------------------------------------------------------------------
#define split_sep_case(o) ({							\
	(o)->sep_flags &= ~ ( SPLIT_SEP_NOCASE );				\
})

//-----------------------------------------------------------------------------
// macro	split_sep_dup
//
// purpose	Make an internal duplicate of the currently specified separator
//		memory data, so it is no longer referenced from its original
//		memory location.
//
// note		This memory will be deallocated when a new separator is
//		specified or when this object is destroyed.
//
// arguments	1 (split_p) split object to duplicate separator in
//
// yields	(int)  0 : OK, duplicated
//		(int)  1 : OK, already duplicated, or not needed
//		(int) -1 : ERROR, malloc failed
//-----------------------------------------------------------------------------
#define split_sep_dup(o) ({							\
	split_p		libh__sp;						\
	char *		libh__sep;						\
	size_t		libh__len;						\
	libh__sp = (o);								\
	( libh__sp->flags & SPLIT_SEP_REF_ALLOC != SPLIT_SEP_REFERENCE )	\
	? 1									\
	: ({									\
		libh__len = libh__sp->sep.data.end - libh__sp->sep.data.ptr;	\
		libh__sep = malloc( libh__len + 1 );				\
		libh__sep;							\
	})									\
	? ({									\
		memcpy( libh__sep, libh__sp->sep.data.ptr, libh__len );		\
		libh__sep[libh__len] = 0;					\
		libh__sp->flags |= SPLIT_SEP_ALLOCATED;				\
		libh__sp->next_ptr = libh__sep;					\
		libh__sp->sep.data.ptr = libh__sep;				\
		libh__sp->sep.data.end = libh__sep + libh__len;			\
		libh__sp->part_ptr = NULL;					\
		libh__sp->part_end = NULL;					\
		0;								\
	})									\
	: -1;									\
})

//-----------------------------------------------------------------------------
// macro	split_sep_dup_alloca
//
// purpose	Make an internal duplicate of the currently specified separator
//		memory data, so it is no longer referenced from its original
//		memory location, using automatic frame memory as returned by
//		the alloca() function.  The split state is reset to the
//		beginning.
//
// note		This data will be deallocated when the function that does
//		split_sep_dup_alloca() returns.
//
// warning	Do not use split_sep_dup_alloca() unless you understand
//		all the implications of using alloca().
//
// arguments	1 (split_p) split object to duplicate string in
//
// yields	(int)  0 : OK, now duplicated
//		(int)  1 : OK, already duplicated, or not needed
//		(int) -1 : ERROR, alloca failed
//-----------------------------------------------------------------------------
#define split_sep_dup_alloca(o) ({						\
	split_p		libh__sp;						\
	char *		libh__sep;						\
	size_t		libh__len;						\
	libh__sp = (o);								\
	( libh__sp->flags & SPLIT_SEP_REF_ALLOC != SPLIT_SEP_REFERENCE )	\
	? 1									\
	: ({									\
		libh__len = libh__sp->sep.data.end - libh__sp->sep.data.ptr;	\
		libh__sep = alloca( libh__len + 1 );				\
		libh__sep;							\
	})									\
	? ({									\
		memcpy( libh__sep, libh__sp->sep.data.ptr, libh__len );		\
		libh__sep[libh__len] = 0;					\
		libh__sp->next_ptr = libh__sep;					\
		libh__sp->sep.data.ptr = libh__sep;				\
		libh__sp->sep.data.end = libh__sep + libh__len;			\
		libh__sp->part_ptr = NULL;					\
		libh__sp->part_end = NULL;					\
		0;								\
	})									\
	: -1;									\
})

//-----------------------------------------------------------------------------
// macro	split_part_ptr
//
// purpose	Access the pointer of the most recently split part.
//
// arguments	1 (split_p) split object to access
//
// yields	(char *) pointer of split part
//-----------------------------------------------------------------------------
#define split_part_ptr(o) ((o)->part_ptr)

//-----------------------------------------------------------------------------
// macro	split_part_end
//
// purpose	Access the end pointer of the most recently split part.
//
// arguments	1 (split_p) split object to access
//
// yields	(char *) end pointer of split part
//-----------------------------------------------------------------------------
#define split_part_end(o) ((o)->part_end)

//-----------------------------------------------------------------------------
// macro	split_part_len
//
// purpose	Get the length of the most recently split part.
//
// arguments	1 (split_p) split object to access
//
// yields	(size_t) length of split part
//-----------------------------------------------------------------------------
#define split_part_len(o) ({							\
	split_p		libh__sp;						\
	libh__sp = (o);								\
	libh__sp->part_end - libh__sp->part_ptr;				\
})

//-----------------------------------------------------------------------------
// macro	split_part_str (see WARNING)
//
// purpose	Get the pointer of the most recently split part after zero
//		terminating it for use as an ordinary C string.
//
// arguments	1 (split_p) split object to access
//
// yields	(char *) pointer of split part
//
// WARNING	This macro modifies the string the parts are selected from.
//		If this is the original string, modifying it is likely to
//		impact re-use of the string, including repeat splits.  The
//		use of split_data_dup() provides safer operation.
//-----------------------------------------------------------------------------
#define split_part_str(o) ({							\
	split_p		libh__sp;						\
	libh__sp = (o);								\
	* libh__sp->part_end = 0;						\
	libh__sp->part_ptr;							\
})

//-----------------------------------------------------------------------------
// macro	split_part_strpl
//
// purpose	Access the pointer and length of the most recently split part.
//
// arguments	1 (split_p) split object to access
//
// yields	(strpl_t) pointer/length of split part
//-----------------------------------------------------------------------------
#define split_part_ptrpl(o) (strpl(((o)->part_ptr),(((o)->part_end)-((o)->part_ptr))))

//-----------------------------------------------------------------------------
// macro	split_part_strpe
//
// purpose	Access the pointer and end pointer of the most recently split
//		part.
//
// arguments	1 (split_p) split object to access
//
// yields	(strpe_t) pointer/end of split part
//-----------------------------------------------------------------------------
#define split_part_ptrpe(o) (strpe(((o)->part_ptr),((o)->part_end)))

//-----------------------------------------------------------------------------
// macro	split_part_mem_copy
//
// purpose	Copy the split part to the specified destination location.
//
// arguments	1 (split_p) split object to access
//		2 (char *) destination location
//		3 (size_t) size of destination
//
// yields	(size_t) length actually copied
//-----------------------------------------------------------------------------
#define split_part_mem_copy(o,p,s) ({						\
	split_p		libh__sp;						\
	size_t		libh__len;						\
	libh__sp = (o);								\
	libh__len = (s);							\
	if ( libh__len > libh__sp->part_end - libh__sp->part_ptr )		\
	     libh__len = libh__sp->part_end - libh__sp->part_ptr;		\
	memcpy( (p), libh__sp->part_ptr, libh__len );				\
	libh__len;								\
})

//-----------------------------------------------------------------------------
// macro	split_part_str_copy
//
// purpose	Copy the split part to the destination location and make sure
//		it is zero terminated.
//
// arguments	1 (split_p) split object to access
//		2 (char *) destination location
//		3 (size_t) size of destination (including zero termination)
//
// yields	(size_t) length actually copied
//-----------------------------------------------------------------------------
#define split_part_str_copy(o,p,s) ({						\
	split_p		libh__sp;						\
	char *		libh__ptr;						\
	size_t		libh__len;						\
	libh__sp = (o);								\
	libh__len = (s) - 1;							\
	if ( libh__len > libh__sp->part_end - libh__sp->part_ptr )		\
	     libh__len = libh__sp->part_end - libh__sp->part_ptr;		\
	libh__ptr = (p);							\
	memcpy( libh__ptr, libh__sp->part_ptr, libh__len );			\
	libh__ptr[ libh__len ] = 0;						\
	libh__len;								\
})

//-----------------------------------------------------------------------------
// macro	split_part_str_append
//
// purpose	Append the split part to the destination string (already zero
//		terminated) and make sure it remains zero terminated.
//
// arguments	1 (split_p) split object to access
//		2 (char *) destination location (with zero terminated string)
//		3 (size_t) size of destination (including zero termination)
//
// yields	(size_t) length actually copied
//-----------------------------------------------------------------------------
#define split_part_str_append(o,p,s) ({						\
	split_p		libh__sp;						\
	char *		libh__ptr;						\
	size_t		libh__len;						\
	libh__sp = (o);								\
	libh__ptr = (p);							\
	libh__len = (s) - 1;							\
	while ( * libh__ptr && libh__len ) { ++ libh__ptr; -- libh__len; }	\
	if ( libh__len > libh__sp->part_end - libh__sp->part_ptr )		\
	     libh__len = libh__sp->part_end - libh__sp->part_ptr;		\
	memcpy( libh__ptr, libh__sp->part_ptr, libh__len );			\
	libh__ptr[ libh__len ] = 0;						\
	libh__len;								\
})

//-----------------------------------------------------------------------------
// macro	split_part_str_dup
//
// purpose	Duplicate the split part to allocated memory and make sure it
//		is zero terminated.
//
// arguments	1 (split_p) split object to access
//
// yields	(char *) pointer to allocated memory with duplicated part
//		(char *) NULL if memory cannot be allocated
//
// note		The caller must free() this memory when no longer needed for
//		this purpose and memory usage is to be recycled.
//-----------------------------------------------------------------------------
#define split_part_str_dup(o) ({						\
	split_p		libh__sp;						\
	char *		libh__ptr;						\
	size_t		libh__len;						\
	libh__sp = (o);								\
	libh__len = libh__sp->part_end - libh__sp->part_ptr;			\
	libh__ptr = malloc( libh__len + 1 );					\
	if ( libh__ptr ) {							\
		memcpy( libh__ptr, libh__sp->part_ptr, libh__len );		\
		libh__ptr[ libh__len ] = 0;					\
	}									\
	libh__ptr;								\
})

//-----------------------------------------------------------------------------
// macro	split_part_str_dup_alloca
//
// purpose	Duplicate the split part to memory allocated using the alloca()
//		function and make sure it is zero terminated.
//
// arguments	1 (split_p) split object to access
//
// yields	(char *) pointer to allocated memory with duplicated part
//		(char *) NULL if memory cannot be allocated
//
// note		The memory used for this duplicate is not directly freeable
//		but will be freed upon exit from the function this macro is
//		used in.
//
// note		Use of this macro should only be made where use of the alloca()
//		function is appropriate.  If you do not understand how alloca()
//		works, then do not use the split_part_str_dup_alloca() macro.
//-----------------------------------------------------------------------------
#define split_part_str_dup_alloca(o) ({						\
	split_p		libh__sp;						\
	char *		libh__ptr;						\
	size_t		libh__len;						\
	libh__sp = (o);								\
	libh__len = libh__sp->part_end - libh__sp->part_ptr;			\
	libh__ptr = alloca( libh__len + 1 );					\
	if ( libh__ptr ) {							\
		memcpy( libh__ptr, libh__sp->part_ptr, libh__len );		\
		libh__ptr[ libh__len ] = 0;					\
	}									\
	libh__ptr;								\
})

//-----------------------------------------------------------------------------
// macro	mem_cpy
// alias	mem_copy
//
// purpose	Copy to specified space, limited by given size, the contents
//		of one or more strings of bytes specified with pointer and
//		length pairs.  Multiple data sources are concatenated.
//
// arguments	1 (char *) target space to copy strings to
//		2 (size_t) maximum size of target space
//		3 (const char *) source string to copy
//		4 (size_t) length of source string to copy
//		... repeat last 2
//
// returns	(size_t) total bytes actually copied
//-----------------------------------------------------------------------------
#define mem_cpy(t,s,p,x...) ((mem_cpy)((t),(s),(p),x,(const char*)(NULL)))
#define mem_copy(t,s,p,x...) ((mem_cpy)((t),(s),(p),x,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	mem_dup_len
//
// purpose	Duplicate into allocated space the contents of one or more
//		strings of bytes specified with pointer and length pairs,
//		and store the total length of the result where specified.
//
// arguments	1 (size_t *) where to store total length
//		2 (const char *) source string to duplicate
//		3 (size_t) length of string to duplicate
//		... repeat last 2
//
// returns	(char *) new allocated duplicate string, caller must free
//-----------------------------------------------------------------------------
#define mem_dup_len(l,p,x...) ((mem_dup_len)((l),(p),x,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// function	mem_dup
//
// purpose	Duplicate into allocated space the contents of one or more
//		strings of bytes specified with pointer and length pairs.
//
// arguments	1 (const char *) source string to duplicate
//		2 (size_t) length of string to duplicate
//		... repeat last 2
//
// returns	(char *) new allocated duplicate string, caller must free
//-----------------------------------------------------------------------------
#define mem_dup(p,x...) ((mem_dup_len)((size_t*)(NULL),(p),x,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	mem_dup_ptr_ptr
//
// purpose	Duplicate into allocated space the contents of one string of
//		bytes beginning where the first pointer points to and ending
//		just before where the second pointer pounts to.
//
// arguments	1 (const char *) pointer to where string begins
//		2 (const char *) pointer to where string ends
//
// returns	(char *) new allocated duplicate string, caller must free
//-----------------------------------------------------------------------------
#define mem_dup_ptr_ptr(ptr,end) ({						\
	const char * libh__ptr = ( ptr );					\
	const char * libh__end = ( end );					\
	size_t libh__len = libh__end - libh__ptr;				\
	char * libh__new = malloc( libh__len + 1 );				\
	if ( libh__new ) {							\
		memcpy( libh__new, libh__ptr, libh__len );			\
		libh__new[ libh__len ] = 0;					\
	}									\
	libh__new;								\
})

//-----------------------------------------------------------------------------
// macro	while_split
//
// purpose	Iterate over the parts of a split object.
//
// arguments	1 (split_p) pointer to split object
//
// usage	split_while( split_obj ) {
//			/* statements */
//		}
//-----------------------------------------------------------------------------
#define while_split(o) while ( split_next( (o) ) == 0 )

//-----------------------------------------------------------------------------
// macro	str_app_arg
//
// purpose	Append to the first string (given with maximum available space)
//		the contents of zero or more (multiple) strings.
//
//		Do append only if all source strings can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3..N (const char *) source strings to append
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
#define str_app_arg(t,l,a...) ((str_app_arg)((t),(l),a,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	str_app_arg_lim
//
// purpose	Append to the first string (given with maximum available space)
//		the contents of zero or more (multiple) strings limited by
//		number of characters.
//
//		Do append only if all source strings can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3..N-1 (const char *) source strings to append
//		4..N (size_t) limit length of source strings
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
#define str_app_arg_lim(t,l,a...) ((str_app_arg_lim)((t),(l),a,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	str_app_arg_lim_sep
//
// purpose	Append to the first string (given with maximum available space)
//		the contents of zero or more (multiple) strings with a
//		separator string inserted between multiple strings limited by
//		number of characters.
//
//		Do append only if all source strings can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) separator string to insert
//		4 (size_t) limit length of separator string
//		5..N-1 (const char *) source strings to append
//		6..N (size_t) limit length of source string
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
#define str_app_arg_lim_sep(t,l,s,z,a...) ((str_app_arg_lim_sep)((t),(l),(s),(z),a,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	str_app_arg_sep
//
// purpose	Append to the first string (given with maximum available space)
//		the contents of zero or more (multiple) strings with a
//		separator string inserted between multiple strings.
//
//		Do append only if all source strings can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) separator string to insert
//		4..N (const char *) source strings to append
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
#define str_app_arg_sep(t,l,s,a...) ((str_app_arg_sep)((t),(l),(s),a,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macros	str_app_da str_app_dA
//		str_app_de str_app_dE
//		str_app_df str_app_dF
//		str_app_dg str_app_dG
//
// purpose	Append to the first string (given with maximum available space)
//		a string of digits representing the double value.  Use a format
//		type the same as the ending character of the macro.
//
//		Do append only if all the digits can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (double) the number to convert
//		4 (int) total conversion field size
//		5 (int) fractional precision size
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
#if LIBC1999 == 1
#define str_app_da(t,s,v,f,p)	str_app_d((t),(s),(v),(f),(p),'a')
#define str_app_dA(t,s,v,f,p)	str_app_d((t),(s),(v),(f),(p),'A')
#endif

#define str_app_de(t,s,v,f,p)	str_app_d((t),(s),(v),(f),(p),'e')
#define str_app_dE(t,s,v,f,p)	str_app_d((t),(s),(v),(f),(p),'E')

#define str_app_df(t,s,v,f,p)	str_app_d((t),(s),(v),(f),(p),'f')
#define str_app_dF(t,s,v,f,p)	str_app_d((t),(s),(v),(f),(p),'F')

#define str_app_dg(t,s,v,f,p)	str_app_d((t),(s),(v),(f),(p),'g')
#define str_app_dG(t,s,v,f,p)	str_app_d((t),(s),(v),(f),(p),'G')

//-----------------------------------------------------------------------------
// macros	str_app_lda str_app_ldA
//		str_app_lde str_app_ldE
//		str_app_ldf str_app_ldF
//		str_app_ldg str_app_ldG
//
// purpose	Append to the first string (given with maximum available space)
//		a string of digits representing the long double value.  Use a
//		format type the same as the ending character of the macro.
//
//		Do append only if all the digits can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (long double) the number to convert
//		4 (int) total conversion field size
//		5 (int) fractional precision size
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
#if LIBC1999 == 1
#define str_app_lda(t,s,v,f,p)	str_app_ld((t),(s),(v),(f),(p),'a')
#define str_app_ldA(t,s,v,f,p)	str_app_ld((t),(s),(v),(f),(p),'A')
#endif

#define str_app_lde(t,s,v,f,p)	str_app_ld((t),(s),(v),(f),(p),'e')
#define str_app_ldE(t,s,v,f,p)	str_app_ld((t),(s),(v),(f),(p),'E')

#define str_app_ldf(t,s,v,f,p)	str_app_ld((t),(s),(v),(f),(p),'f')
#define str_app_ldF(t,s,v,f,p)	str_app_ld((t),(s),(v),(f),(p),'F')

#define str_app_ldg(t,s,v,f,p)	str_app_ld((t),(s),(v),(f),(p),'g')
#define str_app_ldG(t,s,v,f,p)	str_app_ld((t),(s),(v),(f),(p),'G')

//-----------------------------------------------------------------------------
// macros	str_app_si_bin
//		str_app_si_oct
//		str_app_si_dec
//		str_app_si_cet
//
// purpose	Append to the first string (given with maximum available space)
//		a string of digits representing the signed int value converted
//		in the named base.
//
//		Do append only if all the digits can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (signed int) the number to convert
//		4 (int) minimum number of digits
//		5 (int) (char) padding character
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
#define str_app_si_bin(t,s,v,m,p)	str_app_si((t),(s),(v),(m),(p),2,(const char*)(NULL))
#define str_app_si_oct(t,s,v,m,p)	str_app_si((t),(s),(v),(m),(p),8,(const char*)(NULL))
#define str_app_si_dec(t,s,v,m,p)	str_app_si((t),(s),(v),(m),(p),10,(const char*)(NULL))
#define str_app_si_cet(t,s,v,m,p)	str_app_si((t),(s),(v),(m),(p),16,(const char*)(NULL))

//-----------------------------------------------------------------------------
// function	str_app_si_text
//
// purpose	Append to the first string (given with maximum available space)
//		a string of text representing the signed int value.
//
//		Do append only if all the text can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (signed int) the number to convert
//		4 (const char *) language id string, default "eng"
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
#define str_app_si_text(t,s,v,l) (str_app_sl_text((t),(s),((signed long)(v)),(l)))

//-----------------------------------------------------------------------------
// macros	str_app_sl_bin
//		str_app_sl_oct
//		str_app_sl_dec
//		str_app_sl_cet
//
// purpose	Append to the first string (given with maximum available space)
//		a string of digits representing the signed long value converted
//		in the named base.
//
//		Do append only if all the digits can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (signed long) the number to convert
//		4 (int) minimum number of digits
//		5 (int) (char) padding character
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
#define str_app_sl_bin(t,s,v,m,p)	str_app_sl((t),(s),(v),(m),(p),2,(const char*)(NULL))
#define str_app_sl_oct(t,s,v,m,p)	str_app_sl((t),(s),(v),(m),(p),8,(const char*)(NULL))
#define str_app_sl_dec(t,s,v,m,p)	str_app_sl((t),(s),(v),(m),(p),10,(const char*)(NULL))
#define str_app_sl_cet(t,s,v,m,p)	str_app_sl((t),(s),(v),(m),(p),16,(const char*)(NULL))

//-----------------------------------------------------------------------------
// macros	str_app_sll_bin
//		str_app_sll_oct
//		str_app_sll_dec
//		str_app_sll_cet
//
// purpose	Append to the first string (given with maximum available space)
//		a string of digits representing the signed long long value
//		converted in the named base.
//
//		Do append only if all the digits can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (signed long long) the number to convert
//		4 (int) minimum number of digits
//		5 (int) (char) padding character
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
#define str_app_sll_bin(t,s,v,m,p)	str_app_sll((t),(s),(v),(m),(p),2,(const char*)(NULL))
#define str_app_sll_oct(t,s,v,m,p)	str_app_sll((t),(s),(v),(m),(p),8,(const char*)(NULL))
#define str_app_sll_dec(t,s,v,m,p)	str_app_sll((t),(s),(v),(m),(p),10,(const char*)(NULL))
#define str_app_sll_cet(t,s,v,m,p)	str_app_sll((t),(s),(v),(m),(p),16,(const char*)(NULL))

//-----------------------------------------------------------------------------
// macros	str_app_ui_bin
//		str_app_ui_oct
//		str_app_ui_dec
//		str_app_ui_cet
//
// purpose	Append to the first string (given with maximum available space)
//		a string of digits representing the unsigned int value
//		converted in the named base.
//
//		Do append only if all the digits can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (unsigned int) the number to convert
//		4 (int) minimum number of digits
//		5 (int) (char) padding character
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
#define str_app_ui_bin(t,s,v,m,p)	str_app_ui((t),(s),(v),(m),(p),2,(const char*)(NULL))
#define str_app_ui_oct(t,s,v,m,p)	str_app_ui((t),(s),(v),(m),(p),8,(const char*)(NULL))
#define str_app_ui_dec(t,s,v,m,p)	str_app_ui((t),(s),(v),(m),(p),10,(const char*)(NULL))
#define str_app_ui_cet(t,s,v,m,p)	str_app_ui((t),(s),(v),(m),(p),16,(const char*)(NULL))

//-----------------------------------------------------------------------------
// function	str_app_ui_text
//
// purpose	Append to the first string (given with maximum available space)
//		a string of text representing the unsigned int value.
//
//		Do append only if all the text can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (unsigned int) the number to convert
//		4 (const char *) language id string, default "eng"
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
#define str_app_ui_text(t,s,v,l) (str_app_ul_text((t),(s),((unsigned long)(v)),(l)))

//-----------------------------------------------------------------------------
// macros	str_app_ul_bin
//		str_app_ul_oct
//		str_app_ul_dec
//		str_app_ul_cet
//
// purpose	Append to the first string (given with maximum available space)
//		a string of digits representing the unsigned long value
//		converted in the named base.
//
//		Do append only if all the digits can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (unsigned long) the number to convert
//		4 (int) minimum number of digits
//		5 (int) (char) padding character
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
#define str_app_ul_bin(t,s,v,m,p)	str_app_ul((t),(s),(v),(m),(p),2,(const char*)(NULL))
#define str_app_ul_oct(t,s,v,m,p)	str_app_ul((t),(s),(v),(m),(p),8,(const char*)(NULL))
#define str_app_ul_dec(t,s,v,m,p)	str_app_ul((t),(s),(v),(m),(p),10,(const char*)(NULL))
#define str_app_ul_cet(t,s,v,m,p)	str_app_ul((t),(s),(v),(m),(p),16,(const char*)(NULL))

//-----------------------------------------------------------------------------
// macros	str_app_ull_bin
//		str_app_ull_oct
//		str_app_ull_dec
//		str_app_ull_cet
//
// purpose	Append to the first string (given with maximum available space)
//		a string of digits representing the unsigned long long value
//		converted in the named base.
//
//		Do append only if all the digits can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (unsigned long long) the number to convert
//		4 (int) minimum number of digits
//		5 (int) (char) padding character
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
#define str_app_ull_bin(t,s,v,m,p)	str_app_ull((t),(s),(v),(m),(p),2,(const char*)(NULL))
#define str_app_ull_oct(t,s,v,m,p)	str_app_ull((t),(s),(v),(m),(p),8,(const char*)(NULL))
#define str_app_ull_dec(t,s,v,m,p)	str_app_ull((t),(s),(v),(m),(p),10,(const char*)(NULL))
#define str_app_ull_cet(t,s,v,m,p)	str_app_ull((t),(s),(v),(m),(p),16,(const char*)(NULL))

//-----------------------------------------------------------------------------
// function	str_cmp
//
// purpose	Compare two strings, returning a result that indicates the
//		collation order of the strings.
//
// arguments	1 (const char *) string 1
//		2 (const char *) string 2
//
// returns	(int) -1 : string 1 <  string 2
//		(int)  0 : string 1 == string 2
//		(int)  1 : string 1 >  string 2
//-----------------------------------------------------------------------------
#define str_cmp(a,b) strcmp((a),(b))

//-----------------------------------------------------------------------------
// function	str_cmp_lim
//
// purpose	Compare two strings, returning a result	that indicates the
//		collation order of the strings.
//
// arguments	1 (const char *) string 1
//		2 (const char *) string 2
//		3 (size_t) maximum length to compare, or ~0 to compare full
//
// returns	(int) -1 : string 1 <  string 2
//		(int)  0 : string 1 == string 2
//		(int)  1 : string 1 >  string 2
//-----------------------------------------------------------------------------
#define str_cmp_lim(a,b,l) strncmp((a),(b),(l))

//-----------------------------------------------------------------------------
// macro	str_cpy_arg
//
// purpose      Copy to the first string (given with maximum available space)
//              the contents of zero or more (multiple) strings.
//
//              Do copy only if all source strings can be copied, else make
//              target string empty.
//
// arguments    1 (char *) pointer to target string
//              2 (size_t) maximum space in target string
//              3..N (const char *) source strings to append
//
// returns      (size_t) new length of string, or ~0 for error
//-----------------------------------------------------------------------------
#define str_cpy_arg(t,l,a...) (str_empty((t)),str_app_arg((t),(l),a,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	str_cpy_arg_sep
//
// purpose      Copy to the first string (given with maximum available space)
//              the contents of zero or more (multiple) strings with a
//              separator string inserted between multiple strings.
//
//              Do copy only if all source strings can be copied, else make
//              target string empty.
//
// arguments    1 (char *) pointer to target string
//              2 (size_t) maximum space in target string
//              3 (const char *) separator string to insert
//              4..N (const char *) source strings to copy
//
// returns      (size_t) new length of string, or ~0 for error
//
// note         Contents of available space after the termination character
//              are destroyed if the copy fails due to insufficient space.
//              Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
#define str_cpy_arg_sep(t,l,s,a...) (str_empty((t)),str_app_arg_sep((t),(l),(s),a,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	str_cpy_arr
//
// purpose      Copy to the first string (given with maximum available space)
//              the contents of zero or more (multiple) strings provided as
//              an array of string pointers.
//
//              Do copy only if all source strings can be copied, else make
//              target string empty.
//
// arguments    1 (char *) pointer to target string
//              2 (size_t) maximum space in target string
//              3 (const char * const *) array of string pointers to copy
//
// returns      (size_t) new length of string, or ~0 for error
//
// note         Contents of available space after the termination character
//              are destroyed if the copy fails due to insufficient space.
//              Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
#define str_cpy_arr(t,l,s) (str_empty((t)),str_app_arr((t),(l),(s)))

//-----------------------------------------------------------------------------
// macro	str_cpy_arr_sep
//
// purpose      Copy to the first string (given with maximum available space)
//              the contents of zero or more (multiple) strings provided as
//              an array of string pointers with a separator string inserted
//              between multiple strings.
//
//              Do copy only if all source strings can be copied, else make
//              target string empty.
//
// arguments    1 (char *) pointer to target string
//              2 (size_t) maximum space in target string
//              3 (const char *) separator string to insert
//              4 (const char * const *) array of string pointers to copy
//
// returns      (size_t) new length of string, or ~0 for error
//
// note         Contents of available space after the termination character
//              are destroyed if the copy fails due to insufficient space.
//              Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
#define str_cpy_arr_sep(t,l,d,s) (str_empty((t)),str_app_arr_sep((t),(l),(d),(s)))

//-----------------------------------------------------------------------------
// macro	str_cpy_decode_base64
//
// purpose	Copy to given target the decoded string from a base 64
//		encoded input string.
//
// arguments    1 (char *) pointer to target string
//              2 (size_t) maximum space in target string
//		3 (const char *) source string
//
// returns	(size_t) final length if target string
//		(size_t) ~0 : error
//-----------------------------------------------------------------------------
#define str_cpy_decode_base64(t,l,s) (str_empty((t)),str_app_decode_base64((t),(l),(s)))

//-----------------------------------------------------------------------------
// macro	str_cpy_escape
//
// purpose      Append to the first string (given with maximum available space)
//		the contents of another string converted to an escaped format
//		to protect text files from control characters.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) source string to copy
//
// returns	(size_t) new length of string, or ~0 for error
//-----------------------------------------------------------------------------
#define str_cpy_escape(t,l,s) (str_empty((t)),str_app_escape((t),(l),(s)))

//-----------------------------------------------------------------------------
// macro	str_cpy_lim
//
// purpose      Copy to the first string (given with maximum available space)
//              the contents of another string limited by number of characters.
//
//              Do copy only if all source strings can be copied, else make
//              target string empty.
//
// arguments    1 (char *) pointer to target string
//              2 (size_t) maximum space in target string
//              3 (const char *) source string to copy
//              4 (size_t) limit length of source string
//
// returns      (size_t) new length of string, or ~0 for error
//-----------------------------------------------------------------------------
#define str_cpy_lim(t,m,s,l) (str_empty((t)),str_app_lim((t),(m),(s),(l)))

//-----------------------------------------------------------------------------
// macro	str_cpy_lower
//
// purpose	COpy to the first string (given with maximum available space)
//		the contents of another string converted to lower case.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) source string to copy
//
// returns	(size_t) new length of string, or ~0 for error
//-----------------------------------------------------------------------------
#define str_cpy_lower(t,l,s) (str_empty((t)),str_app_lower((t),(l),(s)))

//-----------------------------------------------------------------------------
// macro	str_cpy_part_ch
//
// purpose	Copy to the target string given with maximum available space
//		the first part found in the source string separated by a given
//		character, returning a pointer to the remainder of the source
//		string.
//
//		If the string to be copied does not fit the available space,
//		it remains truncated without indication.
//
//		Unlike str_split_ch(), this function does not modify the
//		source string.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) input string
//		4 (int) separator character
//
// returns	(char *) NULL : no item found or error
//		(char *) pointer to remainder of input string.
//-----------------------------------------------------------------------------
#define str_cpy_part_ch(t,l,s,x) (str_empty((t)),str_app_part_ch((t),(l),(s),(x)))

//-----------------------------------------------------------------------------
// macro	str_cpy_part_chs
//
// purpose	Copy to the target string given with maximum available space
//		the first item found in the source string separated by one of
//		a given set of characters, returning a pointer to the remainder
//		of the source string.
//
//		If the string to be copied does not fit the available space,
//		it remains truncated without indication.
//
//		Unlike str_split_chs(), this function does not modify the
//		source string.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) input string
//		4 (const char *) string of separator characters
//
// returns	(char *) NULL : no item found or error
//		(char *) pointer to remainder of source string.
//-----------------------------------------------------------------------------
#define str_cpy_part_chs(t,l,s,x) (str_empty((t)),str_app_part_chs((t),(l),(s),(x)))

//-----------------------------------------------------------------------------
// macro	str_cpy_part_space
//
// purpose	Copy to the target string given with maximum available space
//		the first item found in the source string separated by a space
//		character, returning a pointer to the remainder of the source.
//
//		If the string to be copied does not fit the available space,
//		it remains truncated without indication.
//
//		Unlike str_split_space(), this function does not modify the
//		source string.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) input string
//
// returns	(char *) NULL : no item found or error
//		(char *) pointer to remainder in input string.
//
// note		Spaces are determined according to the isspace() macro or
//		function in the C language.  Multiple space characters in a
//		group constitute a single delimiter, even if mixed.
//
// note		Any leading spaces will be removed.  If the caller needs to
//		know if there were any leading spaces, it should check the
//		original string first with isspace().
//-----------------------------------------------------------------------------
#define str_cpy_part_space(t,l,s) (str_empty((t)),str_app_part_space((t),(l),(s)))

//-----------------------------------------------------------------------------
// macro	str_cpy_part_str
//
// purpose	Copy to the target string given with maximum available space
//		the first item found in the source string separated by a given
//		string, returning a pointer to the remainder of the source.
//
//		If the string to be copied does not fit the available space,
//		it remains truncated without indication.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) input string
//		4 (const char *) separator string
//
// returns	(char *) NULL : no item found or error
//		(char *) pointer to remainder in input string.
//-----------------------------------------------------------------------------
#define str_cpy_part_str(t,l,s,x) (str_empty((t)),str_app_part_str((t),(l),(s),(x)))

//-----------------------------------------------------------------------------
// macro	str_cpy_rep
//
// purpose	Copy to the target string the contents of the source string,
//		with all occurrences of the pattern string replaced by the
//		replacement string.
//
//		If entire resultant string will not fit the available space,
//		then rollback everything leaving the original target empty.
//
// arguments	1 (char *) pointer to beginning of target string
//		2 (size_t) total maximum length of target string
//		3 (const char *) pattern string to be matched in source
//		4 (const char *) replacement string to replace each pattern
//		5 (const char *) source string to be appended to target
//
// returns	(size_t) final length of target string
//		(size_t) ~0 : error
//
// note		Contents of target string space will be overwritten if there
//		is insufficient space to complete the copy.  Nothing should
//		be stored there, anyway.
//-----------------------------------------------------------------------------
#define str_cpy_rep(t,l,p,r,s) (str_empty((t)),str_app_rep((t),(l),(p),(r),(s)))

//-----------------------------------------------------------------------------
// macro	str_cpy_si_text
//
// purpose      Copy to the first string (given with maximum available space)
//              a string of text representing the signed int value.
//
//              Do copy only if all source strings can be copied, else make
//              target string empty.
//
// arguments    1 (char *) pointer to target string
//              2 (size_t) maximum space in target string
//              3 (signed int) the number to convert
//              4 (const char *) language id string, default "eng"
//
// returns      (size_t) new length of string, or ~0 for error
//-----------------------------------------------------------------------------
#define str_cpy_si_text(t,s,v,l) str_cpy_sl_text((t),(s),((signed long)(v)),(l))

//-----------------------------------------------------------------------------
// macro	str_cpy_sl_text
//
// purpose      Copy to the first string (given with maximum available space)
//              a string of text representing the signed long value.
//
//              Do copy only if all source strings can be copied, else make
//              target string empty.
//
// arguments    1 (char *) pointer to target string
//              2 (size_t) maximum space in target string
//              3 (signed long) the number to convert
//              4 (const char *) language id string, default "eng"
//
// returns      (size_t) new length of string, or ~0 for error
//-----------------------------------------------------------------------------
#define str_cpy_sl_text(t,s,v,l) (str_empty((t)),str_app_sl_text((t),(s),(v),(l)))

//-----------------------------------------------------------------------------
// macro	str_cpy_sll_text
//
// purpose      Copy to the first string (given with maximum available space)
//              a string of text representing the signed long long value.
//
//              Do copy only if all source strings can be copied, else make
//              target string empty.
//
// arguments    1 (char *) pointer to target string
//              2 (size_t) maximum space in target string
//              3 (signed long long) the number to convert
//              4 (const char *) language id string, default "eng"
//
// returns      (size_t) new length of string, or ~0 for error
//-----------------------------------------------------------------------------
#define str_cpy_sll_text(t,s,v,l) (str_empty((t)),str_app_sll_text((t),(s),(v),(l)))

//-----------------------------------------------------------------------------
// macro	str_cpy_str
//
// purpose	Copy to the first string (given with maximum available space)
//		the contents of another string.
//
//		Do copy only if the entire source string can be copied.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) source string to copy
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		This function is different than Standard C strncpy().  The
//		str_cpy_str() function will always correctly terminate a
//		truncated string where as the strncpy() function will not.
//-----------------------------------------------------------------------------
#define str_cpy_str(t,l,s) (str_empty((t)),str_app_str((t),(l),(s)))

//-----------------------------------------------------------------------------
// macro	str_cpy_sub
//
// purpose	Copy to the target string the contents of a substring of the
//		source string between two specified points in the source.  The
//		substring bounding points are specified as a count of number
//		of occurrences of a pattern string, with one count and pattern
//		to determine the first point, and another count and pattern to
//		determine the second point.  If the pattern strings are not
//		given, the counts are the number of characters.  A positive
//		count steps forward from the start of the source string.  A
//		negative count steps backwards from the end of the source
//		string with -1 being the end of the source string.  If the
//		first point ends up after the second point, the substring is
//		then an empty string and an empty string is copied.
//
//		The use of -1 to refer to the end of the source string means
//		that -2 ends the substring one character or pattern string
//		before the end.  To remove M characters or pattern strings
//		from the end of a string, code (-(M))-1 in argument 5.
//
//		If the resultant string will not fit the available space,
//		then rollback everything leaving the original target string.
//
// arguments	1 (char *) pointer to beginning of target string
//		2 (size_t) total maximum length of target string
//		3 (int) count of steps to determine the first substring point
//		4 (const char *) pattern string to find first substring point
//		5 (int) count of steps to determine the second substring point
//		6 (const char *) pattern string to find second substring point
//		7 (const char *) source string to select substring from
//
// returns	(size_t) final length of target string
//		(size_t) ~0 : error
//-----------------------------------------------------------------------------
#define str_cpy_sub(t,l,bc,bp,ec,ep,s) (str_empty((t)),str_app_sub((t),(l),(bc),(bp),(ec),(ep),(s)))

//-----------------------------------------------------------------------------
// macro	str_cpy_ui_text
//
// purpose      Copy to the first string (given with maximum available space)
//              a string of text representing the unsigned int value.
//
//              Do copy only if all source strings can be copied, else make
//              target string empty.
//
// arguments    1 (char *) pointer to target string
//              2 (size_t) maximum space in target string
//              3 (unsigned int) the number to convert
//              4 (const char *) language id string, default "eng"
//
// returns      (size_t) new length of string, or ~0 for error
//-----------------------------------------------------------------------------
#define str_cpy_ui_text(t,s,v,l) str_cpy_ul_text((t),(s),((unsigned long)(v)),(l))

//-----------------------------------------------------------------------------
// macro	str_cpy_ul_text
//
// purpose      Copy to the first string (given with maximum available space)
//              a string of text representing the unsigned long value.
//
//              Do copy only if all source strings can be copied, else make
//              target string empty.
//
// arguments    1 (char *) pointer to target string
//              2 (size_t) maximum space in target string
//              3 (unsigned long) the number to convert
//              4 (const char *) language id string, default "eng"
//
// returns      (size_t) new length of string, or ~0 for error
//-----------------------------------------------------------------------------
#define str_cpy_ul_text(t,s,v,l) (str_empty((t)),str_app_ul_text((t),(s),(v),(l)))

//-----------------------------------------------------------------------------
// macro	str_cpy_ull_text
//
// purpose      Copy to the first string (given with maximum available space)
//              a string of text representing the unsigned long long value.
//
//              Do copy only if all source strings can be copied, else make
//              target string empty.
//
// arguments    1 (char *) pointer to target string
//              2 (size_t) maximum space in target string
//              3 (unsigned long long) the number to convert
//              4 (const char *) language id string, default "en_US"
//
// returns      (size_t) new length of string, or ~0 for error
//-----------------------------------------------------------------------------
#define str_cpy_ull_text(t,s,v,l) (str_empty((t)),str_app_ull_text((t),(s),(v),(l)))

//-----------------------------------------------------------------------------
// macro	str_cpy_upper
//
// purpose	Copy to the first string (given with maximum available space)
//		the contents of another string converted to upper case.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) source string to append
//
// returns	(size_t) new length of string, or ~0 for error
//-----------------------------------------------------------------------------
#define str_cpy_upper(t,l,s) (str_empty((t)),str_app_upper((t),(l),(s)))

//-----------------------------------------------------------------------------
// macro	str_cpy_url
//
// purpose      Copy to the first string (given with maximum available space)
//              a URL formed from several component strings.
//
// arguments    1 (char *) pointer to target string
//              2 (size_t) maximum space in target string
//              3 (const char *) protocol, "http", "ftp"
//              4 (const char *) username
//              5 (const char *) password
//              6 (const char *) hostname
//              7 (const char *) portnum
//              8 (const char *) fullpath
//              9 (const char *) querystring
//
// returns      (size_t) length of string, or ~0 for error
//-----------------------------------------------------------------------------
#define str_cpy_url(t,l,a,b,c,d,e,f,g) (str_empty((t)),str_app_url((t),(l),(a),(b),(c),(d),(e),(f),(g)))

//-----------------------------------------------------------------------------
// macro	str_cpy_var
//
// purpose      Copy to the first string (given with maximum available space)
//              the contents of zero or more (multiple) strings provided as
//              a variable argument list passed in one argument.
//
//              Do copy only if all source strings can be copied, else make
//              target string empty.
//
// arguments    1 (char *) pointer to target string
//              2 (size_t) maximum space in target string
//              3 (va_list) source strings to copy
//
// returns      (size_t) new length of string, or ~0 for error
//
// note         Contents of available space after the termination character
//              are destroyed if the copy fails due to insufficient space.
//              Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
#define str_cpy_var(t,l,s) (str_empty((t)),str_app_var((t),(l),(s)))

//-----------------------------------------------------------------------------
// macro	str_cpy_var_sep
//
// purpose      Copy to the first string (given with maximum available space)
//              the contents of zero or more (multiple) strings provided as
//              a variable argument list passed in one argument with a
//              separator string inserted between multiple strings.
//
//              Do copy only if all source strings can be copied, else make
//              target string empty.
//
// arguments    1 (char *) pointer to target string
//              2 (size_t) maximum space in target string
//              3 (const char *) separator string to insert
//              4 (va_list) source strings to copy
//
// returns      (size_t) new length of string, or ~0 for error
//
// note         Contents of available space after the termination character
//              are destroyed if the copy fails due to insufficient space.
//              Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
#define str_cpy_var_sep(t,l,d,s) (str_empty((t)),str_app_var_sep((t),(l),(d),(s)))

//-----------------------------------------------------------------------------
// macro	str_dup_arg
//
// purpose	Duplicate into allocated space the contents of zero or more
//		(multiple) strings.
//
// arguments	1..N (const char *) source strings to duplicate
//
// returns	(char *) new allocated duplicate string, caller must free
//		(char *) NULL : error allocating string or other error
//
// warning	If other threads modify any source strings during duplication,
//		then results are unpredictable.
//-----------------------------------------------------------------------------
#define str_dup_arg(a...) ((str_dup_arg)(a,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	str_dup_arg_lim
//
// purpose	Duplicate into allocated space the contents of zero or more
//		(multiple) strings limited by number of characters.
//
// arguments	1..N-1 (const char *) source strings to duplicate
//		2..N (size_t) limit lengths of source strings
//
// returns	(char *) new allocated duplicate string, caller must free
//		(char *) NULL : error allocating string or other error
//
// warning	If other threads modify any source strings during duplication,
//		then results are unpredictable.
//-----------------------------------------------------------------------------
#define str_dup_arg_lim(a...) ((str_dup_arg_lim)(a,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	str_dup_arg_lim_sep
//
// purpose	Duplicate into allocated space the contents of zero or more
//		(multiple) strings with a separator string inserted between
//		multiple strings limited by number of characters.
//
// arguments	1 (const char *) separator string to insert
//		2 (size_t) limit length of separator string
//		3..N-1 (const char *) source strings to duplicate
//		4..N (size_t) limit lengths of source strings
//
// returns	(char *) new allocated duplicate string, caller must free
//		(char *) NULL : error allocating string or other error
//
// warning	If other threads modify any source strings during duplication,
//		then results are unpredictable.
//-----------------------------------------------------------------------------
#define str_dup_arg_lim_sep(sp,sl,a...) ((str_dup_arg_lim_sep)((sp),(sl),a,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	str_dup_arg_sep
//
// purpose	Duplicate into allocated space the contents of zero or more
//		(multiple) strings with a separator string inserted between
//		multiple strings.
//
// arguments	1 (const char *) separator string to insert
//		1..N (const char *) source strings to duplicate
//
// returns	(char *) new allocated duplicate string, caller must free
//		(char *) NULL : error allocating string or other error
//
// warning	If other threads modify any source strings during duplication,
//		then results are unpredictable.
//-----------------------------------------------------------------------------
#define str_dup_arg_sep(sp,a...) ((str_dup_arg_sep)((sp),a,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// function	str_empty
//
// purpose	Make a string target empty.
//
// arguments	1 (char *) pointer to target string
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define str_empty(s) (*(s)=0)

//-----------------------------------------------------------------------------
// macro	str_eq
//
// purpose	Compare two strings for equality.
//
// arguments	1 (const char *) string 1
//		2 (const char *) string 2
//
// returns	(int) 0 : strings are not equal
//		(int) 1 : strings are equal
//-----------------------------------------------------------------------------
#define str_eq(a,b) (str_eq_opt((a),(b),~(size_t)0,0))

//-----------------------------------------------------------------------------
// macro	str_eq_nocase
//
// purpose	Compare two strings for equality disregarding the case of
//		alphabetic characters.
//
// arguments	1 (const char *) string 1
//		2 (const char *) string 2
//
// returns	(int) 0 : strings are not equal
//		(int) 1 : strings are equal
//-----------------------------------------------------------------------------
#define str_eq_nocase(a,b) (str_eq_opt((a),(b),~(size_t)0,STR_EQ_NOCASE))

//-----------------------------------------------------------------------------
// macro	str_eq_space
//
// purpose	Compare two strings for equality with whitespace clusters
//		considered to be one space.
//
// arguments	1 (const char *) string 1
//		2 (const char *) string 2
//
// returns	(int) 0 : strings are not equal
//		(int) 1 : strings are equal
//-----------------------------------------------------------------------------
#define str_eq_space(a,b) (str_eq_opt((a),(b),~(size_t)0,STR_EQ_SPACE))

//-----------------------------------------------------------------------------
// macro	str_eq_nocase_space
//
// purpose	Compare two strings for equality disregarding the case of
//		alphabetic characters and with whitespace clusters considered
//		to be one space.
//
// arguments	1 (const char *) string 1
//		2 (const char *) string 2
//
// returns	(int) 0 : strings are not equal
//		(int) 1 : strings are equal
//-----------------------------------------------------------------------------
#define str_eq_nocase_space(a,b) (str_eq_opt((a),(b),~(size_t)0,STR_EQ_NOCASE|STR_EQ_SPACE))

//-----------------------------------------------------------------------------
// macro	str_eq_numeric
//
// purpose	Compare two strings for equality with numeric digits compared
//		based on their value.
//
// arguments	1 (const char *) string 1
//		2 (const char *) string 2
//
// returns	(int) 0 : strings are not equal
//		(int) 1 : strings are equal
//-----------------------------------------------------------------------------
#define str_eq_numeric(a,b) (str_eq_opt((a),(b),~(size_t)0,STR_EQ_NUMERIC))

//-----------------------------------------------------------------------------
// macro	str_eq_nocase_numeric
//
// purpose	Compare two strings for equality disregarding the case of
//		alphabetic characters and with numeric digits compared based
//		on their value.
//
// arguments	1 (const char *) string 1
//		2 (const char *) string 2
//
// returns	(int) 0 : strings are not equal
//		(int) 1 : strings are equal
//-----------------------------------------------------------------------------
#define str_eq_nocase_numeric(a,b) (str_eq_opt((a),(b),~(size_t)0,STR_EQ_NOCASE|STR_EQ_NUMERIC))

//-----------------------------------------------------------------------------
// macro	str_eq_space_numeric
//
// purpose	Compare two strings for equality with whitespace clusters
//		considered to be one space and with numeric digits compared
//		based on their value.
//
// arguments	1 (const char *) string 1
//		2 (const char *) string 2
//
// returns	(int) 0 : strings are not equal
//		(int) 1 : strings are equal
//-----------------------------------------------------------------------------
#define str_eq_space_numeric(a,b) (str_eq_opt((a),(b),~(size_t)0,STR_EQ_SPACE|STR_EQ_NUMERIC))

//-----------------------------------------------------------------------------
// macro	str_eq_nocase_space_numeric
//
// purpose	Compare two strings for equality disregarding the case of
//		alphabetic characters and with numeric digits compared based
//		on their value with numeric digits compared based on their
//		value.
//
// arguments	1 (const char *) string 1
//		2 (const char *) string 2
//
// returns	(int) 0 : strings are not equal
//		(int) 1 : strings are equal
//-----------------------------------------------------------------------------
#define str_eq_nocase_space_numeric(a,b) (str_eq_opt((a),(b),~(size_t)0,STR_EQ_NOCASE|STR_EQ_SPACE|STR_EQ_NUMERIC))

//-----------------------------------------------------------------------------
// function	str_equal
//
// purpose	Compare two strings for equality, returning true or false.
//
// arguments	1 (const char *) string 1
//		2 (const char *) string 2
//
// returns	(int) 0 : strings are not equal
//		(int) 1 : strings are equal
//-----------------------------------------------------------------------------
#define str_equal(a,b) ((strcmp((a),(b)))==0)

//-----------------------------------------------------------------------------
// macro	str_expr_to_d
//
// purpose	Convert a string containing a numeric expression into a
//		double.
//
// arguments	1 (const char *) pointer to string to convert
//		2 (char * *) pointer to where to store end pointer
//
// returns	(double) converted result
//
// note		Numeric values that exceed the maximums for this data type
//		will result in undefined values being returned.
//-----------------------------------------------------------------------------
#define str_expr_to_d(s,e) ((str_expr_to_d)((pow),(s),(e)))

//-----------------------------------------------------------------------------
// function	str_expr_to_ld
//
// purpose	Convert a string containing a numeric expression into a
//		long double.
//
// arguments	1 (const char *) pointer to string to convert
//		2 (char * *) pointer to string to store end pointer
//
// returns	(long double) converted result
//
// note		Numeric values that exceed the maximums for this data type
//		will result in undefined values being returned.
//-----------------------------------------------------------------------------
#define str_expr_to_ld(s,e) ((str_expr_to_ld)((powl),(s),(e)))

//-----------------------------------------------------------------------------
// macro	str_find_arg
//
// purpose      Search arguments 2..N for a string that matches argument 1,
//		returning 0 for not found, and 1..N-1 for which argument is
//		found.  The comparison IS case sensitive.
//
// arguments    1 (const char *) string to search for
//              2..N (const char *) strings to search
//
// returns	(int) == -1 : error
//		(int) ==  0 : not found
//		(int>  >  0 : found string at argument N-1
//-----------------------------------------------------------------------------
#define str_find_arg(s,a...) ((str_find_arg)((s),a,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	str_find_nocase_arg
//
// purpose      Search arguments 2..N for a string that matches argument 1,
//              returning 0 for not found, and 1..N-1 for which argument is
//              found.  The comparison is NOT case sensitive.
//
// arguments    1 (const char *) string to search for
//              2..N (const char *) strings to search
//
// returns      (int) == -1 : error
//              (int) ==  0 : not found
//              (int>  >  0 : found string at argument N-1
//-----------------------------------------------------------------------------
#define str_find_nocase_arg(s,a...) ((str_find_nocase_arg)((s),a,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	str_last_part
//
// purpose      Find the last part of a string based on a single character
//		separator.
//
// arguments    1 (char *) pointer to string to access
//		2 (int) separator character
//
// returns      (char *) pointer to last part within original string
//-----------------------------------------------------------------------------
#define str_last_part(s,c) ({							\
	__typeof__(s) libh__ptr;						\
	__typeof__(s) libh__last;						\
	__typeof__(c) libh__ch;							\
	libh__ch = (c);								\
	libh__last = libh__ptr = (s);						\
	while ( * libh__ptr ) {							\
		if ( * libh__ptr ++ == libh__ch ) libh__last = libh__ptr;	\
	}									\
	libh__last;								\
})

//-----------------------------------------------------------------------------
// function	str_metric_to_si
//
// purpose	Convert a string containing a number followed by an optional
//		metric suffix into an signed integer.
//
// arguments	1 (const char *) pointer to string to convert
//		2 (char * *) pointer to string to store end pointer
//		3 (int) numeric base value, 2 to 36
//		4 (signed int) metric scale for lower case suffixes
//		5 (signed int) metric scale for upper case suffixes
//
// returns	(signed int) converted result
//
// note		Numeric values that exceed the maximums for this data type
//		will result in undefined values being returned.
//
// suffixes	'k' or 'K' multiplies the preceeding number by scale**1
//		'm' or 'M' multiplies the preceeding number by scale**2
//		'g' or 'G' multiplies the preceeding number by scale**3
//		't' or 'T' multiplies the preceeding number by scale**4
//		'p' or 'P' multiplies the preceeding number by scale**5
//		'e' or 'E' multiplies the preceeding number by scale**6
//
// warning	Metric suffixes are not recognized if the base setting
//		uses that character for its symbol set.
//-----------------------------------------------------------------------------
#define str_metric_to_si(s,e,b,l,u) str_metric_to_sl((s),(e),(b),((signed long)(l)),((signed long)(u)))
//-----------------------------------------------------------------------------
// function	str_metric_to_ui
//
// purpose	Convert a string containing a number followed by an optional
//		metric suffix into an unsigned integer.
//
// arguments	1 (const char *) pointer to string to convert
//		2 (char * *) pointer to string to store end pointer
//		3 (int) numeric base value, 2 to 36
//		4 (unsigned int) metric scale for lower case suffixes
//		5 (unsigned int) metric scale for upper case suffixes
//
// returns	(unsigned int) converted result
//
// note		Numeric values that exceed the maximums for this data type
//		will result in undefined values being returned.
//
// suffixes	'k' or 'K' multiplies the preceeding number by scale**1
//		'm' or 'M' multiplies the preceeding number by scale**2
//		'g' or 'G' multiplies the preceeding number by scale**3
//		't' or 'T' multiplies the preceeding number by scale**4
//		'p' or 'P' multiplies the preceeding number by scale**5
//		'e' or 'E' multiplies the preceeding number by scale**6
//
// warning	Metric suffixes are not recognized if the base setting
//		uses that character for its symbol set.
//-----------------------------------------------------------------------------
#define str_metric_to_ui(s,e,b,l,u) str_metric_to_ul((s),(e),(b),((unsigned long)(l)),((unsigned long)(u)))
//-----------------------------------------------------------------------------
// macro	str_skip_hspace
//
// purpose	Skip forward over horizontal space characters until a another
//		character (or string termination) is encountered, returning a
//		pointer to that location.
//
// arguments	1 ([const] scalar *) string to skip in
//
// returns	([const] scalar *) pointer to first non-space character
//
// scalar	Any [un]signed scalar type:  char, short, int, long, long long
//-----------------------------------------------------------------------------
#define str_skip_hspace(s) ({							\
	__typeof__( s ) __str;							\
	__str = ( s );								\
	while ( * __str != ' ' && * __str != '\t' ) ++ __str;			\
	__str;									\
})

//-----------------------------------------------------------------------------
// macro	str_skip_space
//
// purpose	Skip forward over space characters until a another character
//		(or string termination) is encountered, returning a pointer to
//		that location.
//
// arguments	1 ([const] scalar *) string to skip in
//
// returns	([const] scalar *) pointer to first non-space character
//
// scalar	Any [un]signed scalar type:  char, short, int, long, long long
//-----------------------------------------------------------------------------
#define str_skip_space(s) ({							\
	__typeof__( s ) __str;							\
	__str = ( s );								\
	while ( ! isspace( * _str ) ) ++ __str;					\
	__str;									\
})

//-----------------------------------------------------------------------------
// macro	str_tail
//
// purpose	Select N components of the tail of a string and return a
//		pointer with the result.
//
// arguments	1 (const char *) string to extract tail from
//		2 (int) the delimiter character code
//		3 (int) the number of components to select
//
// returns	(char *) returned pointer
//-----------------------------------------------------------------------------
#define str_tail(s,d,n) ({							\
	__typeof__(*(s)) *libh__str;						\
	__typeof__(*(s)) *libh__ptr;						\
	int libh__del;								\
	int libh__num;								\
	libh__ptr = libh__str = (__typeof__(libh__str)) (s);			\
	while ( * libh__ptr ) ++ libh__ptr;					\
	libh__del = (d);							\
	libh__num = (n);							\
	while ( -- libh__ptr >= libh__str && ( * libh__ptr != libh__del || -- libh__num > 0 ) );	\
	libh__ptr + 1;								\
})

//-----------------------------------------------------------------------------
// macro	str_tail_one
//
// purpose	Select one component of the tail of a string and return a
//		pointer with the result.
//
// arguments	1 (const char *) string to extract tail from
//		2 (int) the delimiter character code
//
// returns	(char *) returned pointer
//-----------------------------------------------------------------------------
#define str_tail_one(s,d) ({							\
	__typeof__(*(s)) *libh__str;						\
	__typeof__(*(s)) *libh__ptr;						\
	int libh__del;								\
	libh__ptr = libh__str = (__typeof__(libh__str)) (s);			\
	while ( * libh__ptr ) ++ libh__ptr;					\
	libh__del = (d);							\
	while ( -- libh__ptr >= libh__str && * libh__ptr != libh__del );	\
	libh__ptr + 1;								\
})

//-----------------------------------------------------------------------------
// macro	strpe
//
// purpose	Create a strpe value consisting of a string pointer and end
//		pointer.
//
// arguments	1 (char *) string pointer
//		2 (char *) string end pointer
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpe(p,e) ({								\
	struct strpe_s libh__spe;						\
	libh__spe.ptr = (p);							\
	libh__spe.end = (e);							\
	libh__spe;								\
})

//-----------------------------------------------------------------------------
// macro	strpe_null
//
// purpose	Make a null value strpe.
//-----------------------------------------------------------------------------
#define strpe_null strpe((NULL),(NULL))

//-----------------------------------------------------------------------------
// macro	strpe_ptr
//
// purpose	Extract the string pointer from a strpe value.
//-----------------------------------------------------------------------------
#define strpe_ptr(s) ((s).ptr)

//-----------------------------------------------------------------------------
// macro	strpe_end
//
// purpose	Extract the string end pointer from a strpe value.
//-----------------------------------------------------------------------------
#define strpe_end(s) ((s).end)

//-----------------------------------------------------------------------------
// macro	strpe_len
//
// purpose	Extract the string length from a strpe value.
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpe_len(s) ({								\
	struct strpe_s libh__spe;						\
	libh__spe = (s);							\
	libh__spe.end - libh__spe.ptr;						\
})

//-----------------------------------------------------------------------------
// macro	strpe_strpl
//
// purpose	Convert a strpe value to a strpl value.
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpe_strpl(s) ({							\
	struct strpe_s libh__strpe;						\
	struct strpl_s libh__strpl;						\
	libh__strpl.ptr = libh__strpe.ptr;					\
	libh__strpl.len = libh__strpe.end - libh__strpe.ptr;			\
	libh__strpl;								\
})

//-----------------------------------------------------------------------------
// macro        strpea
//
// purpose      Create a strpea_s value consisting of a string pointer, end
//		pointer, and allocation end pointer.
//
// arguments    1 (char *) string pointer
//              2 (char *) string end pointer
//		3 (char *) allocation end pointer
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpea(p,e,q) ({							\
	struct strpea_s libh__strpea;						\
	libh__strpea.ptr = (p);							\
	libh__strpea.end = (e);							\
	libh__strpea.aend = (q);						\
	libh__strpea;								\
})

//-----------------------------------------------------------------------------
// macro	strpea_null
//
// purpose	Make a null value strpea.
//-----------------------------------------------------------------------------
#define strpea_null strpea((NULL),(NULL),(NULL))

//-----------------------------------------------------------------------------
// macro        strpea_ptr
//
// purpose      Extract the string pointer from a strpea_s value.
//-----------------------------------------------------------------------------
#define strpea_ptr(s) ((s).ptr)

//-----------------------------------------------------------------------------
// macro        strpea_end
//
// purpose      Extract the end pointer from strpea_s value.
//-----------------------------------------------------------------------------
#define strpea_end(s) ((s).end)

//-----------------------------------------------------------------------------
// macro        strpea_aend
//
// purpose      Extract the allocation end pointer from strpea_s value.
//-----------------------------------------------------------------------------
#define strpea_aend(s) ((s).aend)

//-----------------------------------------------------------------------------
// macro        strpea_len
//
// purpose      Extract the string length from a strpea_s value.
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpea_len(s) ({							\
	struct strpea_s libh__strpea;						\
	libh__strpea = (s);							\
	libh__strpea.end - libh__strpea.ptr;					\
})

//-----------------------------------------------------------------------------
// macro        strpea_alen
//
// purpose      Extract the allocation length from a strpea_s value.
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpea_alen(s) ({							\
	struct strpea_s libh__strpea;						\
	libh__strpea = (s);							\
	libh__strpea.aend - libh__strpea.ptr;					\
})

//-----------------------------------------------------------------------------
// macro	strpea_strpe
//
// purpose	Convert a strpea value to a strpe value.
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpea_strpe(s) ({							\
	struct strpea_s libh__strpea;						\
	struct strpe_s libh__strpe;						\
	libh__strpe.ptr = libh__strpea.ptr;					\
	libh__strpe.end = libh__strpea.end;					\
	libh__strpe;								\
})

//-----------------------------------------------------------------------------
// macro	strpea_strpl
//
// purpose	Convert a strpea value to a strpl value.
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpea_strpl(s) ({							\
	struct strpea_s libh__strpea;						\
	struct strpl_s libh__strpl;						\
	libh__strpl.ptr = libh__strpea.ptr;					\
	libh__strpl.len = libh__strpea.end - libh__strpea.ptr;			\
	libh__strpl;								\
})

//-----------------------------------------------------------------------------
// macro	strpea_strpla
//
// purpose	Convert a strpea value to a strpla value.
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpea_strpla(s) ({							\
	struct strpea_s libh__strpea;						\
	struct strpla_s libh__strpla;						\
	libh__strpla.ptr = libh__strpea.ptr;					\
	libh__strpla.len = libh__strpea.end - libh__strpea.ptr;			\
	libh__strpla.alen = libh__strpea.aend - libh__strpea.ptr;		\
	libh__strpla;								\
})

//-----------------------------------------------------------------------------
// macro	strpea_trim
//
// purpose	Trim the memory allocation of a pointed to strpea struct to
//		the size of the current string length, plus one for space to
//		append a null termination character.
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpea_trim(p) ({							\
	struct strpea_s *libh__strpea_p;					\
	char *libh__ptr;							\
	libh__strpea_p = (p);							\
	libh__ptr = realloc( libh__strpea_p->ptr, libh__strpea_p->len + 1 );	\
	if ( libh__ptr ) {							\
		libh__strpea_p->ptr = libh__ptr;				\
		libh__strpea_p->aend = libh__ptr + libh__strpea_p->len + 1;	\
	}									\
})

//-----------------------------------------------------------------------------
// macro	strpea_new
//
// purpose	Allocate new string space of the specified size and return a
//		strpea value using this allocation.
//
// note		If the returned value is assigned to a struct strpea, that
//		struct strpea must not have any existing allocation of string
//		space or this will create a memory leak.  This is intended for
//		first time initialization.
//
// note		If memory allocation fails, or the requested size is 0, the
//		returned value will have a NULL pointer and zero length.
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpea_new(s) ({							\
	struct strpea_s libh__strpea;						\
	size_t libh__alen;							\
	libh__alen = (s);							\
	libh__strpea.ptr = libh__alen ? malloc( libh__alen ) : (NULL);		\
	libh__strpea.end = libh__strpea.ptr;					\
	libh__strpea.aend = libh__strpea.ptr;					\
	if ( libh__strpea.aend ) libh__strpea.aend += libh__alen;		\
	libh__strpea;								\
})

//-----------------------------------------------------------------------------
// macro	strpea_realloc
//
// purpose	Reallocate memory for the pointed to strpea struct to the
//		specified size, if the current string will fit.  If the string
//		will not fit, make no changes.
//
// yields	0 - string fits in size, reallocation performed
//		1 - string does not fit, reallocation not performed
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpea_realloc(p,s) ({							\
	struct strpea_s *libh__strpea_p;					\
	char *libh__ptr;							\
	size_t libh__size;							\
	libh__strpea_p = (p);							\
	libh__size = (s);							\
	libh__strpea_p->end - libh__strpea_p->ptr >= libh__size ? ({		\
		libh__ptr = realloc( libh__strpea_p->ptr, libh__size );		\
		if ( libh__ptr || ! libh__size ) {				\
			libh__strpea_p->ptr = libh__ptr;			\
			libh__strpea_p->aend = libh__ptr + libh__size;		\
		}								\
	0; }) : 1;								\
})

//-----------------------------------------------------------------------------
// macro	strpea_free
//
// purpose	Free memory allocated for string space in the specified struct
//		strpea.
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpea_free(p) ({							\
	struct strpea_s *libh__strpea_p;					\
	libh__strpea_p = (p);							\
	if ( libh__strpea_p->ptr && libh__strpea_p->aend ) {			\
		free( libh__strpea_p->ptr );					\
		libh__strpea_p->ptr = (NULL);					\
		libh__strpea_p->end = (NULL);					\
		libh__strpea_p->aend = (NULL);					\
	}									\
	libh__strpea_p;								\
})

//-----------------------------------------------------------------------------
// macro	strpl
//
// purpose	Create a strpl value consisting of a string pointer and
//		length.
//
// arguments	1 (char *) string pointer
//		2 (size_t) string length
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpl(p,l) ({								\
	struct strpl_s libh__spl;						\
	libh__spl.ptr = (p);							\
	libh__spl.len = (l);							\
	libh__spl;								\
})

//-----------------------------------------------------------------------------
// macro	strpl_null
//
// purpose	Make a null value strpl.
//-----------------------------------------------------------------------------
#define strpl_null strpl((NULL),0)

//-----------------------------------------------------------------------------
// macro	strpl_ptr
//
// purpose	Extract the string pointer from a strpl value.
//-----------------------------------------------------------------------------
#define strpl_ptr(s) ((s).ptr)

//-----------------------------------------------------------------------------
// macro	strpl_len
//
// purpose	Extract the string length from a strpl value.
//-----------------------------------------------------------------------------
#define strpl_len(s) ((s).len)

//-----------------------------------------------------------------------------
// macro	strpl_end
//
// purpose	Extract the string end pointer from a strpl value.
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpl_end(s) ({								\
	struct strpl_s libh__spl;						\
	libh__spl = (s);							\
	libh__spl.len + libh__spl.ptr;						\
})

//-----------------------------------------------------------------------------
// macro	strpl_strpe
//
// purpose	Convert a strpl value to a strpe value.
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpl_strpe(s) ({							\
	struct strpl_s libh__strpl;						\
	struct strpe_s libh__strpe;						\
	libh__strpe.ptr = libh__strpl.ptr;					\
	libh__strpe.end = libh__strpl.len + libh__strpl.ptr;			\
	libh__strpe;								\
})

//-----------------------------------------------------------------------------
// macro        strpla
//
// purpose      Create a struct strpla_s value consisting of a string pointer,
//              length, and allocation length.
//
// arguments    1 (char *) string pointer
//              2 (size_t) string length
//		3 (size_t) allocation length
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpla(p,l,a) ({							\
	struct strpla_s libh__spls;						\
	libh__spls.ptr = (p);							\
	libh__spls.len = (l);							\
	libh__spls.alen = (a);							\
	libh__spls;								\
})

//-----------------------------------------------------------------------------
// macro	strpla_null
//
// purpose	Make a null value strpla.
//-----------------------------------------------------------------------------
#define strpla_null strpla((NULL),0,0)

//-----------------------------------------------------------------------------
// macro        strpla_ptr
//
// purpose      Extract the string pointer from a strpla_s value.
//-----------------------------------------------------------------------------
#define strpla_ptr(s) ((s).ptr)

//-----------------------------------------------------------------------------
// macro        strpla_len
//
// purpose      Extract the string length from a strpla_s value.
//-----------------------------------------------------------------------------
#define strpla_len(s) ((s).len)

//-----------------------------------------------------------------------------
// macro        strpla_alen
//
// purpose      Extract the allocation length from a strpla_s value.
//-----------------------------------------------------------------------------
#define strpla_alen(s) ((s).alen)

//-----------------------------------------------------------------------------
// macro        strpla_end
//
// purpose      Extract the string end pointer from a strpla_s value.
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpla_end(s) ({							\
	struct strpla_s libh__spls;						\
	libh__spls = (s);							\
	libh__spls.len + libh__spls.ptr;					\
})

//-----------------------------------------------------------------------------
// macro        strpla_aend
//
// purpose      Extract the allocation end pointer from a struct strpl_s.
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpla_aend(s) ({							\
	struct strpla_s libh__spls;						\
	libh__spls = (s);							\
	libh__spls.alen + libh__spls.ptr;					\
})

//-----------------------------------------------------------------------------
// macro	strpla_strpl
//
// purpose	Convert a strpla value to a strpl value.
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpla_strpl(s) ({							\
	struct strpla_s libh__strpla;						\
	struct strpl_s libh__strpl;						\
	libh__strpl.ptr = libh__strpla.ptr;					\
	libh__strpl.len = libh__strpla.len;					\
	libh__strpl;								\
})

//-----------------------------------------------------------------------------
// macro	strpla_strpe
//
// purpose	Convert a strpla value to a strpe value.
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpla_strpe(s) ({							\
	struct strpla_s libh__strpla;						\
	struct strpe_s libh__strpe;						\
	libh__strpe.ptr = libh__strpla.ptr;					\
	libh__strpe.end = libh__strpla.len + libh__strpla.ptr;			\
	libh__strpe;								\
})

//-----------------------------------------------------------------------------
// macro	strpla_strpea
//
// purpose	Convert a strpla value to a strpea value.
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpla_strpea(s) ({							\
	struct strpla_s libh__strpla;						\
	struct strpea_s libh__strpea;						\
	libh__strpea.ptr = libh__strpla.ptr;					\
	libh__strpea.end = libh__strpla.len + libh__strpla.ptr;			\
	libh__strpea.aend = libh__strpla.alen + libh__strpla.ptr;		\
	libh__strpea;								\
})

//-----------------------------------------------------------------------------
// macro	strpla_trim
//
// purpose	Trim the memory allocation of a pointed to strpla struct to
//		the size of the current string length, plus one for space to
//		append a null termination character.
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpla_trim(p) ({							\
	struct strpla_s *libh__speap;						\
	char *libh__ptr;							\
	libh__speap = (p);							\
	libh__ptr = realloc( libh__speap->ptr, libh__speap->len + 1 );		\
	if ( libh__ptr ) {							\
		libh__speap->ptr = libh__ptr;					\
		libh__speap->alen = libh__speap->len + 1;			\
	}									\
})

//-----------------------------------------------------------------------------
// macro	strpla_new
//
// purpose	Allocate new string space of the specified size and return a
//		strpla value using this allocation.
//
// note		If the returned value is assigned to a struct strpla, that
//		struct strpla must not have any existing allocation of string
//		space or this will create a memory leak.  This is intended for
//		first time initialization.
//
// note		If memory allocation fails, or the requested size is 0, the
//		returned value will have a NULL pointer and zero length.
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpla_new(s) ({							\
	struct strpla_s libh__spea;						\
	size_t libh__alen;							\
	libh__alen = (s);							\
	libh__spea.ptr = libh__alen ? malloc( libh__alen ) : (NULL);		\
	libh__spea.alen = libh__spea.ptr ? libh__alen : 0;			\
	libh__spea.len = 0;							\
	libh__spea;								\
})

//-----------------------------------------------------------------------------
// macro	strpla_realloc
//
// purpose	Reallocate memory for the pointed to strpla struct to the
//		specified size, if the current string will fit.  If the string
//		will not fit, make no changes.
//
// yields	0 - string fits in size, reallocation performed
//		1 - string does not fit, reallocation not performed
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpla_realloc(p,s) ({							\
	struct strpla_s *libh__speap;						\
	char *libh__ptr;							\
	size_t libh__size;							\
	libh__speap = (p);							\
	libh__size = (s);							\
	libh__speap->len >= libh__size ? ({					\
		libh__ptr = realloc( libh__speap->ptr, libh__size );		\
		if ( libh__ptr || ! libh__size ) {				\
			libh__speap->ptr = libh__ptr;				\
			libh__speap->alen = libh__ptr + libh__size;		\
		}								\
	0; }) : 1;								\
})

//-----------------------------------------------------------------------------
// macro	strpla_free
//
// purpose	Free memory allocated for string space in the specified struct
//		strpla.
//
// note		This macro is "multiple evaluation safe".  All expressions
//		passed as arguments will only be evaluated once.
//-----------------------------------------------------------------------------
#define strpla_free(p) ({							\
	struct strpla_s *libh__speap;						\
	libh__speap = (p);							\
	if ( libh__speap->ptr && libh__speap->alen ) {				\
		free( libh__speap->ptr );					\
		libh__speap->ptr = (NULL);					\
		libh__speap->len = 0;						\
		libh__speap->alen = 0;						\
	}									\
	libh__speap;								\
})

//-----------------------------------------------------------------------------
// function	mem_cmp
// alias	mem_compare
//
// purpose	Compare two strings, which are specified by pointer and length,
//		returning the collation order difference as -1, 0, or 1.  If
//		one string ends before a difference is found, then the shorter
//		string is considered to be lower in sequence than the longer
//		string.
//
// arguments	1 (const char *) pointer to string one
//		2 (size_t) length of string one
//		3 (const char *) pointer to string two
//		4 (size_t) length of string two
//
// returns	(int) -1 : string one  < string two
//		(int)  0 : string one == string two
//		(int)  1 : string one  > string two
//-----------------------------------------------------------------------------
#define mem_compare mem_cmp
int
mem_cmp (
    const char *	arg_one_ptr
    ,
    size_t		arg_one_len
    ,
    const char *	arg_two_ptr
    ,
    size_t		arg_two_len
    )
;

//-----------------------------------------------------------------------------
// function	mem_cmp_lower
// alias	mem_compare_lower
//
// purpose	Compare the lower case equivalents of two strings, which are
//		specified by pointer and length, returning the collation order
//		difference as -1, 0, or 1.  If one string ends before a
//		difference is found, then the shorter string is considered to
//		be lower in sequence than the longer string.
//
// arguments	1 (const char *) pointer to string one
//		2 (size_t) length of string one
//		3 (const char *) pointer to string two
//		4 (size_t) length of string two
//
// returns	(int) -1 : string one  < string two
//		(int)  0 : string one == string two
//		(int)  1 : string one  > string two
//-----------------------------------------------------------------------------
#define mem_compare_lower mem_cmp_lower
int
mem_cmp_lower (
    const char *	arg_one_ptr
    ,
    size_t		arg_one_len
    ,
    const char *	arg_two_ptr
    ,
    size_t		arg_two_len
    )
;

//-----------------------------------------------------------------------------
// function	mem_cmp_upper
// alias	mem_compare_upper
//
// purpose	Compare the upper case equivalents of two strings, which are
//		specified by pointer and length, returning the collation order
//		difference as -1, 0, or 1.  If one string ends before a
//		difference is found, then the shorter string is considered to
//		be upper in sequence than the longer string.
//
// arguments	1 (const char *) pointer to string one
//		2 (size_t) length of string one
//		3 (const char *) pointer to string two
//		4 (size_t) length of string two
//
// returns	(int) -1 : string one  < string two
//		(int)  0 : string one == string two
//		(int)  1 : string one  > string two
//-----------------------------------------------------------------------------
#define mem_compare_upper mem_cmp_upper
int
mem_cmp_upper (
    const char *	arg_one_ptr
    ,
    size_t		arg_one_len
    ,
    const char *	arg_two_ptr
    ,
    size_t		arg_two_len
    )
;

//-----------------------------------------------------------------------------
// function	mem_cpy
//
// purpose	Copy to specified space, limited by given size, the contents
//		of one or more strings of bytes specified with pointer and
//		length pairs.  Multiple data sources are concatenated.
//
// arguments	1 (char *) target space to copy strings to
//		2 (size_t) maximum size of target space
//		3 (const char *) source string to copy
//		4 (size_t) length of source string to copy
//		... repeat last 2
//		N (const char *) NULL to end variable arguments
//
// returns	(size_t) total bytes actually copied
//-----------------------------------------------------------------------------
size_t
(mem_cpy) (
    char *		arg_target
    ,
    size_t		arg_size
    ,
    const char *	arg_source
    ,
    size_t		arg_len
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	mem_dup_len
//
// purpose	Duplicate into allocated space the contents of one or more
//		strings of bytes specified with pointer and length pairs,
//		and store the total length of the result where specified.
//
// arguments	1 (size_t *) where to store total length
//		2 (const char *) source string to duplicate
//		3 (size_t) length of string to duplicate
//		... repeat last 2
//		N (const char *) NULL to end variable arguments
//
// returns	(char *) new allocated duplicate string, caller must free
//-----------------------------------------------------------------------------
char *
(mem_dup_len) (
    size_t *		arg_return_len
    ,
    const char *	arg_source_ptr
    ,
    size_t		arg_source_len
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	mem_lower
//
// purpose	Convert each character in a string specified by pointer and
//		length to its lower case equivalent within the same string.
//
// arguments	1 (char *) pointer to target string to convert
//		2 (size_t) length to convert
//
// returns	(size_t) length of string, or ~0 for error
//-----------------------------------------------------------------------------
size_t
mem_lower (
    char *		arg_target
    ,
    size_t		arg_length
    )
;

//-----------------------------------------------------------------------------
// function	mem_upper
//
// purpose	Convert each character in a string specified by pointer and
//		length to its upper case equivalent within the same string.
//
// arguments	1 (char *) pointer to target string to convert
//		2 (size_t) length to convert
//
// returns	(size_t) length of string, or ~0 for error
//-----------------------------------------------------------------------------
size_t
mem_upper (
    char *		arg_target
    ,
    size_t		arg_length
    )
;

//-----------------------------------------------------------------------------
// function	split_cmp_args
//
// purpose	Compare a split string with zero or more argument strings.
//
//		For each function argument string, a part is split from the
//		string associated with the given split object, and the part
//		argument are compared.
//
// note		Only as many split string parts are compared as there are
//		arguments given.  If the split string comes up short, this
//		will result in a mis-match indication at that argument.
//		However, if the split string has more part available, this
//		will not be indicated.  In order to determine if there are
//		more parts available, call split_next() afterwards to see
//		if it indicates a successful split (returns 0).
//
// argument	1 (split_p) pointer to split state
//		2...X-1 (const char *) arguments to compare
//
// note		It is not necessary to supply a NULL after the arguments since
//		split_cmp_args is defined as a macro that provides it.
//
// returns	(int) -1 : error
//		(int)  0 : split string matches arguments
//		(int)  N : mismatch at Nth comparison argument (1..X)
//-----------------------------------------------------------------------------
#define split_cmp_args(s,a...) ((split_cmp_args)((s),a,(const char *)(NULL)))
int
(split_cmp_args) (
    split_p		arg_split
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	split_cmp_args_nocase
//
// purpose	Compare a split string with zero or more argument strings
//		without considering case differences.
//
//		For each function argument string, a part is split from the
//		string associated with the given split object, and the part
//		argument are compared.
//
// note		Only as many split string parts are compared as there are
//		arguments given.  If the split string comes up short, this
//		will result in a mis-match indication at that argument.
//		However, if the split string has more part available, this
//		will not be indicated.  In order to determine if there are
//		more parts available, call split_next() afterwards to see
//		if it indicates a successful split (returns 0).
//
// argument	1 (split_p) pointer to split state
//		2...X-1 (const char *) arguments to compare
//
// note		It is not necessary to supply a NULL after the arguments since
//		split_cmp_args_nocase is defined as a macro that provides it.
//
// returns	(int) -1 : error
//		(int)  0 : split string matches arguments
//		(int)  N : mismatch at Nth comparison argument (1..X)
//-----------------------------------------------------------------------------
#define split_cmp_args_nocase(s,a...) ((split_cmp_args_nocase)((s),a,(const char *)(NULL)))
int
(split_cmp_args_nocase) (
    split_p		arg_split
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	split_next
//
// purpose	Advance the specified split object to the next (first if just
//		initialized) part of the string being split.  Return a status
//		of whether or not a part is found.
//
// argument	1 (split_p) pointer to split state
//
// returns	(int) -1 : error
//		(int)  0 : next part ready
//		(int)  1 : end of string, no next part
//
// note		Use the macros
//			split_part_ptr()
//			split_part_end()
//			split_part_len()
//		to access the actual part.
//-----------------------------------------------------------------------------
int
split_next (
    split_p		arg_split
    )
;

//-----------------------------------------------------------------------------
// function	split_sep_find_call
//
// purpose	Find the start and end of the next separator and update the
//		split state for the new position.
//
//		The separator is determined by a callback function that is
//		given a pointer and length of the remaining data string, and
//		returns the length of a separator, or -1 for not a separator
//		here.
//
// argument	1 (split_p) pointer to split state
//
// returns	(int) -1 : error
//		(int)  0 : separator found
//		(int)  1 : end of string, not found
//-----------------------------------------------------------------------------
int
split_sep_find_call (
    split_p		arg_split
    )
;

//-----------------------------------------------------------------------------
// function	split_sep_find_ch_is
//
// purpose	Find the start and end of the next separator and update the
//		split state for the new position.
//
//		The separator is any character indicated as true by a callback
//		function specified for this split object.
//
// argument	1 (split_p) pointer to split state
//
// returns	(int) -1 : error
//		(int)  0 : separator found
//		(int)  1 : end of string, not found
//-----------------------------------------------------------------------------
int
split_sep_find_ch_is (
    split_p		arg_split
    )
;

//-----------------------------------------------------------------------------
// function	split_sep_find_char
//
// purpose	Find the start and end of the next separator and update the
//		split state for the new position.
//
//		The separator is a specified character.
//
// argument	1 (split_p) pointer to split state
//
// returns	(int) -1 : error
//		(int)  0 : separator found
//		(int)  1 : end of string, not found
//-----------------------------------------------------------------------------
int
split_sep_find_char (
    split_p		arg_split
    )
;

//-----------------------------------------------------------------------------
// function	split_sep_find_clist
//
// purpose	Find the start and end of the next separator and update the
//		split state for the new position.
//
//		The separator is any character found in a specified list given
//		as a string.
//
// argument	1 (split_p) pointer to split state
//
// returns	(int) -1 : error
//		(int)  0 : separator found
//		(int)  1 : end of string, not found
//-----------------------------------------------------------------------------
int
split_sep_find_clist (
    split_p		arg_split
    )
;

//-----------------------------------------------------------------------------
// function	split_sep_find_crlf
//
// purpose	Find the start and end of the next separator and update the
//		split state for the new position.
//
//		The separator is a sequence of CRLF or CR or LF.
//
// argument	1 (split_p) pointer to split state
//
// returns	(int) -1 : error
//		(int)  0 : separator found
//		(int)  1 : end of string, not found
//-----------------------------------------------------------------------------
int
split_sep_find_crlf (
    split_p		arg_split
    )
;

//-----------------------------------------------------------------------------
// function	split_sep_find_ctable
//
// purpose	Find the start and end of the next separator and update the
//		split state for the new position.
//
//		The separator is any character, when indexed into a specified
//		table, yields a true result.
//
// argument	1 (split_p) pointer to split state
//
// returns	(int) -1 : error
//		(int)  0 : separator found
//		(int)  1 : end of string, not found
//-----------------------------------------------------------------------------
int
split_sep_find_ctable (
    split_p		arg_split
    )
;

//-----------------------------------------------------------------------------
// function	split_sep_find_line
//
// purpose	Find the start and end of the next separator and update the
//		split state for the new position.
//
//		The separator is a sequence of CR(any NL) or CR or (any NL).
//
// argument	1 (split_p) pointer to split state
//
// returns	(int) -1 : error
//		(int)  0 : separator found
//		(int)  1 : end of string, not found
//-----------------------------------------------------------------------------
int
split_sep_find_line (
    split_p		arg_split
    )
;

//-----------------------------------------------------------------------------
// function	split_sep_find_punct
//
// purpose	Find the start and end of the next separator and update the
//		split state for the new position.
//
//		The separator is a single punctuation character as defined by
//		the standard ispunct() function/macro.
//
// argument	1 (split_p) pointer to split state
//
// returns	(int) -1 : error
//		(int)  0 : separator found
//		(int)  1 : end of string, not found
//-----------------------------------------------------------------------------
int
split_sep_find_punct (
    split_p		arg_split
    )
;

//-----------------------------------------------------------------------------
// function	split_sep_find_space
//
// purpose	Find the start and end of the next separator and update the
//		split state for the new position.
//
//		The separator is a sequence of any space character as define
//		by the standard isspace() function/macro.
//
// argument	1 (split_p) pointer to split state
//
// returns	(int) -1 : error
//		(int)  0 : separator found
//		(int)  1 : end of string, not found
//-----------------------------------------------------------------------------
int
split_sep_find_space (
    split_p		arg_split
    )
;

//-----------------------------------------------------------------------------
// function	split_sep_find_spctl
//
// purpose	Find the start and end of the next separator and update the
//		split state for the new position.
//
//		The separator is a sequence of any space characters and/or
//		control characters as defined by the standard isspace() and
//		iscntrl() functions/macros.
//
// argument	1 (split_p) pointer to split state
//
// returns	(int) -1 : error
//		(int)  0 : separator found
//		(int)  1 : end of string, not found
//-----------------------------------------------------------------------------
int
split_sep_find_spctl (
    split_p		arg_split
    )
;

//-----------------------------------------------------------------------------
// function	split_sep_find_sptab
//
// purpose	Find the start and end of the next separator and update the
//		split state for the new position.
//
//		The separator is a sequence of one or more spaces and/or tabs.
//
// argument	1 (split_p) pointer to split state
//
// returns	(int) -1 : error
//		(int)  0 : separator found
//		(int)  1 : end of string, not found
//-----------------------------------------------------------------------------
int
split_sep_find_sptab (
    split_p		arg_split
    )
;

//-----------------------------------------------------------------------------
// function	split_sep_find_string
//
// purpose	Find the start and end of the next separator and update the
//		split state for the new position.
//
//		The separator is a specified character.
//
// argument	1 (split_p) pointer to split state
//
// returns	(int) -1 : error
//		(int)  0 : separator found
//		(int)  1 : end of string, not found
//-----------------------------------------------------------------------------
int
split_sep_find_string (
    split_p		arg_split
    )
;

//-----------------------------------------------------------------------------
// function	split_to_common
//
// purpose	Collect parts from the specified split object and construct
//		a common pointer array and string pool from the parts.
//
// argument	1 (split_p) pointer to split state
//		2 (char * * *) where to store pointer to common area
//
// returns	(int) -1 : error
//		(int)  0 : no parts found
//		(int)  N : number of parts found
//-----------------------------------------------------------------------------
int
split_to_common (
    split_p		arg_split
    ,
    char * * *		arg_common
    )
;

//-----------------------------------------------------------------------------
// function	(str_app_arg)
//
// purpose	Append to the first string (given with maximum available space)
//		the contents of zero or more (multiple) strings.
//
//		Do append only if all source strings can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3..N (const char *) source strings to append
//		N+1 (const char *) NULL to indicate no more strings
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
(str_app_arg) (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    const char *	arg_source
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(str_app_arg_lim)
//
// purpose	Append to the first string (given with maximum available space)
//		the contents of zero or more (multiple) strings limited by
//		number of characters.
//
//		Do append only if all source strings can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3..N-1 (const char *) source strings to append
//		4..N (size_t) limit length of source strings
//		N+1 (const char *) NULL to indicate no more strings
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
(str_app_arg_lim) (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    const char *	arg_source
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(str_app_arg_lim_sep)
//
// purpose	Append to the first string (given with maximum available space)
//		the contents of zero or more (multiple) strings with a
//		separator string inserted between multiple strings limited by
//		number of characters.
//
//		Do append only if all source strings can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) separator string to insert
//		4 (size_t) limit length of separator string
//		5..N-1 (const char *) source strings to append
//		6..N (size_t) limit length of source string
//		N+1 (const char *) NULL to indicate no more strings
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
(str_app_arg_lim_sep) (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    const char *	arg_sep
    ,
    size_t		arg_seplen
    ,
    const char *	arg_source
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(str_app_arg_sep)
//
// purpose	Append to the first string (given with maximum available space)
//		the contents of zero or more (multiple) strings with a
//		separator string inserted between multiple strings.
//
//		Do append only if all source strings can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) separator string to insert
//		4..N (const char *) source strings to append
//		N+1 (const char *) NULL to indicate no more strings
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
(str_app_arg_sep) (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    const char *	arg_sep
    ,
    const char *	arg_source
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	str_app_arr
//
// purpose	Append to the first string (given with maximum available space)
//		the contents of zero or more (multiple) strings provided as
//		an array of string pointers.
//
//		Do append only if all source strings can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char * const *) array of string pointers to append
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_arr (
    char *			arg_target
    ,
    size_t			arg_length
    ,
    const char * const *	arg_source
    )
;

//-----------------------------------------------------------------------------
// function	str_app_arr_sep
//
// purpose	Append to the first string (given with maximum available space)
//		the contents of zero or more (multiple) strings provided as
//		an array of string pointers with a separator string inserted
//		between multiple strings.
//
//		Do append only if all source strings can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) separator string to insert
//		4 (const char * const *) array of string pointers to append
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_arr_sep (
    char *			arg_target
    ,
    size_t			arg_length
    ,
    const char *		arg_sep
    ,
    const char * const *	arg_source
    )
;

//-----------------------------------------------------------------------------
// function	str_app_ch
//
// purpose	Append to the first string (given with maximum available space)
//		a single character value.
//
//		Do append only if there is space for one more character.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (int) (char) character to be appended
//
// returns	(size_t) new length of string, or ~0 for error
//-----------------------------------------------------------------------------
size_t
str_app_ch (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    int			arg_char
    )
;

//-----------------------------------------------------------------------------
// function	str_app_d
//
// purpose	Append to the first string (given with maximum available space)
//		a string of digits representing the double value.
//
//		Do append only if all the digits can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (double) the number to convert
//		4 (int) total conversion field size
//		5 (int) fractional precision size
//		6 (int) (char) format type, one of (a,A,e,E,f,F,g,G)
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_d (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    double		arg_value
    ,
    int			arg_field
    ,
    int			arg_prec
    ,
    int			arg_format
    )
;

//-----------------------------------------------------------------------------
// function	str_app_d_cet
//
// purpose	Append to the first string (given with maximum available space)
//		a double value in cetal/hexadecimal representation.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (double) value to encode
//
// returns	(size_t) new length of string, or ~0 for error
//-----------------------------------------------------------------------------
size_t
str_app_d_cet (
    char *		arg_target
    ,
    size_t		arg_size
    ,
    double		arg_value
    )
;

//-----------------------------------------------------------------------------
// function	str_app_decode_base64
//
// purpose	Append to given target the decoded string from a base 64
//		encoded input string.
//
//              Do append only if all source strings can be appended.
//
// arguments    1 (char *) pointer to target string
//              2 (size_t) maximum space in target string
//		3 (const char *) source string
//
// returns	(size_t) final length if target string
//		(size_t) ~0 : error
//-----------------------------------------------------------------------------
size_t
str_app_decode_base64 (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    const char *	arg_source
    )
;

//-----------------------------------------------------------------------------
// function	str_app_escape
//
// purpose	Append to the first string (given with maximum available space)
//		the contents of another string converted to an escaped format
//		to protect text files from control characters.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) source string to append
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_escape (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    const char *	arg_source
    )
;

//-----------------------------------------------------------------------------
// function	str_app_ipv4
//
// purpose	Append to the first string (given with maximum available space)
//		a string of dotted decimal digits representing the IPv4 network
//		address given as an unsigned long.
//
//		Do append only if all the digits can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (unsigned long) the IPv4 address to convert
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_ipv4 (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    unsigned long	arg_ipv4
    )
;

//-----------------------------------------------------------------------------
// function	str_app_ipv4_min
//
// purpose	Append to the first string (given with maximum available space)
//		a string of dotted decimal digits representing the IPv4 network
//		address given as an unsigned long, with a specified minimum
//		number of digits per octet.
//
//		Do append only if all the digits can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (unsigned long) the IPv4 address to convert
//		4 (int) minimum number of digits per octet.
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_ipv4_min (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    unsigned long	arg_ipv4
    ,
    int			arg_min
    )
;

//-----------------------------------------------------------------------------
// function	str_app_ld
//
// purpose	Append to the first string (given with maximum available space)
//		a string of digits representing the long double value.
//
//		Do append only if all the digits can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (long double) the number to convert
//		4 (int) total conversion field size
//		5 (int) fractional precision size
//		6 (int) (char) format type, one of (a,A,e,E,f,F,g,G)
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_ld (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    long double		arg_value
    ,
    int			arg_field
    ,
    int			arg_prec
    ,
    int			arg_format
    )
;

//-----------------------------------------------------------------------------
// function	str_app_ld_cet
//
// purpose	Append to the first string (given with maximum available space)
//		a long double value in cetal/hexadecimal representation.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (long double) value to encode
//
// returns	(size_t) new length of string, or ~0 for error
//-----------------------------------------------------------------------------
size_t
str_app_ld_cet (
    char *		arg_target
    ,
    size_t		arg_size
    ,
    long double		arg_value
    )
;

//-----------------------------------------------------------------------------
// function	str_app_lim
//
// purpose	Append to the first string (given with maximum available space)
//		the contents of another string limited by number of characters.
//
//		Do append only if the entire source string can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) source string to append
//		4 (size_t) limit length of source string
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_lim (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    const char *	arg_source
    ,
    size_t		arg_srclen
    )
;

//-----------------------------------------------------------------------------
// function	str_app_lim_lower
//
// purpose	Append to the first string (given with maximum available space)
//		the contents of another string converted to lower case limited
//		by number of characters.
//
//		Do append only if the entire source string can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) source string to append
//		4 (size_t) limit length of source string
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_lim_lower (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    const char *	arg_source
    ,
    size_t		arg_srclen
    )
;

//-----------------------------------------------------------------------------
// function	str_app_lim_upper
//
// purpose	Append to the first string (given with maximum available space)
//		the contents of another string converted to upper case limited
//		by number of characters.
//
//		Do append only if the entire source string can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) source string to append
//		4 (size_t) limit length of source string
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_lim_upper (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    const char *	arg_source
    ,
    size_t		arg_srclen
    )
;

//-----------------------------------------------------------------------------
// function	str_app_lower
//
// purpose	Append to the first string (given with maximum available space)
//		the contents of another string converted to lower case.
//
//		Do append only if the entire source string can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) source string to append
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_lower (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    const char *	arg_source
    )
;

//-----------------------------------------------------------------------------
// function	str_app_part_ch
//
// purpose	Append to the target string given with maximum available space
//		the first part found in the source string separated by a given
//		character, returning a pointer to the remainder of the source
//		string.
//
//		If the string to be appended does not fit the available space,
//		it remains truncated without indication.
//
//		Unlike str_split_ch(), this function does not modify the
//		source string.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) input string
//		4 (int) separator character
//
// returns	(char *) NULL : no item found or error
//		(char *) pointer to remainder of input string.
//-----------------------------------------------------------------------------
char *
str_app_part_ch (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    const char *	arg_source
    ,
    int			arg_ch
    )
;

//-----------------------------------------------------------------------------
// function	str_app_part_chs
//
// purpose	Append to the target string given with maximum available space
//		the first item found in the source string separated by one of
//		a given set of characters, returning a pointer to the remainder
//		of the source string.
//
//		If the string to be appended does not fit the available space,
//		it remains truncated without indication.
//
//		Unlike str_split_chs(), this function does not modify the
//		source string.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) input string
//		4 (const char *) string of separator characters
//
// returns	(char *) NULL : no item found or error
//		(char *) pointer to remainder of source string.
//-----------------------------------------------------------------------------
char *
str_app_part_chs (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    const char *	arg_source
    ,
    const char *	arg_ch_str
    )
;

//-----------------------------------------------------------------------------
// function	str_app_part_space
//
// purpose	Append to the target string given with maximum available space
//		the first item found in the source string separated by a space
//		character, returning a pointer to the remainder of the source.
//
//		If the string to be appended does not fit the available space,
//		it remains truncated without indication.
//
//		Unlike str_split_space(), this function does not modify the
//		source string.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) input string
//
// returns	(char *) NULL : no item found or error
//		(char *) pointer to remainder in input string.
//
// note		Spaces are determined according to the isspace() macro or
//		function in the C language.  Multiple space characters in a
//		group constitute a single delimiter, even if mixed.
//
// note		Any leading spaces will be removed.  If the caller needs to
//		know if there were any leading spaces, it should check the
//		original string first with isspace().
//-----------------------------------------------------------------------------
char *
str_app_part_space (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    const char *	arg_source
    )
;

//-----------------------------------------------------------------------------
// function	str_app_part_str
//
// purpose	Append to the target string given with maximum available space
//		the first item found in the source string separated by a given
//		string, returning a pointer to the remainder of the source.
//
//		If the string to be appended does not fit the available space,
//		it remains truncated without indication.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) input string
//		4 (const char *) separator string
//
// returns	(char *) NULL : no item found or error
//		(char *) pointer to remainder in input string.
//-----------------------------------------------------------------------------
char *
str_app_part_str (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    const char *	arg_source
    ,
    const char *	arg_sep_str
    )
;

//-----------------------------------------------------------------------------
// function	str_app_rep
//
// purpose	Append to the target string the contents of the source string,
//		with all occurrences of the pattern string replaced by the
//		replacement string.
//
//		If entire resultant string will not fit the available space,
//		then rollback everything leaving the original target string.
//
// arguments	1 (char *) pointer to beginning of target string
//		2 (size_t) total maximum length of target string
//		3 (const char *) pattern string to be matched in source
//		4 (const char *) replacement string to replace each pattern
//		5 (const char *) source string to be appended to target
//
// returns	(size_t) final length of target string
//		(size_t) ~0 : error
//
// note		Contents of target string space located beyond the original
//		string termination character will be overwritten if there is
//		insufficient space to complete the append.  Nothing should be
//		stored there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_rep (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    const char *	arg_pat
    ,
    const char *	arg_rep
    ,
    const char *	arg_source
    )
;

//-----------------------------------------------------------------------------
// function	str_app_si
//
// purpose	Append to the first string (given with maximum available space)
//		a string of digits representing the signed int value.
//
//		Do append only if all the digits can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (signed int) the number to convert
//		4 (int) minimum number of digits
//		5 (int) (char) padding character
//		6 (int) conversion base
//		7 (const char *) alternate digit set
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_si (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    signed int		arg_value
    ,
    int			arg_min
    ,
    int			arg_pad_ch
    ,
    int			arg_base
    ,
    const char *	arg_digit_set
    )
;

//-----------------------------------------------------------------------------
// function	str_app_sl
// alias	str_app_sl_bin
//		str_app_sl_oct
//		str_app_sl_dec
//		str_app_sl_cet
//
// purpose	Append to the first string (given with maximum available space)
//		a string of digits representing the signed long value.
//
//		Do append only if all the digits can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (signed long) the number to convert
//		4 (int) minimum number of digits
//		5 (int) (char) padding character
//		6 (int) conversion base
//		7 (const char *) alternate digit set
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_sl (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    signed long		arg_value
    ,
    int			arg_min
    ,
    int			arg_pad_ch
    ,
    int			arg_base
    ,
    const char *	arg_digit_set
    )
;

//-----------------------------------------------------------------------------
// function	str_app_sl_text
//
// purpose	Append to the first string (given with maximum available space)
//		a string of text representing the signed long value.
//
//		Do append only if all the text can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (signed long) the number to convert
//		4 (const char *) language id string, default "eng"
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_sl_text (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    signed long		arg_value
    ,
    const char *	arg_lang
    )
;

//-----------------------------------------------------------------------------
// function	str_app_sll
//
// purpose	Append to the first string (given with maximum available space)
//		a string of digits representing the signed long long value.
//
//		Do append only if all the digits can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (signed long long) the number to convert
//		4 (int) minimum number of digits
//		5 (int) (char) padding character
//		6 (int) conversion base
//		7 (const char *) alternate digit set
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_sll (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    signed long long	arg_value
    ,
    int			arg_min
    ,
    int			arg_pad_ch
    ,
    int			arg_base
    ,
    const char *	arg_digit_set
    )
;

//-----------------------------------------------------------------------------
// function	str_app_sll_text
//
// purpose	Append to the first string (given with maximum available space)
//		a string of text representing the signed long long value.
//
//		Do append only if all the text can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (signed long long) the number to convert
//		4 (const char *) language id string, default "eng"
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_sll_text (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    signed long long	arg_value
    ,
    const char *	arg_lang
    )
;

//-----------------------------------------------------------------------------
// function	str_app_str
//
// purpose	Append to the first string (given with maximum available space)
//		the contents of another string.
//
//		Do append only if the entire source string can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) source string to append
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
#define	str_app		str_app_str
size_t
str_app_str (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    const char *	arg_source
    )
;

//-----------------------------------------------------------------------------
// function	str_app_sub
//
// purpose	Append to the target string the contents of a substring of the
//		source string from one specified part to another specified part
//		as separated by a given separator pattern string.
//
//		A positive index refers to parts between separators in the
//		source string starting with number 0 at the beginning of the
//		source string and increasing in the forward direction.
//
//		A negative index refers to parts between separators in the
//		source string starting with number -1 at the end of the source
//		string and decreasing in the reverse direction.
//
//		If a pattern string is not given, the parts are characters and
//		the separators are between each character.
//
//		If the resultant string will not fit the available space,
//		then rollback everything leaving the original target string.
//
// arguments	1 (char *) pointer to beginning of target string
//		2 (size_t) total maximum length of target string
//		3 (int) index of first part to begin the substring
//		4 (int) index of last part to end the substring
//		5 (const char *) source string to select substring from
//		6 (const char *) pattern string to separate source parts
//
// returns	(size_t) final length of target string
//		(size_t) ~0 : error
//
// note		Any contents of the target string space located beyond the
//		first termination character found in the target space will be
//		overwritten even if there is insufficient space to complete
//		the appending.  No data of any kind should be kept there.
//-----------------------------------------------------------------------------
size_t
str_app_sub (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    int			arg_bindex
    ,
    int			arg_eindex
    ,
    const char *	arg_source
    ,
    const char *	arg_pattern
    )
;

//-----------------------------------------------------------------------------
// function	str_app_ui
//
// purpose	Append to the first string (given with maximum available space)
//		a string of digits representing the unsigned int value.
//
//		Do append only if all the digits can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (unsigned int) the number to convert
//		4 (int) minimum number of digits
//		5 (int) (char) padding character
//		6 (int) conversion base
//		7 (const char *) alternate digit set
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_ui (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    unsigned int	arg_value
    ,
    int			arg_min
    ,
    int			arg_pad_ch
    ,
    int			arg_base
    ,
    const char *	arg_digit_set
    )
;

//-----------------------------------------------------------------------------
// function	str_app_ul
//
// purpose	Append to the first string (given with maximum available space)
//		a string of digits representing the unsigned long value.
//
//		Do append only if all the digits can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (unsigned long) the number to convert
//		4 (int) minimum number of digits
//		5 (int) (char) padding character
//		6 (int) conversion base
//		7 (const char *) alternate digit set
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_ul (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    unsigned long	arg_value
    ,
    int			arg_min
    ,
    int			arg_pad_ch
    ,
    int			arg_base
    ,
    const char *	arg_digit_set
    )
;

//-----------------------------------------------------------------------------
// function	str_app_ul_text
//
// purpose	Append to the first string (given with maximum available space)
//		a string of text representing the unsigned long value.
//
//		Do append only if all the text can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (unsigned long) the number to convert
//		4 (const char *) language id string, default "eng"
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_ul_text (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    unsigned long	arg_value
    ,
    const char *	arg_lang
    )
;

//-----------------------------------------------------------------------------
// function	str_app_ull
//
// purpose	Append to the first string (given with maximum available space)
//		a string of digits representing the unsigned long long value.
//
//		Do append only if all the digits can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (unsigned long long) the number to convert
//		4 (int) minimum number of digits
//		5 (int) (char) padding character
//		6 (int) conversion base
//		7 (const char *) alternate digit set
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_ull (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    unsigned long long	arg_value
    ,
    int			arg_min
    ,
    int			arg_pad_ch
    ,
    int			arg_base
    ,
    const char *	arg_digit_set
    )
;

//-----------------------------------------------------------------------------
// function	str_app_ull_text
//
// purpose	Append to the first string (given with maximum available space)
//		a string of text representing the unsigned long long value.
//
//		Do append only if all the text can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (unsigned long long) the number to convert
//		4 (const char *) language id string, default "en_US"
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_ull_text (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    unsigned long long	arg_value
    ,
    const char *	arg_lang
    )
;

//-----------------------------------------------------------------------------
// function	str_app_upper
//
// purpose	Append to the first string (given with maximum available space)
//		the contents of another string converted to upper case.
//
//		Do append only if the entire source string can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) source string to append
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_upper (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    const char *	arg_source
    )
;

//-----------------------------------------------------------------------------
// function	str_app_url
//
// purpose	Append to the first string given with maximum available space
//		a URL formed from several component strings.
//
//		Do append only if all source strings can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) protocol, "http", "ftp"
//		4 (const char *) username
//		5 (const char *) password
//		6 (const char *) hostname
//		7 (const char *) portnum
//		8 (const char *) fullpath
//		9 (const char *) querystring
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_url (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    const char *	arg_protocol
    ,
    const char *	arg_username
    ,
    const char *	arg_password
    ,
    const char *	arg_hostname
    ,
    const char *	arg_portnum
    ,
    const char *	arg_fullpath
    ,
    const char *	arg_querystring
    )
;

//-----------------------------------------------------------------------------
// function	str_app_var
//
// purpose	Append to the first string (given with maximum available space)
//		the contents of zero or more (multiple) strings provided as
//		a variable argument list passed in one argument.
//
//		Do append only if all source strings can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (va_list) source strings to append
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_var (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    va_list		arg_source
    )
;

//-----------------------------------------------------------------------------
// function	str_app_var_sep
//
// purpose	Append to the first string (given with maximum available space)
//		the contents of zero or more (multiple) strings provided as
//		a variable argument list passed in one argument with a
//		separator string inserted between multiple strings.
//
//		Do append only if all source strings can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) separator string to insert
//		4 (va_list) source strings to append
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_var_sep (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    const char *	arg_sep
    ,
    va_list		arg_source
    )
;

//-----------------------------------------------------------------------------
// function	str_cmp_lower
//
// purpose	Compare two strings converted to lower case, returning a result
//		that indicates the collation order of the strings.
//
// arguments	1 (const char *) string 1
//		2 (const char *) string 2
//
// returns	(int) -1 : string 1 <  string 2
//		(int)  0 : string 1 == string 2
//		(int)  1 : string 1 >  string 2
//-----------------------------------------------------------------------------
#define str_cmp_lower(a,b) str_cmp_lim_lower((a),(b),~0)

//-----------------------------------------------------------------------------
// function     str_equal_lower
//
// purpose      Compare two strings converted to lower case for equality,
//		returning true or false.
//
// arguments    1 (const char *) string 1
//              2 (const char *) string 2
//
// returns      (int) 0 : strings are not equal
//              (int) 1 : strings are equal
//-----------------------------------------------------------------------------
#define str_equal_lower(a,b) ((str_cmp_lim_lower((a),(b),~0))==0)

//-----------------------------------------------------------------------------
// function	str_cmp_lim_lower
//
// purpose	Compare two strings converted to lower case, returning a result
//		that indicates the collation order of the strings.
//
// arguments	1 (const char *) string 1
//		2 (const char *) string 2
//		3 (size_t) maximum length to compare, or ~0 to compare full
//
// returns	(int) -1 : string 1 <  string 2
//		(int)  0 : string 1 == string 2
//		(int)  1 : string 1 >  string 2
//-----------------------------------------------------------------------------
int
str_cmp_lim_lower (
    const char *	arg_string1
    ,
    const char *	arg_string2
    ,
    size_t		arg_length
    )
;

//-----------------------------------------------------------------------------
// function	str_cmp_prefix
//
// purpose	Compare two strings to determine if string one is an exact
//		prefix of string two.
//
// arguments	1 (const char *) string one (prefix candidate)
//		2 (const char *) string two
//
// returns	(char *) pointer to end of prefix in string two
//		(char *) NULL if not a prefix or error
//
// note		If the returned pointer points to a termination character the
//		the two strings are identical.
//-----------------------------------------------------------------------------
char *
str_cmp_prefix (
    const char *	arg_str_one
    ,
    const char *	arg_str_two
    )
;

//-----------------------------------------------------------------------------
// function	str_cmp_prefix_nocase
//
// purpose	Compare two strings to determine if string one is an exact
//		prefix of string two, without considering alpha case.
//
// arguments	1 (const char *) string one (prefix candidate)
//		2 (const char *) string two
//
// returns	(char *) pointer to end of prefix in string two
//		(char *) NULL if not a prefix or error
//
// note		If the returned pointer points to a termination character the
//		the two strings are equivalent.
//-----------------------------------------------------------------------------
char *
str_cmp_prefix_nocase (
    const char *	arg_str_one
    ,
    const char *	arg_str_two
    )
;

//-----------------------------------------------------------------------------
// function	str_cmp_upper
//
// purpose	Compare two strings converted to upper case, returning a result
//		that indicates the collation order of the strings.
//
// arguments	1 (const char *) string 1
//		2 (const char *) string 2
//
// returns	(int) -1 : string 1 <  string 2
//		(int)  0 : string 1 == string 2
//		(int)  1 : string 1 >  string 2
//-----------------------------------------------------------------------------
#define str_cmp_upper(a,b) str_cmp_lim_upper((a),(b),~0)

//-----------------------------------------------------------------------------
// function     str_equal_upper
//
// purpose      Compare two strings converted to upper case for equality,
//              returning true or false.
//
// arguments    1 (const char *) string 1
//              2 (const char *) string 2
//
// returns      (int) 0 : strings are not equal
//              (int) 1 : strings are equal
//-----------------------------------------------------------------------------
#define str_equal_upper(a,b) ((str_cmp_lim_upper((a),(b),~0))==0)

//-----------------------------------------------------------------------------
// function	str_cmp_lim_upper
//
// purpose	Compare two strings converted to upper case, returning a result
//		that indicates the collation order of the strings.
//
// arguments	1 (const char *) string 1
//		2 (const char *) string 2
//		3 (size_t) maximum length to compare, or ~0 to compare full
//
// returns	(int) -1 : string 1 <  string 2
//		(int)  0 : string 1 == string 2
//		(int)  1 : string 1 >  string 2
//-----------------------------------------------------------------------------
int
str_cmp_lim_upper (
    const char *	arg_string1
    ,
    const char *	arg_string2
    ,
    size_t		arg_length
    )
;

//-----------------------------------------------------------------------------
// function	(str_dup_arg)
//
// purpose	Duplicate into allocated space the contents of zero or more
//		(multiple) strings.
//
// arguments	1..N (const char *) source strings to duplicate
//		N+1 (const char *) NULL to indicate no more strings
//
// returns	(char *) new allocated duplicate string, caller must free
//		(char *) NULL : error allocating string or other error
//
// warning	If other threads modify any source strings during duplication,
//		then results are unpredictable.
//-----------------------------------------------------------------------------
char *
(str_dup_arg) (
    const char *	arg_source
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(str_dup_arg_lim)
//
// purpose	Duplicate into allocated space the contents of zero or more
//		(multiple) strings limited by number of characters.
//
// arguments	1..N-1 (const char *) source strings to duplicate
//		2..N (size_t) limit lengths of source strings
//		N+1 (const char *) NULL to indicate no more strings
//
// returns	(char *) new allocated duplicate string, caller must free
//		(char *) NULL : error allocating string or other error
//
// warning	If other threads modify any source strings during duplication,
//		then results are unpredictable.
//-----------------------------------------------------------------------------
char *
(str_dup_arg_lim) (
    const char *	arg_source
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(str_dup_arg_lim_sep)
//
// purpose	Duplicate into allocated space the contents of zero or more
//		(multiple) strings with a separator string inserted between
//		multiple strings limited by number of characters.
//
// arguments	1 (const char *) separator string to insert
//		2 (size_t) limit length of separator string
//		3..N-1 (const char *) source strings to duplicate
//		4..N (size_t) limit lengths of source strings
//		N+1 (const char *) NULL to indicate no more strings
//
// returns	(char *) new allocated duplicate string, caller must free
//		(char *) NULL : error allocating string or other error
//
// warning	If other threads modify any source strings during duplication,
//		then results are unpredictable.
//-----------------------------------------------------------------------------
char *
(str_dup_arg_lim_sep) (
    const char *	arg_sep
    ,
    size_t		arg_seplen
    ,
    const char *	arg_source
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(str_dup_arg_sep)
//
// purpose	Duplicate into allocated space the contents of zero or more
//		(multiple) strings with a separator string inserted between
//		multiple strings.
//
// arguments	1 (const char *) separator string to insert
//		1..N (const char *) source strings to duplicate
//		N+1 (const char *) NULL to indicate no more strings
//
// returns	(char *) new allocated duplicate string, caller must free
//		(char *) NULL : error allocating string or other error
//
// warning	If other threads modify any source strings during duplication,
//		then results are unpredictable.
//-----------------------------------------------------------------------------
char *
(str_dup_arg_sep) (
    const char *	arg_sep
    ,
    const char *	arg_source
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	str_dup_arr
//
// purpose	Duplicate into allocated space the contents of zero or more
//		(multiple) strings provided as an array of string pointers.
//
// arguments	1 (const char * const *) source strings to duplicate
//
// returns	(char *) new allocated duplicate string, caller must free
//		(char *) NULL : error allocating string or other error
//
// warning	If other threads modify any source strings during duplication,
//		then results are unpredictable.
//-----------------------------------------------------------------------------
char *
str_dup_arr (
    const char * const *	arg_source
    )
;

//-----------------------------------------------------------------------------
// function	str_dup_arr_sep
//
// purpose	Duplicate into allocated space the contents of zero or more
//		(multiple) strings provided as an array of string pointers
//		with a separator string inserted between multiple strings.
//
// arguments	1 (const char *) separator string to insert
//		2 (const char * const *) source strings to duplicate
//
// returns	(char *) new allocated duplicate string, caller must free
//		(char *) NULL : error allocating string or other error
//
// warning	If other threads modify any source strings during duplication,
//		then results are unpredictable.
//-----------------------------------------------------------------------------
char *
str_dup_arr_sep (
    const char *		arg_sep
    ,
    const char * const *	arg_source
    )
;

//-----------------------------------------------------------------------------
// function	str_dup_lim
//
// purpose	Duplicate into allocated space the contents of another string
//		limited by number of characters.
//
// arguments	1 (const char *) source string to duplicate
//		2 (size_t) limit length of source string
//
// returns	(char *) new allocated duplicate string, caller must free
//
// note		This function is similar to the common but non-standard
//		function strdup().
//-----------------------------------------------------------------------------
char *
str_dup_lim (
    const char *	arg_source
    ,
    size_t		arg_srclen
    )
;

//-----------------------------------------------------------------------------
// function	str_dup_lim_lower
//
// purpose	Duplicate into allocated space the contents of another string
//		converted to lower case limited by number of characters.
//
// arguments	1 (const char *) source string to duplicate and convert
//		2 (size_t) limit length of source string
//
// returns	(char *) new allocated duplicate string, caller must free
//-----------------------------------------------------------------------------
char *
str_dup_lim_lower (
    const char *	arg_source
    ,
    size_t		arg_srclen
    )
;

//-----------------------------------------------------------------------------
// function	str_dup_lim_upper
//
// purpose	Duplicate into allocated space the contents of another string
//		converted to upper case limited by number of characters.
//
// arguments	1 (const char *) source string to duplicate and convert
//		2 (size_t) limit length of source string
//
// returns	(char *) new allocated duplicate string, caller must free
//-----------------------------------------------------------------------------
char *
str_dup_lim_upper (
    const char *	arg_source
    ,
    size_t		arg_srclen
    )
;

//-----------------------------------------------------------------------------
// function	str_dup_lower
//
// purpose	Duplicate into allocated space the contents of another string
//		converted to lower case.
//
// arguments	1 (const char *) source string to duplicate and convert
//
// returns	(char *) new allocated duplicate string, caller must free
//-----------------------------------------------------------------------------
char *
str_dup_lower (
    const char *	arg_source
    )
;

//-----------------------------------------------------------------------------
// function	str_dup_str
// alias	str_dup
//
// purpose	Duplicate into allocated space the contents of another string.
//
// arguments	1 (const char *) source string to duplicate
//
// returns	(char *) new allocated duplicate string, caller must free
//
// note		This function is the same as the common but non-standard
//		function strdup().
//-----------------------------------------------------------------------------
#define str_dup(s)	str_dup_str((s))

char *
str_dup_str (
    const char *	arg_source
    )
;

//-----------------------------------------------------------------------------
// function	str_dup_upper
//
// purpose	Duplicate into allocated space the contents of another string
//		converted to upper case.
//
// arguments	1 (const char *) source string to duplicate and convert
//
// returns	(char *) new allocated duplicate string, caller must free
//-----------------------------------------------------------------------------
char *
str_dup_upper (
    const char *	arg_source
    )
;

//-----------------------------------------------------------------------------
// function	str_dup_var
//
// purpose	Duplicate into allocated space the contents of zero or more
//		(multiple) strings provided as a variable argument list passed
//		in one argument.
//
// arguments	1 (va_list) source strings to duplicate
//
// returns	(char *) new allocated duplicate string, caller must free
//		(char *) NULL : error allocating string or other error
//
// warning	If other threads modify any source strings during duplication,
//		then results are unpredictable.
//-----------------------------------------------------------------------------
char *
str_dup_var (
    va_list		arg_list
    )
;

//-----------------------------------------------------------------------------
// function	str_dup_var_sep
//
// purpose	Duplicate into allocated space the contents of zero or more
//		(multiple) strings provided as a variable argument list passed
//		in one argument with a separator string inserted between
//		multiple strings.
//
// arguments	1 (const char *) separator string to insert
//		2 (va_list) source strings to duplicate
//
// returns	(char *) new allocated duplicate string, caller must free
//		(char *) NULL : error allocating string or other error
//
// warning	If other threads modify any source strings during duplication,
//		then results are unpredictable.
//-----------------------------------------------------------------------------
char *
str_dup_var_sep (
    const char *	arg_sep
    ,
    va_list		arg_list
    )
;

//-----------------------------------------------------------------------------
// function	str_eq_opt
//
// purpose	Compare two strings for equality according to specified
//		options:
//
//		STR_EQ_NOCASE	disregard case for alphabetics
//		STR_EQ_SPACE	treat whitespace clusters as one
//		STR_EQ_NUMERIC	handle digits like numeric
//
// arguments	1 (const char *) string 1
//		2 (const char *) string 2
//		3 (size_t) maximum length to compare, or ~0 to compare full
//		4 (int) options
//
// returns	(int)  0 : string 1 != string 2
//		(int)  1 : string 1 == string 2
//-----------------------------------------------------------------------------
int
str_eq_opt (
    const char *	arg_string1
    ,
    const char *	arg_string2
    ,
    size_t		arg_length
    ,
    int			arg_option
    )
;

//-----------------------------------------------------------------------------
// function	(str_expr_to_d)
//
// purpose	Convert a string containing a numeric expression into a
//		double.
//
// arguments	1 ((*)()) function pointer to pow passed by macro
//		2 (const char *) pointer to string to convert
//		3 (char * *) pointer to where to store end pointer
//
// returns	(double) converted result
//
// note		Numeric values that exceed the maximums for this data type
//		will result in undefined values being returned.
//
// note		The reference to pow is passed from the calling program via
//		the wrapper macro in argument one so that LIBH itself does not
//		generate a reference to libm.
//-----------------------------------------------------------------------------
double
(str_expr_to_d) (
    double	( *	arg_pow		)( double, double )
    ,
    const char *	arg_str
    ,
    char * *		arg_end_ptr
    )
;

//-----------------------------------------------------------------------------
// function	str_expr_to_ld
//
// purpose	Convert a string containing a numeric expression into a
//		long double.
//
// arguments	1 (const char *) pointer to string to convert
//		2 (char * *) pointer to string to store end pointer
//
// returns	(long double) converted result
//
// note		Numeric values that exceed the maximums for this data type
//		will result in undefined values being returned.
//
// note		The reference to powl is passed from the calling program via
//		the wrapper macro in argument one so that LIBH itself does not
//		generate a reference to libm.
//-----------------------------------------------------------------------------
long double
(str_expr_to_ld) (
    long double		( *	arg_powl	)( long double, long double )
    ,
    const char *		arg_str
    ,
    char * *			arg_end_ptr
    )
;

//-----------------------------------------------------------------------------
// function	str_expr_to_si
//
// purpose	Convert a string containing a numeric expression into a
//		signed int.
//
// arguments	1 (const char *) pointer to string to convert
//		2 (char * *) pointer to string to store end pointer
//		3 (int) numeric base value, 2 to 36
//
// returns	(signed int) converted result
//
// note		Numeric values that exceed the maximums for this data type
//		will result in undefined values being returned.
//-----------------------------------------------------------------------------
signed int
str_expr_to_si (
    const char *	arg_str
    ,
    char * *		arg_end_str
    ,
    int			arg_base
    )
;

//-----------------------------------------------------------------------------
// function	str_expr_to_sl
//
// purpose	Convert a string containing a numeric expression into a
//		signed long integer.
//
// arguments	1 (const char *) pointer to string to convert
//		2 (char * *) pointer to string to store end pointer
//		3 (int) numeric base value, 2 to 36
//
// returns	(signed long) converted result
//
// note		Numeric values that exceed the maximums for this data type
//		will result in undefined values being returned.
//-----------------------------------------------------------------------------
signed long
str_expr_to_sl (
    const char *	arg_str
    ,
    char * *		arg_end_str
    ,
    int			arg_base
    )
;

//-----------------------------------------------------------------------------
// function	str_expr_to_sll
//
// purpose	Convert a string containing a numeric expression into a
//		signed long long integer.
//
// arguments	1 (const char *) pointer to string to convert
//		2 (char * *) pointer to string to store end pointer
//		3 (int) numeric base value, 2 to 36
//
// returns	(signed long long) converted result
//
// note		Numeric values that exceed the maximums for this data type
//		will result in undefined values being returned.
//-----------------------------------------------------------------------------
signed long long
str_expr_to_sll (
    const char *	arg_str
    ,
    char * *		arg_end_str
    ,
    int			arg_base
    )
;

//-----------------------------------------------------------------------------
// function	str_expr_to_ui
//
// purpose	Convert a string containing a numeric expression into an
//		unsigned int.
//
// arguments	1 (const char *) pointer to string to convert
//		2 (char * *) pointer to string to store end pointer
//		3 (int) numeric base value, 2 to 36
//
// returns	(unsigned int) converted result
//
// note		Numeric values that exceed the maximums for this data type
//		will result in undefined values being returned.
//-----------------------------------------------------------------------------
unsigned int
str_expr_to_ui (
    const char *	arg_str
    ,
    char * *		arg_end_str
    ,
    int			arg_base
    )
;

//-----------------------------------------------------------------------------
// function	str_expr_to_ul
//
// purpose	Convert a string containing a numeric expression into an
//		unsigned long integer.
//
// arguments	1 (const char *) pointer to string to convert
//		2 (char * *) pointer to string to store end pointer
//		3 (int) numeric base value, 2 to 36
//
// returns	(unsigned long) converted result
//
// note		Numeric values that exceed the maximums for this data type
//		will result in undefined values being returned.
//-----------------------------------------------------------------------------
unsigned long
str_expr_to_ul (
    const char *	arg_str
    ,
    char * *		arg_end_str
    ,
    int			arg_base
    )
;

//-----------------------------------------------------------------------------
// function	str_expr_to_ull
//
// purpose	Convert a string containing a numeric expression into an
//		unsigned long long integer.
//
// arguments	1 (const char *) pointer to string to convert
//		2 (char * *) pointer to string to store end pointer
//		3 (int) numeric base value, 2 to 36
//
// returns	(unsigned long long) converted result
//
// note		Numeric values that exceed the maximums for this data type
//		will result in undefined values being returned.
//-----------------------------------------------------------------------------
unsigned long long
str_expr_to_ull (
    const char *	arg_str
    ,
    char * *		arg_end_str
    ,
    int			arg_base
    )
;

//-----------------------------------------------------------------------------
// function	(str_find_arg)
//
// purpose      Search arguments 2..N for a string that matches argument 1,
//		returning 0 for not found, and 1..N-1 for which argument is
//		found.  The comparison IS case sensitive.
//
// arguments    1 (const char *) string to search for
//              2..N (const char *) strings to search
//		N+1 (const char *) NULL to indicate end of the list.
//
// returns	(int) == -1 : error
//		(int) ==  0 : not found
//		(int>  >  0 : found string at argument N-1
//-----------------------------------------------------------------------------
size_t
(str_find_arg) (
    const char *	arg_string
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(str_find_nocase_arg)
//
// purpose      Search arguments 2..N for a string that matches argument 1,
//              returning 0 for not found, and 1..N-1 for which argument is
//              found.  The comparison is NOT case sensitive.
//
// arguments    1 (const char *) string to search for
//              2..N (const char *) strings to search
//		N+1 (const char *) NULL to indicate end of list
//
// returns      (int) == -1 : error
//              (int) ==  0 : not found
//              (int>  >  0 : found string at argument N-1
//-----------------------------------------------------------------------------
size_t
(str_find_nocase_arg) (
    const char *	arg_string
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function     str_find_nocase_var
//
// purpose	Search the given variable arguments list for a string that
//		matches argument 1, returning 0 for not found, and 1..N-1 for
//		which argument is found.  The comparison is NOT case sensitive.
//
// arguments    1 (const char *) string to search for
//		2 (va_list) string variable arguments list to search in
//
// returns      (int) == -1 : error
//              (int) ==  0 : not found
//              (int>  >  0 : found string at argument N-1
//-----------------------------------------------------------------------------
size_t
str_find_nocase_var (
    const char *	arg_string
    ,
    va_list		arg_list
    )
;

//-----------------------------------------------------------------------------
// function     str_find_var
//
// purpose      Search the given variable arguments list for a string that
//              matches argument 1, returning 0 for not found, and 1..N-1 for
//              which argument is found.  The comparison IS case sensitive.
//
// arguments    1 (const char *) string to search for
//              2 (va_list) string variable arguments list to search in
//
// returns      (int) == -1 : error
//              (int) ==  0 : not found
//              (int>  >  0 : found string at argument N-1
//-----------------------------------------------------------------------------
size_t
str_find_var (
    const char *	arg_string
    ,
    va_list		arg_list
    )
;

//-----------------------------------------------------------------------------
// function	str_lower
//
// purpose	Convert each character in a given string to its lower case
//		equivalent within the same string.
//
// arguments	1 (char *) pointer to target string to convert
//
// returns	(size_t) length of string, or ~0 for error
//-----------------------------------------------------------------------------
size_t
str_lower (
    char *		arg_target
    )
;

//-----------------------------------------------------------------------------
// function	str_mem_cmp
// alias	str_mem_compare
//
// purpose	Compare two strings, the first specified by pointer and is a
//		zero terminated string, the second specified by pointer and
//		length, returning the collation difference as -1, 0, or 1.
//		If one string ends before a difference is found, then the
//		shorter string is considered to be lower in sequence than the
//		longer string.
//
// arguments	1 (const char *) pointer to string one (zero terminated)
//		2 (const char *) pointer to string two
//		3 (size_t) length of string two
//
// returns	(int) -1 : string one  < string two
//		(int)  0 : string one == string two
//		(int)  1 : string one  > string two
//-----------------------------------------------------------------------------
#define str_mem_compare str_mem_cmp
int
str_mem_cmp (
    const char *	arg_one_ptr
    ,
    const char *	arg_two_ptr
    ,
    size_t		arg_two_len
    )
;

//-----------------------------------------------------------------------------
// function	str_mem_cmp_lower
// alias	str_mem_compare_lower
//
// purpose	Compare the lower case equivalents of two strings, the first
//		specified by pointer and is a zero terminated string, the
//		second specified by pointer and length, returning the collation
//		difference as -1, 0, or 1.  If one string ends before a
//		difference is found, then the shorter string is considered to
//		be lower in sequence than the longer string.
//
// arguments	1 (const char *) pointer to string one (zero terminated)
//		2 (const char *) pointer to string two
//		3 (size_t) length of string two
//
// returns	(int) -1 : string one  < string two
//		(int)  0 : string one == string two
//		(int)  1 : string one  > string two
//-----------------------------------------------------------------------------
#define str_mem_compare_lower str_mem_cmp_lower
int
str_mem_cmp_lower (
    const char *	arg_one_ptr
    ,
    const char *	arg_two_ptr
    ,
    size_t		arg_two_len
    )
;

//-----------------------------------------------------------------------------
// function	str_mem_cmp_upper
// alias	str_mem_compare_upper
//
// purpose	Compare the upper case equivalents of two strings, the first
//		specified by pointer and is a zero terminated string, the
//		second specified by pointer and length, returning the collation
//		difference as -1, 0, or 1.  If one string ends before a
//		difference is found, then the shorter string is considered to
//		be upper in sequence than the longer string.
//
// arguments	1 (const char *) pointer to string one (zero terminated)
//		2 (const char *) pointer to string two
//		3 (size_t) length of string two
//
// returns	(int) -1 : string one  < string two
//		(int)  0 : string one == string two
//		(int)  1 : string one  > string two
//-----------------------------------------------------------------------------
#define str_mem_compare_upper str_mem_cmp_upper
int
str_mem_cmp_upper (
    const char *	arg_one_ptr
    ,
    const char *	arg_two_ptr
    ,
    size_t		arg_two_len
    )
;

//-----------------------------------------------------------------------------
// function	str_metric_to_d
//
// purpose	Convert a string containing a number followed by an optional
//		metric suffix into a double.
//
// arguments	1 (const char *) pointer to string to convert
//		2 (char * *) pointer to string to store end pointer
//		3 (long) metric scale for lower case suffixes, usually 1000
//		4 (long) metric scale for upper case suffixes, usually 1024
//
// returns	(double) converted result
//
// note		Numeric values that exceed the maximums for this data type
//		will result in undefined values being returned.
//
// suffixes	'k' or 'K' multiplies the preceeding number by scale**1
//		'm' or 'M' multiplies the preceeding number by scale**2
//		'g' or 'G' multiplies the preceeding number by scale**3
//		't' or 'T' multiplies the preceeding number by scale**4
//		'p' or 'P' multiplies the preceeding number by scale**5
//-----------------------------------------------------------------------------
double
str_metric_to_d (
    const char *	arg_str
    ,
    char * *		arg_end_ptr
    ,
    double		arg_scale_lower
    ,
    double		arg_scale_upper
    )
;

//-----------------------------------------------------------------------------
// function	str_metric_to_ld
//
// purpose	Convert a string containing a number followed by an optional
//		metric suffix into a long double.
//
// arguments	1 (const char *) pointer to string to convert
//		2 (char * *) pointer to string to store end pointer
//		3 (long) metric scale for lower case suffixes, usually 1000
//		4 (long) metric scale for upper case suffixes, usually 1024
//
// returns	(long double) converted result
//
// note		Numeric values that exceed the maximums for this data type
//		will result in undefined values being returned.
//
// suffixes	'k' or 'K' multiplies the preceeding number by scale**1
//		'm' or 'M' multiplies the preceeding number by scale**2
//		'g' or 'G' multiplies the preceeding number by scale**3
//		't' or 'T' multiplies the preceeding number by scale**4
//		'p' or 'P' multiplies the preceeding number by scale**5
//-----------------------------------------------------------------------------
long double
str_metric_to_ld (
    const char *	arg_str
    ,
    char * *		arg_end_ptr
    ,
    long double		arg_scale_lower
    ,
    long double		arg_scale_upper
    )
;

//-----------------------------------------------------------------------------
// function	str_metric_to_sl
//
// purpose	Convert a string containing a number followed by an optional
//		metric suffix into an signed long integer.
//
// arguments	1 (const char *) pointer to string to convert
//		2 (char * *) pointer to string to store end pointer
//		3 (int) numeric base value, 2 to 36
//		4 (signed long) metric scale for lower case suffixes
//		5 (signed long) metric scale for upper case suffixes
//
// returns	(signed long) converted result
//
// note		Numeric values that exceed the maximums for this data type
//		will result in undefined values being returned.
//
// suffixes	'k' or 'K' multiplies the preceeding number by scale**1
//		'm' or 'M' multiplies the preceeding number by scale**2
//		'g' or 'G' multiplies the preceeding number by scale**3
//		't' or 'T' multiplies the preceeding number by scale**4
//		'p' or 'P' multiplies the preceeding number by scale**5
//		'e' or 'E' multiplies the preceeding number by scale**6
//
// warning	Metric suffixes are not recognized if the base setting
//		uses that character for its symbol set.
//-----------------------------------------------------------------------------
signed long
str_metric_to_sl (
    const char *	arg_str
    ,
    char * *		arg_end_ptr
    ,
    int			arg_base
    ,
    signed long		arg_scale_lower
    ,
    signed long		arg_scale_upper
    )
;

//-----------------------------------------------------------------------------
// function	str_metric_to_sll
//
// purpose	Convert a string containing a number followed by an optional
//		metric suffix into an signed long long integer.
//
// arguments	1 (const char *) pointer to string to convert
//		2 (char * *) pointer to string to store end pointer
//		3 (int) numeric base value, 2 to 36
//		4 (signed long long) metric scale for lower case suffixes
//		5 (signed long long) metric scale for upper case suffixes
//
// returns	(signed long long) converted result
//
// note		Numeric values that exceed the maximums for this data type
//		will result in undefined values being returned.
//
// suffixes	'k' or 'K' multiplies the preceeding number by scale**1
//		'm' or 'M' multiplies the preceeding number by scale**2
//		'g' or 'G' multiplies the preceeding number by scale**3
//		't' or 'T' multiplies the preceeding number by scale**4
//		'p' or 'P' multiplies the preceeding number by scale**5
//		'e' or 'E' multiplies the preceeding number by scale**6
//
// warning	Metric suffixes are not recognized if the base setting
//		uses that character for its symbol set.
//-----------------------------------------------------------------------------
signed long long
str_metric_to_sll (
    const char *	arg_str
    ,
    char * *		arg_end_ptr
    ,
    int			arg_base
    ,
    signed long long	arg_scale_lower
    ,
    signed long long	arg_scale_upper
    )
;

//-----------------------------------------------------------------------------
// function	str_metric_to_ul
//
// purpose	Convert a string containing a number followed by an optional
//		metric suffix into an unsigned long integer.
//
// arguments	1 (const char *) pointer to string to convert
//		2 (char * *) pointer to string to store end pointer
//		3 (int) numeric base value, 2 to 36
//		4 (unsigned long) metric scale for lower case suffixes
//		5 (unsigned long) metric scale for upper case suffixes
//
// returns	(unsigned long) converted result
//
// note		Numeric values that exceed the maximums for this data type
//		will result in undefined values being returned.
//
// suffixes	'k' or 'K' multiplies the preceeding number by scale**1
//		'm' or 'M' multiplies the preceeding number by scale**2
//		'g' or 'G' multiplies the preceeding number by scale**3
//		't' or 'T' multiplies the preceeding number by scale**4
//		'p' or 'P' multiplies the preceeding number by scale**5
//		'e' or 'E' multiplies the preceeding number by scale**6
//
// warning	Metric suffixes are not recognized if the base setting
//		uses that character for its symbol set.
//-----------------------------------------------------------------------------
unsigned long
str_metric_to_ul (
    const char *	arg_str
    ,
    char * *		arg_end_ptr
    ,
    int			arg_base
    ,
    unsigned long	arg_scale_lower
    ,
    unsigned long	arg_scale_upper
    )
;

//-----------------------------------------------------------------------------
// function	str_metric_to_ull
//
// purpose	Convert a string containing a number followed by an optional
//		metric suffix into an unsigned long long integer.
//
// arguments	1 (const char *) pointer to string to convert
//		2 (char * *) pointer to string to store end pointer
//		3 (int) numeric base value, 2 to 36
//		4 (unsigned long long) metric scale for lower case suffixes
//		5 (unsigned long long) metric scale for upper case suffixes
//
// returns	(unsigned long long) converted result
//
// note		Numeric values that exceed the maximums for this data type
//		will result in undefined values being returned.
//
// suffixes	'k' or 'K' multiplies the preceeding number by scale**1
//		'm' or 'M' multiplies the preceeding number by scale**2
//		'g' or 'G' multiplies the preceeding number by scale**3
//		't' or 'T' multiplies the preceeding number by scale**4
//		'p' or 'P' multiplies the preceeding number by scale**5
//		'e' or 'E' multiplies the preceeding number by scale**6
//
// warning	Metric suffixes are not recognized if the base setting
//		uses that character for its symbol set.
//-----------------------------------------------------------------------------
unsigned long long
str_metric_to_ull (
    const char *	arg_str
    ,
    char * *		arg_end_ptr
    ,
    int			arg_base
    ,
    unsigned long long	arg_scale_lower
    ,
    unsigned long long	arg_scale_upper
    )
;

//-----------------------------------------------------------------------------
// function	str_select_app
//
// arguments	1 (char *) location to append selected substring to
//		2 (size_t) total size of location to append to
//		3 (const char *) string to select a substring from
//		4 (const char *) selection command string
//		... (various) arguments used by command string
//
// returns	(size_t) selected substring length (not truncated)
//-----------------------------------------------------------------------------
size_t
str_select_app (
    char *		arg_dest_ptr
    ,
    size_t		arg_dest_len
    ,
    const char *	arg_string
    ,
    const char *	arg_control
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	str_select_app_va
//
// arguments	1 (char *) location to append selected substring to
//		2 (size_t) total size of location to append to
//		3 (const char *) string to select a substring from
//		4 (const char *) selection command string
//		5 (va_list) arguments used by command string
//
// returns	(size_t) selected substring length (not considering truncation)
//-----------------------------------------------------------------------------
size_t
str_select_app_va (
    char *		arg_dest_ptr
    ,
    size_t		arg_dest_len
    ,
    const char *	arg_string
    ,
    const char *	arg_control
    ,
    va_list		arg_var_args
    )
;

//-----------------------------------------------------------------------------
// function	str_select_cpy
//
// arguments	1 (char *) location to copy selected substring to
//		2 (size_t) size of location to copy to
//		3 (const char *) string to select a substring from
//		4 (const char *) selection command string
//		... (various) arguments used by command string
//
// returns	(size_t) selected substring length (not truncated)
//-----------------------------------------------------------------------------
size_t
str_select_cpy (
    char *		arg_dest_ptr
    ,
    size_t		arg_dest_len
    ,
    const char *	arg_string
    ,
    const char *	arg_control
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	str_select_cpy_va
//
// arguments	1 (char *) location to copy selected substring to
//		2 (size_t) size of location to copy to
//		3 (const char *) string to select a substring from
//		4 (const char *) selection command string
//		5 (va_list) arguments used by command string
//
// returns	(size_t) selected substring length (not truncated)
//-----------------------------------------------------------------------------
size_t
str_select_cpy_va (
    char *		arg_dest_ptr
    ,
    size_t		arg_dest_len
    ,
    const char *	arg_string
    ,
    const char *	arg_control
    ,
    va_list		arg_var_args
    )
;

//-----------------------------------------------------------------------------
// function	str_select_cut
//
// arguments	1 (char *) string to select a substring from
//		2 (char *) selection command string
//		... (various) arguments used by command string
//
// returns	(char *) pointer to substring within original string
//-----------------------------------------------------------------------------
char *
str_select_cut (
    char *		arg_string
    ,
    char *		arg_control
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	str_select_cut_va
//
// arguments	1 (char *) string to select a substring from
//		2 (char *) selection command string
//		3 (va_list) arguments used by command string
//
// returns	(char *) pointer to substring within original string
//-----------------------------------------------------------------------------
char *
str_select_cut_va (
    char *		arg_string
    ,
    char *		arg_control
    ,
    va_list		arg_var_args
    )
;

//-----------------------------------------------------------------------------
// function	str_select_dup
//
// arguments	1 (const char *) string to select a substring from
//		2 (const char *) selection command string
//		... (various) arguments used by command string
//
// returns	(char *) pointer to allocated space with substring
//-----------------------------------------------------------------------------
char *
str_select_dup (
    const char *	arg_string
    ,
    const char *	arg_control
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	str_select_dup_va
//
// arguments	1 (const char *) string to select a substring from
//		2 (const char *) selection command string
//		3 (va_list) arguments used by command string
//
// returns	(char *) pointer to allocated space with substring
//-----------------------------------------------------------------------------
char *
str_select_dup_va (
    const char *	arg_string
    ,
    const char *	arg_control
    ,
    va_list		arg_var_args
    )
;

//-----------------------------------------------------------------------------
// function	str_select_ptr
//
// arguments	1 (char * *) where to store pointer to selected substring
//		2 (char * *) where to store end pointer to
//		3 (const char *) string to select a substring from
//		4 (const char *) selection command string
//		... (various) arguments used by command string
//
// returns	(size_t) selected substring length
//-----------------------------------------------------------------------------
size_t
str_select_ptr (
    char * *		arg_ss_ptr_p
    ,
    char * *		arg_ss_end_p
    ,
    const char *	arg_string
    ,
    const char *	arg_control
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	str_select_ptr_va
//
// arguments	1 (char * *) where to store pointer to selected substring
//		2 (char * *) where to store end pointer to
//		3 (const char *) string to select a substring from
//		4 (const char *) selection command string
//		5 (va_list) already started arguments used by command string
//
// returns	(size_t) selected substring length
//		(size_t) ~0 if error
//-----------------------------------------------------------------------------
size_t
str_select_ptr_va (
    char * *		arg_ss_ptr_p
    ,
    char * *		arg_ss_end_p
    ,
    const char *	arg_string
    ,
    const char *	arg_control
    ,
    va_list		arg_va_list
    )
;

//-----------------------------------------------------------------------------
// function	str_split_url
//
// purpose	Split a string with a URL into multiple strings, storing
//		pointers to each component in the original buffer into
//		locations in the argument list as pointers.
//
// arguments	1 (char *) string and buffer space to split.
//		2 (char * *) store pointer to protocol here
//		3 (char * *) store pointer to user name here
//		4 (char * *) store pointer to password here
//		5 (char * *) store pointer to host name here
//		6 (char * *) store pointer to port number here
//		7 (char * *) store pointer to full path here
//		8 (char * *) store pointer to query string here
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// example	protocol://username:password@hostname:port/path?querystring
//
// note		The components are split within the given buffer.  If the
//		caller needs the original buffer, a duplicate should be made
//		before calling str_split_url().  If the buffer was allocated
//		with malloc() or similar, the caller can only free the space
//		using the original pointer at which time all the components
//		are freed.
//
// note		This splitter/parser does not handle malformed URLs well.
//		Some bad URLs may be handled as if good.
//-----------------------------------------------------------------------------
int
str_split_url (
    char *		arg_string
    ,
    char * *		arg_prot_p
    ,
    char * *		arg_user_p
    ,
    char * *		arg_pass_p
    ,
    char * *		arg_host_p
    ,
    char * *		arg_port_p
    ,
    char * *		arg_path_p
    ,
    char * *		arg_qstr_p
    )
;

//-----------------------------------------------------------------------------
// function	str_to_d
//
// purpose	Convert an external representation of a double number to
//		an internal representation.
//
//		This implementation focuses on portability across internal
//		representations and accuracy, rather than speed.  If speed
//		is desired, it should come from an implementation custom
//		made for the specific platform and float format, probably
//		in assembly language.
//
// arguments	1 (const char *) string to decode
//		2 (char * *) where to store end pointer
//
// returns	(double) 0.0 and errno set : error
//		(double) converted value
//
// issues
//
// If the base digits given, disregarding the fraction point, represents a
// a number which overflows (not just exceeds precision of) the internal
// format then an undetected conversion error is likely.  This will generally
// require quite a number of digits, although as few as 38 in minimal but
// strictly conforming implementations.
//
// Suffixes of 'f', 'l', 'F', or 'L' are permitted, although the number is
// not actually rounded to the apparently expressed type precision under
// the premise that an external representation does not imply specific C
// implementation limits, and that double is preferred by the caller
// of this function.  The effect of the suffix is to move the end pointer
// on to the next character.
//-----------------------------------------------------------------------------
double
str_to_d (
    const char *	arg_string
    ,
    char * *		arg_endptr
    )
;

//-----------------------------------------------------------------------------
// function	str_to_ld
//
// purpose	Convert an external representation of a long double number to
//		an internal representation.
//
//		This implementation focuses on portability across internal
//		representations and correctness, rather than precision and
//		speed.  If speed is desired, then an implementation custom
//		made for the specific platform and float format, probably
//		in assembly language, should be used.
//
// arguments	1 (const char *) string to decode
//		2 (char * *) where to store end pointer
//
// returns	(long double) 0.0 and errno set : error
//		(long double) converted value
//
// issues
//
// If the base digits given, disregarding the fraction point, represents a
// a number which overflows (not just exceeds precision of) the internal
// format then an undetected conversion error is likely.  This will generally
// require quite a number of digits, although as few as 38 in minimal but
// strictly conforming implementations.
//
// Suffixes of 'f', 'l', 'F', or 'L' are permitted, although the number is
// not actually rounded to the apparently expressed type precision under
// the premise that an external representation does not imply specific C
// implementation limits, and that long double is preferred by the caller
// of this function.  The effect of the suffix is to move the end pointer
// on to the next character.
//
// This function should be depricated when a corrected version of strtold
// is widely distributed across most systems and platforms.
//-----------------------------------------------------------------------------
long double
str_to_ld (
    const char *	arg_string
    ,
    char * *		arg_endptr
    )
;

//-----------------------------------------------------------------------------
// function	str_truncate
//
// purpose	Truncate a null terminated string to a specified length if it
//		is longer than that length.
//
// arguments	1 (char *) pointer to string to truncate
//		2 (size_t) length to truncate string to
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		If the string is shorter that the requested length, then the
//		string remains the same length.
//-----------------------------------------------------------------------------
size_t
str_truncate (
    char *		arg_target
    ,
    size_t		arg_length
    )
;

//-----------------------------------------------------------------------------
// function	str_upper
//
// purpose	Convert each character in a given string to its upper case
//		equivalent within the same string.
//
// arguments	1 (char *) pointer to target string to convert
//
// returns	(size_t) length of string, or ~0 for error
//-----------------------------------------------------------------------------
size_t
str_upper (
    char *		arg_target
    )
;

//-----------------------------------------------------------------------------
// function	str_yes_no
//
// purpose	Determine if a given string represents a yes/true or no/false
//		boolean value in several forms and languages.
//
// argument	1 (const char *) string to determine value from
//
// returns	(int) -1 : unknown
//		(int)  0 : ei/false/na/nao/ne/nee/nei/nein/nem/nie/no/non/nyet
//		(int)  1 : ano/da/do/igem/ja/jaa/jes/kylla/oui/si/sim/tak/true/yes
//-----------------------------------------------------------------------------
#define str_true_false	str_yes_no
#define str_ano_ne	str_yes_no
#define str_ja_nej	str_yes_no
#define str_ja_nee	str_yes_no
#define str_jaa_ei	str_yes_no
#define str_jes_ne	str_yes_no
#define str_kylla_ei	str_yes_no
#define str_oui_non	str_yes_no
#define str_ja_nein	str_yes_no
#define str_igem_nem	str_yes_no
#define str_si_no	str_yes_no
#define str_ja_nei	str_yes_no
#define str_tak_nie	str_yes_no
#define str_sim_nao	str_yes_no
#define str_da_nyet	str_yes_no
#define str_da_ne	str_yes_no
#define str_do_na	str_yes_no

int
str_yes_no (
    const char *	arg_string
    )
;

#endif /* __STRING_H__ */
